package com.example.tdl.common;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.util.CellRangeAddress;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * 导出工具和备件Excel公共方法
 */
public class CommonExcel {
    private String path;
    //显示的导出表的标题
    private String name;
    private String title;
    //导出表的列名
    private String[] rowName;
    //导出的文件名
    private String fileName;
    private List<Object[]> dataList = new ArrayList<Object[]>();

    //构造方法，传入要导出的数据
    public CommonExcel(String name, String title, String[] rowName,List<Object[]> dataList, String fileName) {
        this.name=name;
        this.dataList = dataList;
        this.rowName = rowName;
        this.title = title;
        this.fileName = fileName;
    }

    /*
     * 导出数据
     */
    public HSSFWorkbook downloadExcel() {
        HSSFWorkbook workbook = new HSSFWorkbook();                     // 创建工作簿对象
        HSSFSheet sheet = workbook.createSheet(name);                  // 创建工作表
        HSSFCellStyle style = this.getStyle(workbook);
        try{
            /*
            * 创建第一行（表格标题行）
            * */
            HSSFRow rowm = sheet.createRow((short)0);
            HSSFCell cellTitle=null;
            //设置单元格格式
            HSSFCellStyle titleStyle = this.getTitleStyle(workbook);
            //循环创建列
            for(int n=0;n<rowName.length;n++){
                cellTitle= rowm.createCell(n);
                //设置列的格式
                cellTitle.setCellStyle(titleStyle);
                //将标题添加到列中
                cellTitle.setCellValue(new HSSFRichTextString(title));
            }
            //设置行高
            rowm.setHeight((short)(30*20));
            //合并单元格（参数1：起始行，参数2：截至行，参数3：起始列，参数4：截至列）
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, (rowName.length-1)));

            /*
            * 创建第二行（中文列名）
            * */
            // 定义所需列数
            int columnNum = rowName.length;
            // 在索引2的位置创建行(最顶端的行开始的第二行)
            HSSFRow rowRowName = sheet.createRow(1);
            rowRowName.setHeight((short)(25*20));
            //获取列头样式对象
            HSSFCellStyle columnTopStyle = this.getColumnTopStyle(workbook);
            for(int n=0;n<columnNum;n++){
                //创建列头对应个数的单元格
                HSSFCell cellRowName = rowRowName.createCell(n);
                //设置列头单元格的数据类型
                cellRowName.setCellType(HSSFCell.CELL_TYPE_STRING);
                //设置列头单元格的值
                HSSFRichTextString text = new HSSFRichTextString(rowName[n]);
                cellRowName.setCellValue(text);
                //设置列头单元格样式
                cellRowName.setCellStyle(columnTopStyle);
            }
            sheet.setColumnWidth(0, 35*256);
            sheet.setColumnWidth(1, 18*256);
            sheet.setColumnWidth(2, 18*256);
            sheet.setColumnWidth(3, 13*256);
            sheet.setColumnWidth(4, 15*256);
            sheet.setColumnWidth(5, 13*256);
            sheet.setColumnWidth(6, 13*256);

            /*
            * 循环将查询出的数据设置到sheet对应的单元格中
            * */
            for(int i=0;i<dataList.size();i++){
                //便利集合传过来的值
                Object[] obj = dataList.get(i);
                //从第五行开始创建
                HSSFRow row = sheet.createRow(i+2);
                row.setHeight((short)(30*20));
                for(int j=0; j<obj.length; j++){
                    HSSFCell  cell = null;
                    cell = row.createCell(j,HSSFCell.CELL_TYPE_STRING);
                    if(!"".equals(obj[j]) && obj[j] != null){
                        cell.setCellValue(obj[j].toString());                       //设置单元格的值
                    }
                    cell.setCellStyle(style);                                   //设置单元格样式
                }
            }
            return workbook;
        } catch (Exception e) {
            e.printStackTrace();
        }

//        if(workbook !=null){
//            try {
//                if (response != null){
//                    response.reset();
//                    response.setContentType("application/vnd.ms-excel;charset=utf-8");
//                    response.addHeader("Content-Disposition", "attachment;filename="+ fileName);
////                    response.setHeader("Content-Disposition", "attachment;filename="+new String(fileName.getBytes("gb2312"),"ISO8859-1"));
//                    response.setHeader("Pragma", "no-cache");
//                    response.setHeader("Cache-Control", "no-cache");
//                    response.setDateHeader("Expires", 0);
//                    OutputStream out = response.getOutputStream();
//                    BufferedOutputStream bufferedOutPut = new BufferedOutputStream(out);
//                    workbook.write(bufferedOutPut);
//                    bufferedOutPut.flush();
//                    bufferedOutPut.close();
//                    return workbook;
//                } else {
//                    FileOutputStream outputStream = new FileOutputStream(path+fileName);
//                    workbook.write(outputStream);
//                    outputStream.close();
//                    return workbook;
//                }
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        }
        return workbook;
    }

    /*
     * 中文标题单元格样式
     */
    public HSSFCellStyle getTitleStyle(HSSFWorkbook workbook) {
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short) 18);
        //字体加粗
        font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        //设置字体名字
        font.setFontName("等线");
        //设置样式;
        HSSFCellStyle style = workbook.createCellStyle();
        //设置底边框;
//        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
//        //设置底边框颜色;
//        style.setBottomBorderColor(HSSFColor.WHITE.index);
//        //设置左边框;
//        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
//        //设置左边框颜色;
//        style.setLeftBorderColor(HSSFColor.WHITE.index);
//        //设置右边框;
//        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
//        //设置右边框颜色;
//        style.setRightBorderColor(HSSFColor.WHITE.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.WHITE.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;

    }


    /*
     * 中文列头单元格样式
     */
    public HSSFCellStyle getColumnTopStyle(HSSFWorkbook workbook) {

        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short) 14);
        //字体加粗
//        font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        //设置字体名字
        font.setFontName("微软雅黑");
        //设置样式;
        HSSFCellStyle style = workbook.createCellStyle();
        //设置底边框;
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        //设置底边框颜色;
        style.setBottomBorderColor(HSSFColor.WHITE.index);
        //设置左边框;
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        //设置左边框颜色;
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        //设置右边框;
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        //设置右边框颜色;
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);

        return style;

    }


    /*
     * 列数据信息单元格样式
     */
    public HSSFCellStyle getStyle(HSSFWorkbook workbook) {
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体名字
        font.setFontName("微软雅黑");
        //设置样式;
        HSSFCellStyle style = workbook.createCellStyle();
        //设置底边框;
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        //设置底边框颜色;
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        //设置左边框;
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        //设置左边框颜色;
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        //设置右边框;
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        //设置右边框颜色;
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }
}