package com.example.tdl.config;


import com.aliyun.oss.OSSClient;
import com.example.tdl.common.WarehouseExcel;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.TDLLogService;
import com.example.tdl.service.UserService;
import com.example.tdl.service.WarehouseAlarmLogService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.util.*;
import com.example.tdl.web.I18nController;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ScheduleConfig {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private WarehouseAlarmLogService warehouseAlarmLogService;

    @Autowired
    private I18nController i18n;

    @Autowired
    private  HttpServletRequest request;


    @Autowired
    private InfluxDBTemplate influxDBTemplate;


    @Value("${spring.influxdb.database}")
    private String database ;

    List<Object> head = new ArrayList<Object>(){{
        add("时间");
        add("压力(MPa)");
        add("温度(℃)");
        add("湿度(%RH)");
        add("是否预警");
    }};

    @Scheduled(cron ="0 0 0/12 * * ?") //每隔一小时执行一次
    public void scheduler() {
        Long endTime  = System.currentTimeMillis();
        Long startTime = endTime-12*60*60*1000l;
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //获取所有的仓库信息
        List<ResultWarehouseVo> resultWarehouseList = warehouseService.getAllWarehouse();
        for(int i =0,length =  resultWarehouseList.size();i<length;i++){
            //获取相关的报警联系人
            List<String> emailList = warehouseService.getEmailByWarehouse(resultWarehouseList.get(i).getWarehouseNo());
            if(emailList.size() == 0){
                continue;
            }
            //查询报警情况
            List<WarehouseCountVo> tempCount =   warehouseAlarmLogService.getCount(resultWarehouseList.get(i).getWarehouseNo(),startTime,endTime,"Temperature");
            List<WarehouseCountVo> humCount =   warehouseAlarmLogService.getCount(resultWarehouseList.get(i).getWarehouseNo(),startTime,endTime,"humity");
            if(tempCount.size() == 0 && humCount.size() == 0){
                continue;
            }
            try {
                MailUtil.sendMail(emailList, "辉度物流监测系统------仓库报警提醒",
                        message(resultWarehouseList.get(i).getWarehouseNo(),sdf.format(new Date(Long.parseLong(String.valueOf(startTime)))),
                                sdf.format(new Date(Long.parseLong(String.valueOf(endTime)))),
                                getSumCount(tempCount),
                                getMaxTime(tempCount),
                                getSumCount(humCount),
                                getMaxTime(humCount)));
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }


    @Scheduled(cron ="0 0 0 1 * ? ")
//    @Scheduled(cron ="0/15 * * * * ? ")
    public void scheduler2(){
        //获取每个仓库的历史数据并生成报表发给每个人
        List<ResultWarehouseVo> resultWarehouseList = warehouseService.getAllWarehouse();
        for(int i =0,length =  resultWarehouseList.size();i<length;i++){
            //获取相关的报警联系人
            List<String> emailList = warehouseService.getEmailByWarehouse(resultWarehouseList.get(i).getWarehouseNo());
            if(emailList.size() == 0){
                continue;
            }
//            sendEmail(emailList,resultWarehouseList.get(i).getWarehouseNo(),resultWarehouseList.get(i).getWarehouseName());
            sendEmail(new ArrayList<String>(){{add("13584678050@163.com");}},resultWarehouseList.get(i).getWarehouseNo(),resultWarehouseList.get(i).getWarehouseName(), request);
        }
    }

    public String message(String  warehouseNo,String startTime,String endTime,Integer tempCount,String tempTime,Integer humCount,String humTime){
        String html="<!DOCTYPE html>\n" +
                "<html lang=\"en\">\n" +
                "<head>\n" +
                "    <meta charset=\"UTF-8\">\n" +
                "    <title>Title</title>\n" +
                "</head>\n" +
                "<body style=\"align-content: center;\">\n" +
                "    <div style=\"width: 800px;margin: 0px auto;\">\n" +
                "        <div style=\"font-size: 15px;\">\n" +
                "            <p>尊敬的用户，您好：</p>\n" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;自"+startTime+"到"+endTime+",您所管理的仓库编号为"+warehouseNo+"报警情况如下：</p>" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;发生温度报警"+tempCount+"次,主要集中在"+tempTime+";" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;发生湿度报警"+humCount+"次,主要集中在"+humTime+"</p>\n" +
                "            <br>\n" +
                "            <p>致好！</p>\n" +
                "            <br>\n" +
                "        </div>\n" +
                "        <br>\n" +
                "        <div style=\"background-image: url(http://211.161.200.169:38091/test/dist/assets/images/bg.jpg) ; width: 800px;height: 125px;background-repeat: no-repeat;\">\n" +
                "            <div style=\"float: left;\">\n" +
                "                <div style=\"width:240px;height:110px;padding-top:15px;margin-left: 45px;float: left\">\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 5px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/WitiumGuan.png \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 13px;font-weight: 300;padding-top: 36px\">辉度官方网站</span>\n" +
                "                    <a href=\"http://www.witium.cn\"><img src=\"http://211.161.200.169:38091/test/dist/assets/images/guan.png\" width=\"110\" height=\"35\"  style=\"margin-top: 3px\"></a>\n" +
                "                </div>\n" +
                "                <div style=\"width:240px;height:110px;margin-top:-110px;margin-left: 310px;float: inherit;\" >\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 5px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/WitCloud.png \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 15px;font-weight: 300;padding-top: 50px\">云平台网站</span>\n" +
                "                </div>\n" +
                "                <div style=\"width:240px;height:110px;margin-top: -110px;float: right;\">\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 4px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/Witium.jpg \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 35px;font-weight: 300;padding-top: 50px\">辉度微信公众号</span>\n" +
                "                </div>\n" +
                "            </div>\n" +
                "        </div>\n" +
                "        <div style=\"width: 800px;height:40px;font-size: 14px;background-color: #2196F3;\">\n" +
                "            <span style=\"display:block;color: #f8f7ff;text-align: center;padding: 10px;\">\n" +
                "                TEL:086-21-37774020&nbsp;&nbsp;&nbsp;&nbsp;FAX:086-21-37774010&nbsp;&nbsp;&nbsp;&nbsp;Email:sales@witium.com\n" +
                "            </span>\n" +
                "        </div>\n" +
                "    </div>\n" +
                "</body>\n" +
                "</html>";

        return html;
    }


    public Integer getSumCount(List<WarehouseCountVo> counts){
        Integer num = 0;
        for(int i = 0;i<counts.size();i++){
            num = num +counts.get(i).getNum();
        }
        return  num;
    }


    public String  getMaxTime(List<WarehouseCountVo> counts){
        String time="";
        Collections.sort(counts, Comparator.comparing(WarehouseCountVo::getNum));
        time =  counts.get(0).getHours();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date d = format.parse(time);
            Calendar c = Calendar.getInstance();
            c.setTime(d);
            c.add(Calendar.HOUR_OF_DAY, 1);
            String ts = format.format(c.getTime());
            return time + "到" + ts;
        }catch (Exception e){
            e.printStackTrace();
            return  time;
        }
    }

    public  String  sendEmail(List<String> emailList,String  warehouseNo,String  warehouseName,HttpServletRequest request)  {
        try {
            Calendar c = Calendar.getInstance();
            c.add(Calendar.MONTH, -1);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss");
            String gtimelast = sdf.format(c.getTime()); //上月
            System.out.println(gtimelast);
            int lastMonthMaxDay = c.getActualMaximum(Calendar.DAY_OF_MONTH);
            System.out.println(lastMonthMaxDay);
            c.set(c.get(Calendar.YEAR), c.get(Calendar.MONTH), lastMonthMaxDay, 23, 59, 59);

            //按格式输出
            String gtime = sdf.format(c.getTime()); //上月最后一天
            SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-01  00:00:00");
            String gtime2 = sdf2.format(c.getTime()); //上月第一天
            List<ResultWarehouseNoAndTDLSNAndTimeVo> resultWarehouseNoAndTDLSNAndTimeVos=tdlLogService.getByWarehouseNo(warehouseNo);
            if (resultWarehouseNoAndTDLSNAndTimeVos.size()==0){
                return  null;
            }
            File zipFile = new File("./export/" +warehouseName+"仓库数据.zip");
            File[] srcFiles = new File[resultWarehouseNoAndTDLSNAndTimeVos.size()];
            String address = "./export/";
            for (int a=0;a<resultWarehouseNoAndTDLSNAndTimeVos.size();a++) {
                ResultWarehouseNoAndTDLSNAndTimeVo resultWarehouseNoAndTDLSNAndTimeVo=resultWarehouseNoAndTDLSNAndTimeVos.get(a);
                Long startTime;
                Long stopTime;
                if(sdf.parse(gtime2).getTime() >=resultWarehouseNoAndTDLSNAndTimeVo.getCreateTime()){
                    startTime = sdf.parse(gtime2).getTime()*1000000l;
                }else{
                    startTime = resultWarehouseNoAndTDLSNAndTimeVo.getCreateTime()*1000000l;
                }
                stopTime = sdf.parse(gtime).getTime()*1000000l;
                String sqls = "SELECT \"b\", \"T\", \"h\" FROM \"tdl_policy\".\"" + resultWarehouseNoAndTDLSNAndTimeVo.getGatewayType() + "_" + resultWarehouseNoAndTDLSNAndTimeVo.getGatewaySN() + "\" WHERE (\"tdl\" = 'TDL-" + resultWarehouseNoAndTDLSNAndTimeVo.getTDLSN() + "') and  time >= " + startTime + "  and time<= " + stopTime + " ORDER BY time ";
                QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
                if(queryResults.getResults().get(0).getSeries()==null){
                    continue;
                }
                File myPath = new File( "./export/" );
                if ( !myPath.exists()){//若此目录不存在，则创建之
                    myPath.mkdir();
                }
                String fileName = warehouseNo + "仓库温湿度观测记录-"+resultWarehouseNoAndTDLSNAndTimeVo.getTDLSN();
                List<DownloadWarehouseVo> downloadWarehouseVos = getWarehouseData(queryResults,"+8");
                List<List<Object>> values = DataUtil.getValues(queryResults);
                List<List<Object>> rows =  new ArrayList<>();
                for(List<Object> value:values){
                    List<Object> data = new ArrayList<>();
                    for (int j = 0; j < head.size(); j++) {
                        if(j ==0){
                            data.add(DataUtil.UTCToCST(String.valueOf(value.get(0)),"+8"));
                        }else if(j == (head.size() - 1)){
                            if (Float.valueOf(data.get(2).toString()) > resultWarehouseNoAndTDLSNAndTimeVos.get(a).getTemMax() ||
                                    Float.valueOf(data.get(2).toString()) < resultWarehouseNoAndTDLSNAndTimeVos.get(a).getTemMin() ||
                                    Float.valueOf(data.get(3).toString()) > resultWarehouseNoAndTDLSNAndTimeVos.get(a).getHumidityMax() ||
                                    Float.valueOf(data.get(3).toString()) < resultWarehouseNoAndTDLSNAndTimeVos.get(a).getHumidityMin()) {
                                data.add("报警");
                            } else {
                               data.add("正常");
                            }
                            ;
                        }else{
                            data.add(value.get(j) == null ? "-" :value.get(j).toString());
                        }
                    }
                    rows.add(data);
                }
                CSVUtils.createCSVFile(head,rows,address,fileName);
                srcFiles[a]=new File("./export/"+fileName+".csv");
            }
            ZipCompressUtil.zipFiles(srcFiles,zipFile);
            OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
            String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
            if (md5Key != null) {
                for(int a =0;a<srcFiles.length;a++){
                    if(srcFiles[a] == null){
                        continue;
                    } else{
                        srcFiles[a].delete();
                    }
                }
                zipFile.delete();
            }
            String  html = "<!DOCTYPE html>\n" +
                    "<html lang=\"en\">\n" +
                    "<head>\n" +
                    "    <meta charset=\"UTF-8\">\n" +
                    "    <title>Title</title>\n" +
                    "</head>\n" +
                    "<body style=\"align-content: center;\">\n" +
                    "    <div style=\"width: 800px;margin: 0px auto;\">\n" +
                    "        <div style=\"font-size: 15px;\">\n" +
                    "            <p>尊敬的用户，您好：</p>\n" +
                    "            <p>&nbsp;&nbsp;&nbsp;&nbsp; 附件是上个月的仓库报表，请查收。</p>\n" +
                    "            <br>\n" +
                    "            <p>致好！</p>\n" +
                    "            <br>\n" +
                    "        </div>\n" +
                    "        <br>\n" +
                    "</body>\n" +
                    "</html>";

            MailUtil.sendAttachment(emailList,"仓库每月数据报表","./export/" +warehouseName+"仓库数据.zip",html);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;

    }

    public List<DownloadWarehouseVo> getWarehouseData(QueryResult queryResult,String offset){
        List<List<Object>> values = DataUtil.getValues(queryResult);
        List<DownloadWarehouseVo> downloadWarehouseVos = new ArrayList<>();
        for(List<Object> value:values){
            DownloadWarehouseVo downloadWarehouseVo = new DownloadWarehouseVo();
            downloadWarehouseVo.setTime(DataUtil.UTCToCST(String.valueOf(value.get(0)),offset));
            downloadWarehouseVo.setPressure(Float.parseFloat(value.get(1).toString()));
            downloadWarehouseVo.setTemperature(Float.parseFloat(value.get(2).toString()));
            downloadWarehouseVo.setHumidity(Float.parseFloat(value.get(3).toString()));
            downloadWarehouseVos.add(downloadWarehouseVo);
        }
        return downloadWarehouseVos;
    }
}
