package com.example.tdl.config.mqtt;

import org.apache.commons.codec.binary.Base64;
import org.eclipse.paho.client.mqttv3.MqttAsyncClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;
import java.io.*;
import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.EncodedKeySpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Properties;

/**
 * @Author: lelouch
 * @Description:
 * @Date: Created in 9:29 2018/3/13
 * @Modified By:
 **/
public class MqttTemlateAsync extends MqttAsyncClient {

    public  MqttTemlateAsync(String serverURL, String clientId) throws MqttException {
        super(serverURL, clientId, new MemoryPersistence()); // set default MqttClientPersistence as MemoryPersistence
    }


    private static String getPem(String path) throws Exception{
        FileInputStream fin = new FileInputStream(path);
        BufferedReader br = new BufferedReader(new InputStreamReader(fin));
        String readLine = null;
        StringBuilder sb= new StringBuilder();
        while((readLine= br.readLine())!=null){
            if(readLine.charAt(0)=='-'){
                continue;
            }else{
                sb.append(readLine);
                sb.append('\r');
            }
        }
        fin.close();
        return sb.toString();
    }


    public static PrivateKey getPrivateKey(String path) throws Exception{
        byte[] buffer = new Base64().decode(getPem(path));

        EncodedKeySpec keySpec= new PKCS8EncodedKeySpec(buffer);
        KeyFactory keyFactory= KeyFactory.getInstance("RSA");
        return (RSAPrivateKey) keyFactory.generatePrivate(keySpec);
    }


    private static SSLSocketFactory getSSLSocketFactory(String caPath, String crtPath, String keyPath,
                                                        String password) throws Exception {
        // CA certificate is used to authenticate server
        CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
        FileInputStream caFileInput = new FileInputStream(caPath);
        X509Certificate caCert = (X509Certificate) certFactory.generateCertificate(caFileInput);
        KeyStore caKeyStore = KeyStore.getInstance("JKS");
        caKeyStore.load(null, null);
        caKeyStore.setCertificateEntry("ca-certificate", caCert);
        TrustManagerFactory tmf = TrustManagerFactory.getInstance("PKIX");
        tmf.init(caKeyStore);
        caFileInput.close();

        CertificateFactory clientFactory = CertificateFactory.getInstance("X.509");
        FileInputStream clientCrtInput = new FileInputStream(crtPath);
        X509Certificate clientCert = (X509Certificate) clientFactory.generateCertificate(clientCrtInput);

        clientCrtInput.close();
        // client key and certificates are sent to server so it can authenticate
        // us
        KeyStore clientKeyStore = KeyStore.getInstance(KeyStore.getDefaultType());
        clientKeyStore.load(null, null);
        clientKeyStore.setCertificateEntry("certificate", clientCert);
        password = (password != null) ? password : "";
        clientKeyStore.setKeyEntry("private-key", getPrivateKey(keyPath), password.toCharArray(),
                new java.security.cert.Certificate[]{clientCert});
        KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
        kmf.init(clientKeyStore, password.toCharArray());

        // finally, create SSL socket factory
        SSLContext context = SSLContext.getInstance("TLSv1.2");

        context.init(kmf.getKeyManagers(), tmf.getTrustManagers(), new SecureRandom());

        return context.getSocketFactory();
    }


    @SuppressWarnings("unused")
    private static Properties getSSLProperties() {
        Properties props = null;
        InputStream caInput = null;
        try {
            props = new Properties();
            caInput = MqttTemlateAsync.class.getClassLoader().getResourceAsStream("mqttssl.properties");
            props.load(caInput);
        } catch (FileNotFoundException e) {
            System.out.println("getSSLProperties: mqttssl.properties file not found!");
        } catch (IOException e) {
            System.out.println("getSSLProperties: other IOException!");
        } finally {
            try {
                if(null != caInput) {
                    caInput.close();
                }
            } catch (IOException e) {
                System.out.println("getSSLProperties: mqttssl.properties file stream close exception!");
            }
        }
        return props;
    }


    public static MqttConnectOptions defaultOptions(){
        String str = "witium";
        char[] bm;
        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
        mqttConnectOptions.setCleanSession(false);
        mqttConnectOptions.setUserName("test");
        bm = str.toCharArray();
        mqttConnectOptions.setPassword(bm);
        return mqttConnectOptions;
    }


    /* For raw tcp only */
    public static MqttConnectOptions setOptions(String username, String pwd){
        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
        mqttConnectOptions.setCleanSession(false);
        mqttConnectOptions.setUserName(username);
        mqttConnectOptions.setPassword(pwd.toCharArray());
        mqttConnectOptions.setAutomaticReconnect(true); // enable auto reconnect.
        return mqttConnectOptions;
    }


    public static MqttConnectOptions setSSLOptions(String username, String pwd,
                                                   String cacrt, String clientcrt, String clientkey, String clientpwd){
        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
        SSLSocketFactory sslFactory = null;
        try {
            sslFactory = getSSLSocketFactory(cacrt, clientcrt,
                    clientkey, clientpwd);
        } catch (Exception e) {
            e.printStackTrace();
        }
        mqttConnectOptions.setSocketFactory(sslFactory);

        mqttConnectOptions.setCleanSession(false);
        mqttConnectOptions.setUserName(username);
        mqttConnectOptions.setPassword(pwd.toCharArray());
        mqttConnectOptions.setAutomaticReconnect(true); // enable auto reconnect.
        return mqttConnectOptions;
    }
}
