package com.example.tdl.interceptor;

import com.example.tdl.service.redis.TokenRedisService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;

public class LoginInterceptor implements HandlerInterceptor {
    private static Logger logger = LoggerFactory.getLogger(LoginInterceptor.class);

    @Autowired
    private TokenRedisService tokenRedisService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String url = request.getRequestURI();
        if(url.contains("swagger") || url.contains("/v2/api-docs")){
            return true;
        }
        String token =  request.getHeader("Account_token");
        if(token == null){
            response.setContentType("application/json");
            response.setStatus(200);
            response.setHeader("Content-type", "text/html;charset=UTF-8");
            OutputStream ps = response.getOutputStream();
            ps.write("{\"code\":0,\"message\":\"用户未登录，请先登录\"}".getBytes("UTF-8"));
            return  false;
        }
        String data = tokenRedisService.get("TOKEN_" +token);
        if(data == null){
            response.setContentType("application/json");
            response.setStatus(200);
            response.setHeader("Content-type", "text/html;charset=UTF-8");
            OutputStream ps = response.getOutputStream();
            ps.write("{\"code\":2,\"message\":\"用户登录过期，请重新登录\"}".getBytes("UTF-8"));
            return  false;
        }
        tokenRedisService.refeshTimeOut("TOKEN_" +token,60 * 30);
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }


}