package com.example.tdl.interceptor;

import com.example.tdl.domain.vo.UserVo;
import com.example.tdl.service.PermissionService;
import com.example.tdl.service.RoleService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.List;

public class PermissionInterceptor implements HandlerInterceptor {

    private static Logger logger = LoggerFactory.getLogger(PermissionInterceptor.class);

    @Autowired
    private TokenRedisService redisService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private PermissionService permissionService;


    Gson gson = new Gson();

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String URL = request.getRequestURI();
        if(URL.contains("swagger") || URL.contains("/v2/api-docs")){
            return true;
        }
        String requestUri = request.getRequestURI();
        String contextPath = request.getContextPath();
        String url = requestUri.substring(contextPath.length());

        logger.info("url:"+url);
        //获取用户信息以及对应的权限信息
        UserVo user =  gson.fromJson(redisService.get("TOKEN_" +request.getHeader("Account_token")),UserVo.class);
       //根据用户名获取对应的权限
       List<String> permissions =  permissionService.getByRoleName(user.getRoleName(),user.getCompanyNo());
       for(int i =0,length = permissions.size();i<length;i++){
           if(permissions.get(i).contains(url)){
               return  true;
           }
       }
        response.setContentType("application/json");
        response.setStatus(200);
        response.setHeader("Content-type", "text/html;charset=UTF-8");
        OutputStream ps = response.getOutputStream();
        ps.write("{\"code\":0,\"message\":\"对不起，您没有权限进行此操作\"}".getBytes("UTF-8"));
        return false;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }
}
