package com.example.tdl.util;

import com.example.tdl.domain.vo.ResultAlarmLogABBVo;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.util.CellRangeAddress;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * 导出报警日志EXCEL模板
 */
public class AlarmLogForABBExcelUtil {

    private List<ResultAlarmLogABBVo> resultAlarmLogVos=new ArrayList<>();

    //构造方法，传入要导出的数据
    public AlarmLogForABBExcelUtil(List<ResultAlarmLogABBVo> resultAlarmLogVos) {
        this.resultAlarmLogVos = resultAlarmLogVos;
    }

    /*
     * 导出数据
     */
    public HSSFWorkbook downloadExcel() throws Exception {
        HSSFWorkbook workbook = new HSSFWorkbook();                     // 创建工作簿对象
        HSSFSheet sheet = workbook.createSheet("报警日志数据表");                  // 创建工作表
        HSSFCellStyle cellTitleStyle = getCellTitleStyle(workbook);
        HSSFCellStyle cellStyle = getCellStyle(workbook);
        HSSFCellStyle titleStyle = getTitleStyle(workbook);

        /*
        * 创建第一行(标题行)
        * */
        HSSFRow rowOne = sheet.createRow((short)0);
        HSSFCell cellOne=null;
        //循环创建列
        for(int n=0;n<6;n++){
            cellOne=rowOne.createCell(n);
            cellOne.setCellStyle(titleStyle);
            //将标题添加到列中
            cellOne.setCellValue(new HSSFRichTextString("报  警  日  志  数  据  表"));
        }
        //设置行高
        rowOne.setHeight((short)(35*20));
        //合并单元格（参数1：起始行，参数2：截至行，参数3：起始列，参数4：截至列）
        sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 6));

        /*
        * 创建第二行
        * */
        HSSFRow rowTwo = sheet.createRow((short)1);
        HSSFCell cellTwo=null;
        cellTwo=rowTwo.createCell(0);
        cellTwo.setCellValue("运输编号");
        cellTwo.setCellStyle(cellTitleStyle);
        cellTwo=rowTwo.createCell(1);
        cellTwo.setCellValue("货物编号");
        cellTwo.setCellStyle(cellTitleStyle);
        cellTwo=rowTwo.createCell(2);
        cellTwo.setCellValue("传感器序列号");
        cellTwo.setCellStyle(cellTitleStyle);
        cellTwo=rowTwo.createCell(3);
        cellTwo.setCellValue("描述");
        cellTwo.setCellStyle(cellTitleStyle);
        cellTwo=rowTwo.createCell(4);
        cellTwo.setCellValue("报警数据");
        cellTwo.setCellStyle(cellTitleStyle);
        cellTwo=rowTwo.createCell(5);
        cellTwo.setCellValue("报警时间");
        cellTwo.setCellStyle(cellTitleStyle);
        cellTwo=rowTwo.createCell(6);
        cellTwo.setCellValue("报警参数");
        cellTwo.setCellStyle(cellTitleStyle);
        rowTwo.setHeight((short)(30*20));

        /*
        * 循环创建
        * */
        int rowIndex=1;
        for (int s=0;s<resultAlarmLogVos.size();s++){
            //从第4行开始创建
            rowIndex=rowIndex+1;//2
            HSSFRow row = sheet.createRow(rowIndex);
            HSSFCell cell=null;
            cell=row.createCell(0);
            cell.setCellValue(resultAlarmLogVos.get(s).getTransportationNo());
            cell.setCellStyle(cellStyle);
            cell=row.createCell(1);
            cell.setCellValue(resultAlarmLogVos.get(s).getCargoNo());
            cell.setCellStyle(cellStyle);
            cell=row.createCell(2);
            cell.setCellValue(resultAlarmLogVos.get(s).getTDLSN());
            cell.setCellStyle(cellStyle);
            cell=row.createCell(3);
            cell.setCellValue(resultAlarmLogVos.get(s).getDescription());
            cell.setCellStyle(cellStyle);
            cell=row.createCell(4);
            cell.setCellValue(resultAlarmLogVos.get(s).getData());
            cell.setCellStyle(cellStyle);
            cell=row.createCell(5);
            SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//这个是你要转成后的时间的格式
            String sd = sdf.format(new Date(resultAlarmLogVos.get(s).getAlarmTime()));
            cell.setCellValue(sd);
            cell.setCellStyle(cellStyle);
            cell=row.createCell(6);
            cell.setCellValue(resultAlarmLogVos.get(s).getClassify());
            cell.setCellStyle(cellStyle);
            row.setHeight((short)(25*20));
        }

        sheet.setColumnWidth(0, 26 * 256);
        sheet.setColumnWidth(1, 20 * 256);
        sheet.setColumnWidth(2, 20 * 256);
        sheet.setColumnWidth(3, 30 * 256);
        sheet.setColumnWidth(4, 17 * 256);
        sheet.setColumnWidth(5, 25 * 256);
        sheet.setColumnWidth(6, 12 * 256);

        return workbook;

    }

    /*
     * 标题单元格样式
     */
    public HSSFCellStyle getTitleStyle(HSSFWorkbook workbook) {
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short) 16);
        //字体加粗
        font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        //设置字体名字
        font.setFontName("新宋体");
        //设置样式;
        HSSFCellStyle style = workbook.createCellStyle();
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    /*
     * 列数据信息单元格样式
     */
    public HSSFCellStyle getCellTitleStyle(HSSFWorkbook workbook) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((short) 12);
        font.setFontName("Arial");
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        //设置右边框颜色;
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    /*
     * 列数据信息单元格样式
     */
    public HSSFCellStyle getCellStyle(HSSFWorkbook workbook) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((short) 11);
        font.setFontName("Arial");
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        //设置右边框颜色;
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        //设置顶边框颜色;
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

}