package com.example.tdl.util;

import com.example.tdl.domain.vo.AlarmRuleVo;
import com.example.tdl.domain.vo.ResultBoschAlarmVo;
import com.example.tdl.entity.TDLDataOffset;
import com.example.tdl.service.BoschAlarmService;
import com.example.tdl.service.redis.AlarmRedisService;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class AlarmRule {
    private final static Logger logger = LoggerFactory.getLogger(AlarmRule.class);

    private static  Gson gson = new Gson();

    private static List<String> list = new ArrayList<String>(){{
        add("T");
        add("h");
        add("a1");
        add("ta");
    }};

    public static void saveAlarmRule(String topic, String TDLSN, TDLDataOffset offset,
                                     String alarmType, String companyNo,
                                     BoschAlarmService alarmService, AlarmRedisService alarmRedisService){
        ResultBoschAlarmVo alarmVo =  alarmService.getByType(alarmType,companyNo);
        for(String alias :list){
            List<AlarmRuleVo> alarmRuleVos = new ArrayList<>();
            AlarmRuleVo alarmRuleVo = new AlarmRuleVo();
            alarmRuleVo.setPriority("1");
            switch(alias) {
                case "T" :
                    if(alarmVo.getTemLower() !=null  && alarmVo.getTiltUpper() !=null){
                        alarmRuleVo.setLowerval(alarmVo.getTemLower());
                        alarmRuleVo.setUpperval(alarmVo.getTemUpper());
                        alarmRuleVo.setMaxval(alarmVo.getTemMax());
                        alarmRuleVo.setMinval(alarmVo.getTemMin());
                        alarmRuleVo.setMode("full_normal");
                    }else {
                        alarmRuleVo.setMaxval(alarmVo.getTemMax());
                        alarmRuleVo.setMinval(alarmVo.getTemMin());
                        alarmRuleVo.setMode("twoway");
                    }
                    if (offset.getTempOffset() != null) {
                        alarmRuleVo.setOffset(offset.getTempOffset());
                    } else {
                        alarmRuleVo.setOffset(0.0);
                    }
                    break;
                case "h":
                     if(alarmVo.getHumidityLower() !=null  && alarmVo.getHumidityUpper() !=null){
                        alarmRuleVo.setLowerval(alarmVo.getHumidityUpper());
                        alarmRuleVo.setUpperval(alarmVo.getHumidityLower());
                        alarmRuleVo.setMaxval(alarmVo.getHumidityMax());
                        alarmRuleVo.setMinval(alarmVo.getHumidityMin());
                        alarmRuleVo.setMode("full_normal");
                    }else {
                        alarmRuleVo.setMaxval(alarmVo.getHumidityMax());
                        alarmRuleVo.setMinval(alarmVo.getHumidityMin());
                        alarmRuleVo.setMode("twoway");
                    }
                    if (offset.getHumidityOffset() != null) {
                        alarmRuleVo.setOffset(offset.getHumidityOffset());
                    } else {
                        alarmRuleVo.setOffset(0.0);
                    }
                    break;
                case "ta":
                     if(alarmVo.getTiltUpper() !=null ){
                        alarmRuleVo.setUpperval(alarmVo.getTiltUpper());
                        alarmRuleVo.setMaxval(alarmVo.getTilt());
                        alarmRuleVo.setMode("upper_half");
                    }else {
                        alarmRuleVo.setMaxval(alarmVo.getTilt());
                        alarmRuleVo.setMode("greater");
                    }
                    break;
                case "a1":
                    if(alarmVo.getShockUpper() !=null ){
                        alarmRuleVo.setUpperval(alarmVo.getShockUpper().doubleValue());
                        alarmRuleVo.setMaxval(alarmVo.getShock().doubleValue());
                        alarmRuleVo.setMode("upper_half");
                    }else {
                        alarmRuleVo.setMaxval(alarmVo.getShock().doubleValue());
                        alarmRuleVo.setMode("greater");
                    }
                    break;
                default:
                        break;
            }
            alarmRuleVos.add(alarmRuleVo);
            //向redis中写入报警信息
            alarmRedisService.hmSet(topic,"TDL-" +TDLSN+"_" +alias,gson.toJson(alarmRuleVos));
        }
    }

    public static void updateOffset(String topic, String TDLSN, String alias, Double offset,
                                    AlarmRedisService alarmRedisService) {
        try {
            String hashKey = "TDL-" + TDLSN + "_" + alias;
            /* Get alarm rule from redis */
            Object hashValueObject = alarmRedisService.getHash(topic, hashKey);
            List<AlarmRuleVo> alarmRuleList = gson.fromJson(hashValueObject.toString(),
                    new TypeToken<ArrayList<AlarmRuleVo>>() {}.getType());
            /* Update offset */
            if (alarmRuleList != null && alarmRuleList.size() > 0) {
                alarmRuleList.get(0).setOffset(offset);
                /* Write back to redis */
                alarmRedisService.hmSet(topic, hashKey, gson.toJson(alarmRuleList));
            }
        } catch (JsonSyntaxException e) {
            logger.error("Update offset(" + alias + ") JsonSyntaxException: " + e.getMessage());
        }
    }

    public static void updateOffset(String topic, String TDLSN, TDLDataOffset offset,
                                    AlarmRedisService alarmRedisService) {
        if (offset == null)
            return;

        /* 更新温度校准值 */
        if (offset.getTempOffset() != null) {
            updateOffset(topic, TDLSN, "T", offset.getTempOffset(), alarmRedisService);
        }
        /* 更新湿度校准值 */
        if (offset.getHumidityOffset() != null) {
            updateOffset(topic, TDLSN, "h", offset.getHumidityOffset(), alarmRedisService);
        }
    }
}
