package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.AlarmService;

import com.example.tdl.service.TDLLogService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@RestController
@RequestMapping("/alarm")
public class AlarmController {

    private Logger logger = LoggerFactory.getLogger(AlarmController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private TDLLogService  tdlLogService;

    @Autowired
    private I18nController  i18n;

    //查询所有预警信息
    @ApiOperation(value = "查询所有预警信息",notes = "查询所有预警信息,返回值说明：" +
            "   alarmType：预警类型，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        return alarmService.getAll(user.getCompanyNo());
    }

    //根据预警类型模糊查询
    @ApiOperation(value = "根据预警类型模糊查询",notes = "查询所有预警信息,返回值说明：" +
            "   alarmType：预警类型，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAllByAlarmType",method = RequestMethod.POST)
    public Object getAllByAlarmType(@RequestBody String alarmType,HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        JSONObject jsonObject= JSON.parseObject(alarmType);
        alarmType=(String)jsonObject.get("alarmType");
        return alarmService.getAllByAlarmType(user.getCompanyNo(),alarmType);
    }

    //添加预警信息
    @ApiOperation(value = "添加预警信息",notes = "添加预警信息,companyNo不用传，需要传递参数：" +
            "   alarmType：预警类型，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addAlarm",method = RequestMethod.POST)
    public Object addAlarm(@RequestBody AddAlarmVo addAlarmVo,HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if (StringUtils.isEmpty(addAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmType"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTemMax()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMax"));
            return gson.toJson(fb);
        }
//        if(addAlarmVo.getTemMax()>85f){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"temMaxLimit"));
//            return gson.toJson(fb);
//        }
        if (addAlarmVo.getTemMin()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMin"));
            return gson.toJson(fb);
        }
//        if (addAlarmVo.getTemMin()<-25f){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"temMinLimit"));
//            return gson.toJson(fb);
//        }
//        if(addAlarmVo.getTemMin() >addAlarmVo.getTemMax()){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"wrongTemp"));
//            return gson.toJson(fb);
//        }
        if (addAlarmVo.getHumidityMax()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMax"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getHumidityMin()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMin"));
            return gson.toJson(fb);
        }
//        if (addAlarmVo.getHumidityMax()>100F){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"humidityMaxLimit"));
//            return gson.toJson(fb);
//        }
//        if (addAlarmVo.getHumidityMin()<0F){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"humidityMinLimit"));
//            return gson.toJson(fb);
//        }
//        if(addAlarmVo.getHumidityMin() >addAlarmVo.getHumidityMax()){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"wrongHumidity"));
//            return gson.toJson(fb);
//        }
        if (addAlarmVo.getTilt()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tilt"));
            return gson.toJson(fb);
        }
//        if (addAlarmVo.getTilt()<0 || addAlarmVo.getTilt()>180 ||  addAlarmVo.getTilt()%30 !=0){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"tiltLimit"));
//            return gson.toJson(fb);
//        }
        if (addAlarmVo.getShock()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"shock"));
            return gson.toJson(fb);
        }
//        if (Float.valueOf(addAlarmVo.getShock())<1.5 || Float.valueOf(addAlarmVo.getShock())>13.8){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"shockLimit"));
//            return gson.toJson(fb);
//        }
        if (alarmService.getByType(addAlarmVo.getAlarmType(),user.getCompanyNo())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmExist"));
            return gson.toJson(fb);
        }
        addAlarmVo.setCompanyNo(user.getCompanyNo());
        int a=alarmService.addAlarm(addAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addFailure"));
        }
        return gson.toJson(fb);
    }

    //修改预警信息
    @ApiOperation(value = "修改预警信息",notes = "修改预警信息,companyNo不用传，需要传递参数：" +
            "   oldAlarmType：需要修改的预警类型，" +
            "   alarmType：预警类型，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateAlarm",method = RequestMethod.POST)
    public Object updateAlarm(@RequestBody UpdateAlarmVo updateAlarmVo,HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if (StringUtils.isEmpty(updateAlarmVo.getOldAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedAlarmType"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmType"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTemMax()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMax"));
            return gson.toJson(fb);
        }
//        if(updateAlarmVo.getTemMax()>85f){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"temMaxLimit"));
//            return gson.toJson(fb);
//        }
        if (updateAlarmVo.getTemMin()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMin"));
            return gson.toJson(fb);
        }
//        if (updateAlarmVo.getTemMin()<-25f){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"temMinLimit"));
//            return gson.toJson(fb);
//        }
//        if(updateAlarmVo.getTemMin() >updateAlarmVo.getTemMax()){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"wrongTemp"));
//            return gson.toJson(fb);
//        }
        if (updateAlarmVo.getHumidityMax()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMax"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getHumidityMin()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMin"));
            return gson.toJson(fb);
        }
//        if (updateAlarmVo.getHumidityMax()>100F){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"humidityMaxLimit"));
//            return gson.toJson(fb);
//        }
//        if (updateAlarmVo.getHumidityMin()<0F){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"humidityMinLimit"));
//            return gson.toJson(fb);
//        }
        if(updateAlarmVo.getHumidityMin()>updateAlarmVo.getHumidityMax()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongHumidity"));
            return gson.toJson(fb);
        }
//        if (updateAlarmVo.getTilt()<0 || updateAlarmVo.getTilt()>180 ||  updateAlarmVo.getTilt()%30 !=0){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"tiltLimit"));
//            return gson.toJson(fb);
//        }
        if (updateAlarmVo.getTilt()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tilt"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getShock()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"shock"));
            return gson.toJson(fb);
        }
//        if (Float.valueOf(updateAlarmVo.getShock())<1.5 || Float.valueOf(updateAlarmVo.getShock())>13.8){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"shockLimit"));
//            return gson.toJson(fb);
//        }
        if (alarmService.getByType(updateAlarmVo.getAlarmType(),user.getCompanyNo())!=null&&!updateAlarmVo.getAlarmType().equals(updateAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedAlarmExist"));
            return gson.toJson(fb);
        }
        updateAlarmVo.setCompanyNo(user.getCompanyNo());
        int a=alarmService.updateAlarm(updateAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"modifiedSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedFailure"));
        }
        return gson.toJson(fb);

    }


    //删除预警信息
    @ApiOperation(value = "删除预警信息",notes = "删除预警信息,companyNo不用传，传值说明：" +
            "   alarmType：需要删除的预警类型" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/delAlarm",method = RequestMethod.POST)
    public Object delAlarm(@RequestBody DelAlarmVo delAlarmVo,HttpServletRequest request){
         UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if (StringUtils.isEmpty(delAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedAlarmType"));
            return gson.toJson(fb);
        }
        delAlarmVo.setCompanyNo(user.getCompanyNo());
        if(tdlLogService.getByAlarm(delAlarmVo).size()>0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"dusedAlarm"));
            return gson.toJson(fb);
        }
//        if (warehouseService.getByAlarm(delAlarmVo).size()>0){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"dusedAlarm"));
//            return gson.toJson(fb);
//        }
        int a=alarmService.delAlarm(delAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deletedSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedFailure"));
        }
        return gson.toJson(fb);
    }

}
