package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.OSSClient;
import com.example.tdl.common.CommonExcel;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.CircuitService;
import com.example.tdl.service.TDLDeviceService;
import com.example.tdl.service.redis.RealRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.example.tdl.util.ZipCompressUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.shiro.SecurityUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
public class AlarmDataController {
    private CommFeedback fb=new CommFeedback();

    private Gson gson=new Gson();
    @Autowired
    private CircuitService circuitService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private RealRedisService realRedisService;

    @Autowired
    private TokenRedisService tokenRedisService;


    @Value("${spring.influxdb.database}")
    private   String database ;

    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getTrack",method = RequestMethod.GET)
    public Object getTrack(){
        String transportationNo="CHN20180323G1111";
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=(String)jsonObject.get("transportationNo");
        ResultGatewayVo resultGatewayVo=circuitService.getByNo(transportationNo);
        if (resultGatewayVo==null){
            fb.setCode(0);
            fb.setMessage("该线路没有绑定网关");
            return gson.toJson(fb);
        }
        String device = resultGatewayVo.getType() +"_" +resultGatewayVo.getSN();
        List<String> list=tdlDeviceService.getTdlSN(resultGatewayVo.getSN(),resultGatewayVo.getType());
        String tdl="";
        if (list.size()==1){
            tdl="'tdl-"+list.get(0)+"'";
        }else if(list.size()==0){
            fb.setCode(0);
            fb.setMessage("该网关下没有tdl设备");
            return gson.toJson(fb);
        }else{
            for (int a=0;a<list.size();a++){
                tdl=tdl+"'tdl-"+list.get(a)+"'";
                if (a<list.size()-1){
                    tdl=tdl+"or \"tdl\"=";
                }
            }
        }
        List<LocationDataVo>  locationDataVos = new ArrayList<>();
        String sql ="SELECT \"time\",\"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" order by time";
        QueryResult queryResults  = influxDBTemplate.query(new Query(sql,database));
        if (queryResults.getResults().get(0).getSeries()!=null){
            locationDataVos=getAllAddress(queryResults);
        }
        List<TrackVo> trackVos1=new ArrayList<>();
        for (int a=0;a<locationDataVos.size();a++){
            TrackVo trackVo=new TrackVo();
            trackVo.setTime(dateToStamp(locationDataVos.get(a).getTime()));
            trackVo.setLng(locationDataVos.get(a).getLng().toString());
            trackVo.setLat(locationDataVos.get(a).getLat().toString());
            String sqls ="";
            if (a+1<locationDataVos.size()){
                sqls ="SELECT \"tdl\",\"T\", \"b\", \"h\", \"p\" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"= "+tdl+" AND \"time\"<='"+locationDataVos.get(a).getTime()+"' AND \"time\">='"+locationDataVos.get(a+1).getTime()+"' order by time limit 1";
            }else{
                sqls = "SELECT \"tdl\",\"T\", \"b\", \"h\", \"p\" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"= "+tdl+" AND \"time\"<='"+locationDataVos.get(a).getTime()+"' order by time limit 1";
            }
            QueryResult queryResult  = influxDBTemplate.query(new Query(sqls,database));
            if (queryResult.getResults().get(0).getSeries()!=null) {
                trackVo.setT(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString() : "-");
                trackVo.setB(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(3) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(3).toString() : "-");
                trackVo.setH(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(4) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(4).toString() : "-");
                trackVo.setA0(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(5) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(5).toString() : "-");
            }else{
                trackVo.setB("-");
                trackVo.setT("-");
                trackVo.setH("-");
                trackVo.setA0("-");
            }
            trackVos1.add(trackVo);
        }
        return gson.toJson(trackVos1);
    }

    //下载成表格
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @ApiOperation(value = "将数据下载成表格",notes = "将数据下载成表格")
    @RequestMapping(value = "/downloadExcel", method = RequestMethod.POST)
    public @ResponseBody Object downloadExcel(HttpServletRequest request,HttpServletResponse response)throws Exception{
        String transportationNo="CHN20180323G1111";
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=(String)jsonObject.get("transportationNo");
        ResultGatewayVo resultGatewayVo=circuitService.getByNo(transportationNo);
        if (resultGatewayVo==null){
            fb.setCode(0);
            fb.setMessage("该线路没有绑定网关");
            return gson.toJson(fb);
        }
        String device = resultGatewayVo.getType() +"_" +resultGatewayVo.getSN();
        List<String> list=tdlDeviceService.getTdlSN(resultGatewayVo.getSN(),resultGatewayVo.getType());
        String tdl="";
        if (list.size()==1){
            tdl="'tdl-"+list.get(0)+"'";
        }else if(list.size()==0){
            fb.setCode(0);
            fb.setMessage("该网关下没有tdl设备");
            return gson.toJson(fb);
        }else{
            for (int a=0;a<list.size();a++){
                tdl=tdl+"'tdl-"+list.get(a)+"'";
                if (a<list.size()-1){
                    tdl=tdl+"or \"tdl\"=";
                }
            }
        }
        List<LocationDataVo>  locationDataVos = new ArrayList<>();
        String sql ="SELECT \"time\",\"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" order by time desc";
        QueryResult queryResults  = influxDBTemplate.query(new Query(sql,database));
        if (queryResults.getResults().get(0).getSeries()!=null){
            locationDataVos=getAllAddress(queryResults);
        }
        List<TrackVo> trackVos1=new ArrayList<>();
        for (int a=0;a<locationDataVos.size();a++){
            TrackVo trackVo=new TrackVo();
            trackVo.setTime(locationDataVos.get(a).getTime());
            trackVo.setLng(locationDataVos.get(a).getLng().toString());
            trackVo.setLat(locationDataVos.get(a).getLat().toString());
            String sqls ="";
            if (a+1<locationDataVos.size()){
                sqls ="SELECT \"tdl\",\"T\", \"b\", \"h\", \"p\" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"= "+tdl+" AND \"time\"<='"+locationDataVos.get(a).getTime()+"' AND \"time\">='"+locationDataVos.get(a+1).getTime()+"' order by time limit 1";
            }else{
                sqls = "SELECT \"tdl\",\"T\", \"b\", \"h\", \"p\" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"= "+tdl+" AND \"time\"<='"+locationDataVos.get(a).getTime()+"' order by time limit 1";
            }
            QueryResult queryResult  = influxDBTemplate.query(new Query(sqls,database));
            if (queryResult.getResults().get(0).getSeries()!=null) {
                trackVo.setT(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString() : "-");
                trackVo.setB(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(3) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(3).toString() : "-");
                trackVo.setH(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(4) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(4).toString() : "-");
                trackVo.setA0(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(5) != null ? queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(5).toString() : "-");
            }else{
                trackVo.setB("-");
                trackVo.setT("-");
                trackVo.setH("-");
                trackVo.setA0("-");
            }
            trackVos1.add(trackVo);
        }
        if (trackVos1.size()>0){
            String name = "设备报警信息表";
            String title = "设  备  报  警  信  息  表";
            String[] rowsName = new String[]{"报警时间","经度","纬度","温度","电池电压","湿度","压力"};
            String fileName = transportationNo;
            File file=new File("D:/"+fileName);
            List<Object[]> dataList = new ArrayList<Object[]>();
            Object[] objs = null;
            for(int i = 0; i < trackVos1.size();i++){
                objs = new Object[rowsName.length];
                objs[0] = trackVos1.get(i).getTime();
                objs[1] = trackVos1.get(i).getLng();
                objs[2] = trackVos1.get(i).getLat();
                objs[3] = trackVos1.get(i).getT();
                objs[4] = trackVos1.get(i).getB();
                objs[5] = trackVos1.get(i).getH();
                objs[6] = trackVos1.get(i).getA0();
                dataList.add(objs);
            }
            try {
                CommonExcel ex = new CommonExcel(name,title, rowsName, dataList, fileName);
                HSSFWorkbook workbook=ex.downloadExcel();
                if (workbook!=null){
                    request.setCharacterEncoding("utf-8");
                    response.setCharacterEncoding("utf-8");
                    response.setContentType("application/vnd.ms-excel");
                    response.setHeader("Content-Disposition", "attachment; filename="+fileName);
                    OutputStream os = new FileOutputStream(new File(file+".xls"));
                    workbook.write(os);
                    os.flush();
                    os.close();
                    ZipCompressUtil zipCom = new ZipCompressUtil(file+".zip",file+".xls");
                    zipCom.zip();
                    try {
                        OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
                        String resultStr =  AliyunOSSClientUtil.uploadObject2OSS(ossClient, new File(file +".zip"));
                        if (new File(file+".xls").delete()){
                            fb.setCode(1);
                            fb.setMessage("https://witcloud-oss.oss-cn-shanghai.aliyuncs.com/tdlAlarmData/"+fileName +".zip");
                            return gson.toJson(fb);
                        }else{
                            fb.setCode(0);
                            fb.setMessage("下载失败");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        this.fb.setCode(Integer.valueOf(0));
                        this.fb.setMessage("上传失败");
                    }
                }else{
                    fb.setCode(0);
                    fb.setMessage("下载失败");
                }
            }catch(Exception e){
                e.printStackTrace();
                fb.setCode(0);
                fb.setMessage("下载失败");
            }
        }else{
            fb.setCode(0);
            fb.setMessage("没有数据可以下载");
        }
        return gson.toJson(fb);
    }






    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAlarmCircuit",method = RequestMethod.POST)
    public Object getAlarmCircuit(@RequestBody String transportationNo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        String transportationNo="CHN20180323G1111";
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        ResultGatewayVo resultGatewayVo=circuitService.getByNo(transportationNo);
        if (resultGatewayVo==null){
            fb.setCode(0);
            fb.setMessage("该线路没有绑定网关");
            return gson.toJson(fb);
        }
        Integer circuitState=circuitService.getByTransportationNo(transportationNo,user.getCompanyName()).getCircuitState();
        String device = resultGatewayVo.getType() +"_" +resultGatewayVo.getSN();
        if(circuitState==0){
            return null;
        }else if(circuitState==1){
            List<String> list=tdlDeviceService.getByGatewaySN(resultGatewayVo.getSN(),resultGatewayVo.getType());
            String tdl="";
            if (list.size()==1){
                tdl="'tdl-"+list.get(0)+"'";
            }else if(list.size()==0){
                fb.setCode(0);
                fb.setMessage("该网关下没有tdl设备");
                return gson.toJson(fb);
            }else{
                for (int a=0;a<list.size();a++){
                    tdl=tdl+"'tdl-"+list.get(a)+"'";
                    if (a<list.size()-1){
                        tdl=tdl+"or \"tdl\"=";
                    }
                }
            }
            String sql = "SELECT \"b\",\"tdl\" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"= "+tdl;
            QueryResult queryResult  = influxDBTemplate.query(new Query(sql,database));
            List<CircuitAlarmVo> circuitAlarmVos=new ArrayList<>();
            if (queryResult.getResults().get(0).getSeries()!=null){
                circuitAlarmVos=getAlarmData(queryResult);
            }
            String sqls ="";
            for (int a=0;a<circuitAlarmVos.size();a++){
                sqls=sqls+"SELECT \"time\",\"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" where time <='"+circuitAlarmVos.get(a).getTime()+"' ORDER BY time desc limit 1;";
            }
            QueryResult queryResults  = influxDBTemplate.query(new Query(sqls,database));
            List<LocationDataVo>  locationDataVos = new ArrayList<>();
            if (queryResults.getResults().get(0).getSeries()!=null){
                locationDataVos=getAddress(queryResults);
            }
            String sql1="SELECT \"time\",\"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" order by time";
            QueryResult queryResult1  = influxDBTemplate.query(new Query(sql1,database));
            List<LocationDataVo>  locationDataVos1 = new ArrayList<>();
            if (queryResult1.getResults().get(0).getSeries()!=null){
                locationDataVos1=getAllAddress(queryResult1);
            }
            for (int a=0;a<locationDataVos1.size();a++){
                for (int b=0;b<locationDataVos.size();b++){
                    if (locationDataVos1.get(a).getLng().equals(locationDataVos.get(b).getLng())&&locationDataVos1.get(a).getLat().equals(locationDataVos.get(b).getLat())){
                        locationDataVos1.get(a).setError(1);
                    }
                }
            }
            return gson.toJson(locationDataVos1);
        }else{
//            Map<Object,Object> map=new HashMap<>();
//            map.put(device,locationDataVos1);
//            realRedisService.setHash("CircuitAlarm",map,System.currentTimeMillis());
            Object o =realRedisService.getHash("CircuitAlarm",device);
            return gson.toJson(o);
        }

    }

    public List<TrackVo> getTrack(QueryResult queryResult){
        List<List<Object>> lists=getValues(queryResult);
        List<TrackVo> trackVos=new ArrayList<>();
        for (List<Object> value:lists){
            TrackVo trackVo=new TrackVo();
            trackVo.setTime(dateToStamp(value.get(0).toString()));
            trackVo.setB(value.get(1).toString());
            trackVo.setT(value.get(2).toString());
            trackVo.setH(value.get(3).toString());
            trackVo.setA0(value.get(4).toString());
            trackVos.add(trackVo);
        }
        return trackVos;
    }

    public List<LocationDataVo> getAddress(QueryResult queryResult){
        List<List<Object>> lists=getValue(queryResult);
        List<LocationDataVo> locationDataVos=new ArrayList<>();
        for (List<Object> value:lists){
            LocationDataVo locationDataVo=new LocationDataVo();
            locationDataVo.setTime(dateToStamp(value.get(0).toString()));
            locationDataVo.setLng((Double)value.get(1));
            locationDataVo.setLat((Double)value.get(2));
            locationDataVos.add(locationDataVo);
        }
        return locationDataVos;
    }

    public List<LocationDataVo> getAllAddress(QueryResult queryResult){
        List<List<Object>> lists=getValues(queryResult);
        List<LocationDataVo> locationDataVos=new ArrayList<>();
        for (List<Object> value:lists){
            LocationDataVo locationDataVo=new LocationDataVo();
            locationDataVo.setTime(value.get(0).toString());
            locationDataVo.setLng((Double)value.get(1));
            locationDataVo.setLat((Double)value.get(2));
//            locationDataVo.setError(0);
            locationDataVos.add(locationDataVo);
        }
        return locationDataVos;
    }

    public List<CircuitAlarmVo> getAlarmData(QueryResult queryResult){
        List<List<Object>> lists=getValues(queryResult);
        List<CircuitAlarmVo> alarmVoList=new ArrayList<>();
        for (List<Object> value:lists){
            CircuitAlarmVo circuitAlarmVo=new CircuitAlarmVo();
            circuitAlarmVo.setTime(value.get(0).toString());
            circuitAlarmVo.setShockEnergy(value.get(1).toString());
            circuitAlarmVo.setTdl(value.get(2).toString());
            alarmVoList.add(circuitAlarmVo);
        }
        return alarmVoList;
    }

    public List<List<Object>> getValues(QueryResult queryResult){
        List<List<Object>> lists=new ArrayList<>();
        for (QueryResult.Result result:queryResult.getResults()){
            if (result.getSeries()!=null){
                for (QueryResult.Series series:result.getSeries()){
                    if (series!=null){
                        lists=series.getValues();
                    }
                }
            }
        }
        return lists;
    }

    public List<List<Object>> getValue(QueryResult queryResult){
        List<List<Object>> lists=new ArrayList<>();
        for (QueryResult.Result result:queryResult.getResults()){
            if (result.getSeries()!=null){
                for (int a=0;a<result.getSeries().size();a++){
                    lists.add(result.getSeries().get(a).getValues().get(0));
                }
            }
        }
        return lists;
    }

    public String dateToStamp(String time){
        String res;
        Date date=null;
        try {
            if (time.length()!=20){
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
                date = simpleDateFormat.parse(time);
            }else{
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
                date=simpleDateFormat.parse(time);
            }
            long ts = date.getTime()*1000000;
            res = String.valueOf(ts);
        }catch (Exception e){
            fb.setCode(0);
            fb.setMessage("时间转换出错");
            return gson.toJson(fb);
        }
        return res;
    }
}