package com.example.tdl.web;

import com.aliyun.oss.OSSClient;
import com.csvreader.CsvWriter;
import com.example.tdl.common.CommonExcel;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.example.tdl.util.CSVUtils;
import com.example.tdl.util.ZipCompressUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;



@RestController
@RequestMapping("/alarmLogABB")
public class AlarmLogABBController {
    private Logger logger= LoggerFactory.getLogger(AlarmLogABBController.class);

    Gson gson= new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private AlarmLogABBService alarmLogABBService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private I18nController i18n;


    //获取ABB所有报警记录
    @ApiOperation(value = "获取ABB所有报警记录",notes = "获取ABB所有报警记录," +
            "返回值:" +
            "   transportationNo:运输编号，" +
            "   cargoNo:货号，" +
            "   TDLSN:传感器编号，" +
            "   description:报警内容，" +
            "   data:报警数据，" +
            "   alarmTime:报警时间，" +
            "   classify:报警参数" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @GetMapping("/getAll")
    public Object getAll(@RequestParam(value ="page",defaultValue = "1")Integer page,
                           @RequestParam(value = "rows",defaultValue = "10")Integer rows,
                           HttpServletRequest request){
        UserRedisVo user=gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        List<ResultAlarmLogABBVo> resultAlarmLogABBVos = alarmLogABBService.getAll(user.getCompanyNo(), (page-1)*rows, rows);
        Integer num=alarmLogABBService.getAllCount(user.getCompanyNo());
        EasyUIResult easyUIResult = new EasyUIResult(num, resultAlarmLogABBVos);
        return ResponseEntity.ok(easyUIResult);
    }

    //根据条件查询ABB所有报警记录
    @ApiOperation(value = "根据条件查询ABB所有报警记录",notes = "根据条件查询ABB所有报警记录,传值说明：" +
            "   classify:报警内容，" +
            "   startTime:出发时间（时间戳），" +
            "   endTime:结束时间（时间戳），" +
            "返回值:" +
            "   transportationNo:运输编号，" +
            "   cargoNo:货号，" +
            "   TDLSN:传感器编号，" +
            "   description:报警内容，" +
            "   data:报警数据，" +
            "   alarmTime:报警时间，" +
            "   classify:报警参数" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getByTerm")
    public Object getByTerm(@RequestBody AlarmTermABBVo alarmTermABBVo,
                           @RequestParam(value ="page",defaultValue = "1")Integer page,
                           @RequestParam(value = "rows",defaultValue = "10")Integer rows,
                           HttpServletRequest request){
        UserRedisVo user=gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if(StringUtils.isEmpty(alarmTermABBVo.getClassify())){
            alarmTermABBVo.setClassify(null);
        }
        if(alarmTermABBVo.getStartTime()==null){
            alarmTermABBVo.setStartTime(null);
        }
        if(alarmTermABBVo.getEndTime()==null){
            alarmTermABBVo.setEndTime(null);
        }
        alarmTermABBVo.setCompanyNo(user.getCompanyNo());
        List<ResultAlarmLogABBVo> resultAlarmLogABBVos=alarmLogABBService.getByTerm(alarmTermABBVo, (page-1)*rows, rows);
        Integer num=alarmLogABBService.getByTermCount(alarmTermABBVo);
        EasyUIResult easyUIResult = new EasyUIResult(num, resultAlarmLogABBVos);
        return ResponseEntity.ok(easyUIResult);
    }


    //获取近30天的柱状图数据
    @ApiOperation(value = "获取近30天的柱状图数据",notes = "获取近30天的柱状图数据" +
            "返回值:" +
            "   dates:运输编号，" +
            "   tempCountSum:货号，" +
            "   humidCountSum:传感器编号，" +
            "   shockCountSum:报警内容，" +
            "   leanCountSum:报警数据，" +
            "   countSum:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getCountForLineGraphForDay")
    public Object getCountForLineGraphForDay(){
        List<ResultAlarmLogCountABBVo> resultAlarmLogABBVos=alarmLogABBService.getDayCountForLineGraph();
        return resultAlarmLogABBVos;
    }

    //获取近一年的柱状图数据
    @ApiOperation(value = "获取近一年的柱状图数据",notes = "获取近一年的柱状图数据" +
            "返回值:" +
            "   dates:运输编号，" +
            "   tempCountSum:货号，" +
            "   humidCountSum:传感器编号，" +
            "   shockCountSum:报警内容，" +
            "   leanCountSum:报警数据，" +
            "   countSum:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getCountForLineGraphForYear")
    public Object getCountForLineGraphForYear(){
        List<ResultAlarmLogCountABBVo> resultAlarmLogABBVos=alarmLogABBService.getYearCountForLineGraph();
        return resultAlarmLogABBVos;
    }

    //获取近30天的饼图数据--报警类型占比
    @ApiOperation(value = "获取近30天的饼图数据--报警类型占比",notes = "获取近30天的饼图数据--报警类型占比" +
            "返回值:" +
            "   tempCountSum:货号，" +
            "   humidCountSum:传感器编号，" +
            "   shockCountSum:报警内容，" +
            "   leanCountSum:报警数据，" +
            "   countSum:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getTypeCountForPieGraphForDay")
    public Object getTypeCountForPieGraphForDay(){
        ResultAlarmLogCountPieGraphVo resultAlarmLogCountPieGraphVo=alarmLogABBService.getDaySumForPieGraph();
        return resultAlarmLogCountPieGraphVo;
    }

    //获取近一年的饼图数据--报警类型占比
    @ApiOperation(value = "获取近一年的饼图数据--报警类型占比",notes = "获取近一年的饼图数据--报警类型占比" +
            "返回值:" +
            "   tempCountSum:货号，" +
            "   humidCountSum:传感器编号，" +
            "   shockCountSum:报警内容，" +
            "   leanCountSum:报警数据，" +
            "   countSum:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getTypeCountForPieGraphForYear")
    public Object getTypeCountForPieGraphForYear(){
        ResultAlarmLogCountPieGraphVo resultAlarmLogCountPieGraphVo=alarmLogABBService.getYearSumForPieGraph();
        return resultAlarmLogCountPieGraphVo;
    }

    //获取近30天的饼图数据--报警时间段占比
    @ApiOperation(value = "获取近30天的饼图数据--报警时间段占比",notes = "获取近30天的饼图数据--报警时间段占比" +
            "返回值:" +
            "   house:小时，" +
            "   tempCountSum:货号，" +
            "   humidCountSum:传感器编号，" +
            "   shockCountSum:报警内容，" +
            "   leanCountSum:报警数据，" +
            "   countSum:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getTimeCountForPieGraphForDay")
    public Object getTimeCountForPieGraphForDay(){
        List<ResultAlarmLogHouseCountPieGraphVo> resultAlarmLogHouseCountPieGraphVos=alarmLogABBService.getDayHouseSumForPieGraph();
        return resultAlarmLogHouseCountPieGraphVos;
    }

    //获取近一年的饼图数据--报警时间段占比
    @ApiOperation(value = "获取近一年的饼图数据--报警时间段占比",notes = "获取近一年的饼图数据--报警时间段占比" +
            "返回值:" +
            "   house:小时，" +
            "   tempCountSum:货号，" +
            "   humidCountSum:传感器编号，" +
            "   shockCountSum:报警内容，" +
            "   leanCountSum:报警数据，" +
            "   countSum:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getTimeCountForPieGraphForYear")
    public Object getTimeCountForPieGraphForYear(){
        List<ResultAlarmLogHouseCountPieGraphVo> resultAlarmLogHouseCountPieGraphVos=alarmLogABBService.getYearHouseSumForPieGraph();
        return resultAlarmLogHouseCountPieGraphVos;
    }

    //报警记录下载
    @ApiOperation(value = "报警记录下载",notes = "报警记录下载" +
            "   classify:报警内容，" +
            "   startTime:出发时间（时间戳），" +
            "   endTime:结束时间（时间戳）")
    @RequestMapping(value = "/downloadExcel", method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object downloadExcel(@RequestBody AlarmTermABBVo alarmTermABBVo,HttpServletRequest request,HttpServletResponse response){
        UserRedisVo user=gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if(StringUtils.isEmpty(alarmTermABBVo.getClassify())){
            alarmTermABBVo.setClassify(null);
        }
        if(alarmTermABBVo.getStartTime()==null||alarmTermABBVo.getStartTime()==0){
            alarmTermABBVo.setStartTime(null);
        }
        if(alarmTermABBVo.getEndTime()==null||alarmTermABBVo.getEndTime()==0){
            alarmTermABBVo.setEndTime(null);
        }
        if(alarmTermABBVo.getEndTime()!=null&&alarmTermABBVo.getStartTime()!=null&&
                alarmTermABBVo.getEndTime() < alarmTermABBVo.getStartTime()){
            fb.setCode(0);
            fb.setMessage("开始时间不能大于结束时间");
            return gson.toJson(fb);
        }
        alarmTermABBVo.setCompanyNo(user.getCompanyNo());
        List<ResultAlarmLogABBVo> resultAlarmLogVos=alarmLogABBService.getByTermForDownload(alarmTermABBVo);
        if (resultAlarmLogVos.size()>0){
            List<Object> head = new ArrayList<Object>(){{
                add("运输编号");
                add("货物编号");
                add("传感器序列号");
                add("描述");
                add("报警数据");
                add("报警时间");
                add("报警参数");
            }};
            List<List<Object>> dataList = new ArrayList<>();
            SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            for (int i = 0; i < resultAlarmLogVos.size(); ++i) {
                List<Object> values= new ArrayList<>();
                values.add(resultAlarmLogVos.get(i).getTransportationNo());
                values.add(resultAlarmLogVos.get(i).getCargoNo());
                values.add(resultAlarmLogVos.get(i).getTDLSN());
                values.add(resultAlarmLogVos.get(i).getDescription());
                values.add(resultAlarmLogVos.get(i).getData());
               //这个是你要转成后的时间的格式
                values.add(sdf.format(new Date(resultAlarmLogVos.get(i).getAlarmTime())));
                values.add(resultAlarmLogVos.get(i).getClassify());
                 dataList.add(values);
            }
            String fileName="报警日志数据-"+System.currentTimeMillis();
            String address="./export/";
            CSVUtils.createCSVFile(head,dataList,address,fileName);
            ZipCompressUtil zipCom = new ZipCompressUtil(address + fileName+".zip", address + fileName+".csv");
            try {
                zipCom.zip();
            } catch ( Exception e ) {
                e.printStackTrace();
            }
            OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
            String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, new File(address + fileName+ ".zip"));
            if (md5Key != null) {
                fb.setCode(1);
                fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/abbAlarm/" + fileName+ ".zip");
                new File(address + fileName).delete();
                return gson.toJson(fb);
            } else {
                fb.setCode(0);
                fb.setMessage("上传失败");
            }
//            try {
//                CsvWriter csvWriter = new CsvWriter(address  + fileName+".csv", ',', Charset.forName("GB2312"));
//                // 创建CSV写对象 例如:CsvWriter(文件路径，分隔符，编码格式);
//                String[] rowsName={"运输编号","货物编号","传感器序列号","描述","报警数据","报警时间","报警参数"};
//
//                csvWriter.writeRecord(rowsName);
//                for (int i = 0; i < resultAlarmLogVos.size(); ++i) {
//                    String[] objs = new String[rowsName.length];
//                    objs[0] = String.valueOf(resultAlarmLogVos.get(i).getTransportationNo());
//                    objs[1] = String.valueOf(resultAlarmLogVos.get(i).getCargoNo());
//                    objs[2] = String.valueOf(resultAlarmLogVos.get(i).getTDLSN());
//                    objs[3] = String.valueOf(resultAlarmLogVos.get(i).getDescription());
//                    objs[4] = String.valueOf(resultAlarmLogVos.get(i).getData());
//                    objs[5] = String.valueOf(sdf.format(new Date(resultAlarmLogVos.get(i).getAlarmTime())));
//                    objs[6] = String.valueOf(resultAlarmLogVos.get(i).getClassify());
//                    csvWriter.writeRecord(objs);
//                }
//                csvWriter.flush();
//                csvWriter.close();
//                ZipCompressUtil zipCom = new ZipCompressUtil(address + fileName+".zip", address + fileName+".csv");
//                zipCom.zip();
//                OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
//                String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, new File(address + fileName+ ".zip"));
//                if (md5Key != null) {
//                    fb.setCode(1);
//                    fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/abbAlarm/" + fileName+ ".zip");
//                    new File(address + fileName).delete();
//                    return gson.toJson(fb);
//                } else {
//                    fb.setCode(0);
//                    fb.setMessage("上传失败");
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//                fb.setCode(0);
//                fb.setMessage("下载失败");
//            }
        }else{
            fb.setCode(0);
            fb.setMessage("没有数据可以下载");
        }
        return gson.toJson(fb);
    }

}
