package com.example.tdl.web;

import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.AlarmLog;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AlarmLogUtil;
import com.example.tdl.util.AlidayuUtils;
import com.example.tdl.util.MailUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


@RestController
@RequestMapping("/alarmLog")
public class AlarmLogController {
    private Logger logger= LoggerFactory.getLogger(AlarmLogController.class);

    Gson gson= new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private AlarmLogService alarmLogServcie;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private WarehouseService  warehouseService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private WarehouseAlarmLogService warehouseAlarmLogService;


    @Value("${spring.influxdb.database}")
    private   String database ;

    @Value("${mqtt.userName}")
    private String userName;

    @Value("${mqtt.password}")
    private String password;

    @Value("${alarm.interval.time}")
    private Long alarmIntervalTime;


    //获取报警信息
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "   transportationNo:运输编号" +
            "   description:报警内容" +
            "   startCity:启运城市" +
            "   endCity:目的城市" +
            "   startTime:出发时间（时间戳）" +
            "   endTime:结束时间（时间戳）" +
            "返回值:      transportationNo:运输编号" +
            "   cargoNo:货号" +
            "   startCity:启运城市" +
            "   endCity:目的城市" +
            "   description:报警内容" +
            "   alarmTime:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getAlarm")
    public Object getAlarm(@RequestBody AlarmTermVo alarmTermVo ,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if(StringUtils.isEmpty(alarmTermVo.getTransportationNo())){
            alarmTermVo.setTransportationNo(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getDescription())){
            alarmTermVo.setDescription(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getEndCity())){
            alarmTermVo.setEndCity(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getStartCity())){
            alarmTermVo.setStartCity(null);
        }
        if(alarmTermVo.getStartTime()==null){
            alarmTermVo.setStartTime(null);
        }
        if(alarmTermVo.getDescription()==null){
            alarmTermVo.setEndTime(null);
        }
        alarmTermVo.setCompanyNo(user.getCompanyNo());
        return  alarmLogServcie.getAlarm(alarmTermVo);
    }

    //获取最新报警
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "返回值:  transportationNo:运输编号" +
            "data:报警数据" +
            "description:报警内容" +
            "alarmTime:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getRealTimeAlarm")
    public Object getRealTimeAlarm(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        List<AlarmLogVo> list =   alarmLogServcie.getRealTimeAlarm(user.getCompanyNo());
        return gson.toJson(list);
    }


    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "返回值:  userName:用户名" +
            "password:密码" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @GetMapping("/getAlarmMessage")
    public String getAlarmMessage(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        return  gson.toJson(new MqttUser(userName,password,userRedisVo.getCompanyNo()));
    }


    //新增报警消息
    @ApiOperation(value = "增加报警",notes = "增加报警")
    @RequestMapping(value = "/addAlarmLog",method = RequestMethod.POST)
    public String addAlarmLog(@RequestBody String addAlarmLog){
        logger.info(addAlarmLog);
        AddAlarmLogVo addAlarmLogVo = gson.fromJson(addAlarmLog,AddAlarmLogVo.class);
        logger.info(System.currentTimeMillis()+"--------"+gson.toJson(addAlarmLogVo));
        if(StringUtils.isEmpty(addAlarmLogVo.getGsn())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
        }
        if(StringUtils.isEmpty(addAlarmLogVo.getGtype())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
        }
        if(addAlarmLogVo.getTs() == null){
            fb.setCode(0);
            fb.setMessage("时间不能为空");
        }
        //判断当前报警状态
        AlarmNoVo  alarmNoVo = gatewayService.getByGateway(addAlarmLogVo.getGsn(),addAlarmLogVo.getGtype());
        if(alarmNoVo!=null && alarmNoVo.getMode() == 1){
            ResultCircuitVo resultCircuitVo = circuitService.getCircuit(alarmNoVo.getTransportationNo());
            if(addAlarmLogVo.getAlarmno() !=null){
                //网关离线报警
                String desp ="Gateway("+addAlarmLogVo.getGsn() +")"+ AlarmLogUtil.getDescription(addAlarmLogVo.getAlarmno());
                String classify ="6";
                Long  time  = addAlarmLogVo.getTs();
                if(time.toString().length() == 10){
                    time=time*1000;
                }
                AlarmLog alarmLog = new AlarmLog(alarmNoVo.getTransportationNo(),resultCircuitVo.getStartCity(),resultCircuitVo.getEndCity(),null,desp,null,time,classify,"offline",addAlarmLogVo.getAlarmno());
                alarmLogServcie.addAlarmLog(alarmLog);
            }
            if(addAlarmLogVo.getAlarm() !=null &&  addAlarmLogVo.getAlarm().size()>0){
                for(int i = 0,length=addAlarmLogVo.getAlarm().size();i<length;i++) {
                    if (addAlarmLogVo.getAlarm().get(i).getAlarmno() != null && addAlarmLogVo.getAlarm().get(i).getAlarmno() == 0) {
                        continue;
                    }
                    String classify = null;
                    String desp = AlarmLogUtil.getDescription(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    if(addAlarmLogVo.getAlarm().get(i).getAlias()!=null){
                        if (addAlarmLogVo.getAlarm().get(i).getAlias().contains("T")) {
                            desp =  addAlarmLogVo.getAlarm().get(i).getDesp() + desp;
                            classify = "1";
                        } else if (addAlarmLogVo.getAlarm().get(i).getAlias().contains("h")) {
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp() + desp;
                            classify = "2";
                        } else if (addAlarmLogVo.getAlarm().get(i).getAlias().contains("a1")) {
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp()+ desp;
                            classify = "3";
                        } else if (addAlarmLogVo.getAlarm().get(i).getAlias().contains("ta")) {
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp()+ desp;
                            classify = "4";
                        }
                    }else{
                        desp = "TDL("+addAlarmLogVo.getAlarm().get(i).getTdlsn() +")"+desp;
                        classify = "6";
                    }
                    Long  time  = addAlarmLogVo.getAlarm().get(i).getTs();
                    if(time.toString().length() == 10){
                        time=time*1000;
                    }
                    AlarmLog alarmLog = new AlarmLog(alarmNoVo.getTransportationNo(),resultCircuitVo.getStartCity(),resultCircuitVo.getEndCity(),addAlarmLogVo.getAlarm().get(i).getTdlsn().replace("TDL-","").trim(),desp,addAlarmLogVo.getAlarm().get(i).getValue(),time,classify,addAlarmLogVo.getAlarm().get(i).getDesp(),addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    alarmLogServcie.addAlarmLog(alarmLog);

                }
            }
        }else if(alarmNoVo!=null && alarmNoVo.getMode() == 2) {
            //获取最后一条发送的报警时间
            Long  alarmTime =  warehouseAlarmLogService.getLastProcessedAlarm(alarmNoVo.getWarehouseNo());
            //获取联系人信息
            List<ResultUserVo> userList = warehouseService.getUserByWarehouse(alarmNoVo.getWarehouseNo());
            if(addAlarmLogVo.getAlarmno() !=null && addAlarmLogVo.getAlarmno()!=0){
                //网关离线报警
                String desp = "Gateway("+addAlarmLogVo.getGsn() +")"+ AlarmLogUtil.getDescription(addAlarmLogVo.getAlarmno());
                String classify ="6";
                Long  time  = addAlarmLogVo.getTs();
                if(time.toString().length() == 10){
                    time=time*1000;
                }
                //发送短信邮件
                Integer delState = 0;
                if(alarmTime == null  || (time -alarmTime) >alarmIntervalTime){
                    sendMessage(alarmNoVo.getWarehouseName(),desp,userList);
                    delState = 1;
                }
                WarehouseAlarmLog alarmLog = new  WarehouseAlarmLog(alarmNoVo.getWarehouseNo(),addAlarmLogVo.getGsn(),addAlarmLogVo.getGtype(),null,desp,null,time,"offline",addAlarmLogVo.getAlarmno(),classify,delState);
                warehouseAlarmLogService.add(alarmLog);
            }
            if(addAlarmLogVo.getAlarm() !=null &&  addAlarmLogVo.getAlarm().size()>0){
                for(int i = 0,length=addAlarmLogVo.getAlarm().size();i<length;i++){
                    if(addAlarmLogVo.getAlarm().get(i).getAlarmno() !=null && addAlarmLogVo.getAlarm().get(i).getAlarmno() == 0){
                        continue;
                    }
                    String desp = AlarmLogUtil.getDescription(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    String classify = null;
                    if(addAlarmLogVo.getAlarm().get(i).getAlias() !=null){
                        if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("T")){
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp() + desp;
                            classify = "1";
                        } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("h")){
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp() +desp;
                            classify = "2";
                        } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("a1")){
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp() +desp;
                            classify = "3";
                        } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("ta")) {
                            desp = addAlarmLogVo.getAlarm().get(i).getDesp() + desp;
                            classify = "4";
                        }
                    }else{
                        desp = "TDL("+addAlarmLogVo.getAlarm().get(i).getTdlsn() +") "+desp;
                        classify = "6";
                    }
                    Long  time  = addAlarmLogVo.getAlarm().get(i).getTs();
                    if(time.toString().length() == 10){
                        time=time*1000;
                    }
                     Integer delState = 0;
                    if(alarmTime== null || (time -alarmTime) >alarmIntervalTime){
                        //发送短信邮件
                        sendMessage(alarmNoVo.getWarehouseName(),desp,userList);
                        delState = 1;
                    }
                    WarehouseAlarmLog alarmLog = new  WarehouseAlarmLog(
                            alarmNoVo.getWarehouseNo(),
                            addAlarmLogVo.getGsn(),
                            addAlarmLogVo.getGtype(),
                            addAlarmLogVo.getAlarm().get(i).getTdlsn(),
                            desp,
                            addAlarmLogVo.getAlarm().get(i).getValue(),
                            time,
                            addAlarmLogVo.getAlarm().get(i).getDesp(),
                            addAlarmLogVo.getAlarm().get(i).getAlarmno(),
                            classify,
                            delState);
                    warehouseAlarmLogService.add(alarmLog);
                }
            }
        }
        return gson.toJson(fb);
    }


    //获取报警信息
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "transportationNo:运输编号" +
            "TDLSN:传感器编号" +
            "time :报警时间（13位时间戳）" +
            "type:地图类型（百度：bd09,谷歌：wgs84）" +
            "返回值：" +
            "transportationNo:线路编号" +
            "temp:温度" +
            "humidity：湿度" +
            "shock：震动" +
            "shockEnergy：震动能量" +
            "titl：倾斜" +
            "lng:经度" +
            "lat：纬度" +
            "time：时间")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getAlarmData")
    public Object getAlarmData(@RequestBody AlarmMessageVo alarmMessageVo,HttpServletRequest request){
        if(StringUtils.isEmpty(alarmMessageVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (circuitService.getNo(alarmMessageVo.getTransportationNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(alarmMessageVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDL"));
            return gson.toJson(fb);
        }
        if(alarmMessageVo.getAlarmTime() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"time"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(alarmMessageVo.getType())){
            alarmMessageVo.setType("wgs84");
        }
        //获取设备的网关信息
        TDLLogVo tdlLogVo =  tdlLogService.getByTransportationNo(alarmMessageVo.getTransportationNo());
        if(tdlLogVo ==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        //获取经纬度
        String sql = "";
        String device = tdlLogVo.getgType()+"_" + tdlLogVo.getgSN();
        CircuitAlarmDataVo circuitAlarmDataVo =new CircuitAlarmDataVo();
        if("bd09".equals(alarmMessageVo.getType())){
            sql = "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\""+device+"\" where time >= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time  limit 1;SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\""+device+"\" where time <= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time  desc limit 1";
        }else{
            sql =  "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" where time >= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time  limit 1;SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" where time <= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time desc limit 1;";
        }
        QueryResult queryResult =influxDBTemplate.query(new Query(sql,database));
        if(queryResult.getResults() != null){
            //获取最精确的经纬度
            circuitAlarmDataVo =   parseLocation(queryResult,alarmMessageVo.getAlarmTime());
        }else{
            circuitAlarmDataVo.setLng("");
            circuitAlarmDataVo.setLat("");
        }
        //获取数据
        Map<String,String> map = new HashMap<>();
        map.put("\"T\"","temp");
        map.put("\"h\"","humidity");
        map.put("\"a0\"","shockEnergy");
        map.put("\"a1\"","shock");
        map.put("\"ta\"","tilt");
        String sql1 = "select \"T\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult1 =influxDBTemplate.query(new Query(sql1,database));
        if(queryResult1.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setTemp(parseData(queryResult1,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setTemp("");
        }
        String sql2 = "select \"h\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult2 =influxDBTemplate.query(new Query(sql2,database));
        if(queryResult2.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setHumidity(parseData(queryResult2,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setHumidity("");
        }
        String sql3 = "select \"a0\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult3 =influxDBTemplate.query(new Query(sql3,database));
        if(queryResult3.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setShockEnergy(parseData(queryResult3,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setShockEnergy("");
        }
        String sql4 = "select \"a1\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult4 =influxDBTemplate.query(new Query(sql4,database));
        if(queryResult4.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setShock(parseData(queryResult4,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setShock("");
        }
        String sql5 = "select \"ta\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult5 =influxDBTemplate.query(new Query(sql5,database));
        if(queryResult5.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setTitl(parseData(queryResult5,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setTitl("");
        }
        circuitAlarmDataVo.setTransportationNo(alarmMessageVo.getTransportationNo());
        circuitAlarmDataVo.setTime(alarmMessageVo.getAlarmTime());
        return  gson.toJson(circuitAlarmDataVo);
    }

    //经纬度数据
    public CircuitAlarmDataVo parseLocation(QueryResult queryResult,Long time){
        String t1 ="";
        if(queryResult.getResults().get(0)==null  || queryResult.getResults().get(0) .getSeries() == null){
            t1 = "0";
        }else {
            t1 = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0) == null ? "" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString();
        }
        String t2="";
        if(queryResult.getResults().get(1) == null ||queryResult.getResults().get(1).getSeries()== null){
            t2="0";
        }else{
            t2 = queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(0) == null ? "" : queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(0).toString();
        }
        CircuitAlarmDataVo circuitAlarmDataVo = new CircuitAlarmDataVo();
        try{
            String lng="";
            String lat="";
            if((Long.valueOf(dateToStamp(t1)) +8*60*60*1000-time)<(time-Long.valueOf(dateToStamp(t2))-8*60*60*1000)){
                lng = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1) == null?"" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString();
                lat = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2) == null?"" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString();
            }else{
                lng = queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(1) == null?"" : queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(1).toString();
                lat = queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(2) == null?"" : queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(2).toString();
            }
            circuitAlarmDataVo.setLat(lat);
            circuitAlarmDataVo.setLng(lng);
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
        }
        return  circuitAlarmDataVo;
    }

    //解析数据
    public String parseData(QueryResult queryResult,Long time){
        List<Object> values = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0);
        //判断时间差距
        String t="";
        try{
            t=  dateToStamp(values.get(0).toString());
            logger.info(t);
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
            return "";
        }
        if((time -Long.valueOf(t)-60*60*8*1000)>1000*60){
            return "";
        }else{
            //数据
            String value  = values.get(1) == null?"":values.get(1).toString();
            logger.info(value);
            return  value;
        }


    }


    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }


    public  void sendMessage(String warehouseName,String desp,List<ResultUserVo> userList)  {
        for(int i = 0;i<userList.size();i++){
            AlidayuUtils.sendMessage(userList.get(i).getPhone(),"SMS_173475190","辉泰云平台","{\"name\":\""+warehouseName+"\",\"desp\":\""+desp+"\"}");
            int finalI = i;
            try {
                MailUtil.sendMail(new ArrayList<String>(){{add(userList.get(finalI).getEmail());}}, "辉度物流监测系统------仓库报警提醒",message(warehouseName,desp));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }


    public String message(String  warehouseName,String desp){
        String html="<!DOCTYPE html>\n" +
                "<html lang=\"en\">\n" +
                "<head>\n" +
                "    <meta charset=\"UTF-8\">\n" +
                "    <title>Title</title>\n" +
                "</head>\n" +
                "<body style=\"align-content: center;\">\n" +
                "    <div style=\"width: 800px;margin: 0px auto;\">\n" +
                "        <div style=\"font-size: 15px;\">\n" +
                "            <p>尊敬的用户，您好：</p>\n" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;,您所管理的"+warehouseName+"仓库发生异常:"+desp+"，请及时处理</p>\n"+
                "            <br>\n" +
                "            <p>致好！</p>\n" +
                "            <br>\n" +
                "        </div>\n" +
                "        <br>\n" +
                "        <div style=\"background-image: url(http://211.161.200.169:38091/test/dist/assets/images/bg.jpg) ; width: 800px;height: 125px;background-repeat: no-repeat;\">\n" +
                "            <div style=\"float: left;\">\n" +
                "                <div style=\"width:240px;height:110px;padding-top:15px;margin-left: 45px;float: left\">\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 5px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/WitiumGuan.png \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 13px;font-weight: 300;padding-top: 36px\">辉度官方网站</span>\n" +
                "                    <a href=\"http://www.witium.cn\"><img src=\"http://211.161.200.169:38091/test/dist/assets/images/guan.png\" width=\"110\" height=\"35\"  style=\"margin-top: 3px\"></a>\n" +
                "                </div>\n" +
                "                <div style=\"width:240px;height:110px;margin-top:-110px;margin-left: 310px;float: inherit;\" >\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 5px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/WitCloud.png \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 15px;font-weight: 300;padding-top: 50px\">云平台网站</span>\n" +
                "                </div>\n" +
                "                <div style=\"width:240px;height:110px;margin-top: -110px;float: right;\">\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 4px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/Witium.jpg \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 35px;font-weight: 300;padding-top: 50px\">辉度微信公众号</span>\n" +
                "                </div>\n" +
                "            </div>\n" +
                "        </div>\n" +
                "        <div style=\"width: 800px;height:40px;font-size: 14px;background-color: #2196F3;\">\n" +
                "            <span style=\"display:block;color: #f8f7ff;text-align: center;padding: 10px;\">\n" +
                "                TEL:086-21-37774020&nbsp;&nbsp;&nbsp;&nbsp;FAX:086-21-37774010&nbsp;&nbsp;&nbsp;&nbsp;Email:sales@witium.com\n" +
                "            </span>\n" +
                "        </div>\n" +
                "    </div>\n" +
                "</body>\n" +
                "</html>";

        return html;
    }
}
