package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.AlarmService;
import com.example.tdl.service.CarService;
import com.example.tdl.service.TDLLogService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;


@RestController
@RequestMapping("/car")
public class CarController {

    private Logger logger = LoggerFactory.getLogger(CarController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private CarService carService=new CarService();

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;

    //查询该公司的车辆信息
    @ApiOperation(value = "查询该公司的车辆信息",notes = "查询该公司的车辆信息,返回值说明：" +
            "   carNo：车牌号，" +
            "   type：车辆类型，" +
            "   img：车辆图片地址，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return carService.getAll(user.getCompanyNo());
    }

    //根据车牌号模糊查询
    @ApiOperation(value = "根据车牌号模糊查询",notes = "根据车牌号模糊查询,返回值说明：" +
            "   carNo：车牌号，" +
            "   type：车辆类型，" +
            "   img：车辆图片地址，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByCarNoForLike",method = RequestMethod.POST)
    public Object getByCarNoForLike(@RequestBody String carNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(carNo)){
            fb.setMessage(i18n.getMessage(request,"carNo"));
            fb.setCode(0);
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(carNo);
        carNo=(String)jsonObject.get("carNo");
        return carService.getByCarNoForLike(user.getCompanyNo(),carNo);
    }

    //添加车辆信息
    @ApiOperation(value = "添加车辆信息",notes = "添加车辆信息,companyName不用传，需要传递参数：" +
            "   carNo：车牌号，" +
            "   type：车辆类型，" +
            "   img：车辆图片地址，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addCar",method = RequestMethod.POST)
    public Object addCar(@RequestBody AddCarVo resultCarVo, HttpServletRequest request){
        String port =request.getHeader("AccountLanguage");
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(resultCarVo.getCarNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"carNo"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultCarVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"type"));
            return gson.toJson(fb);
        }
        if (carService.getByCarNo(user.getCompanyNo(),resultCarVo.getCarNo())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repetitionCarNo"));
            return gson.toJson(fb);
        }
        resultCarVo.setCompanyNo(user.getCompanyNo());
        int a=carService.addCar(resultCarVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addCarSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addCarFailure"));
        }
        return gson.toJson(fb);
    }

    //修改车辆信息
    @ApiOperation(value = "修改车辆信息",notes = "修改车辆信息,companyName不用传，需要传递参数：" +
            "   oldCarNo：需要修改的预警类型，" +
            "   carNo：车牌号，" +
            "   type：车辆类型，" +
            "   img：车辆图片地址，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateCar",method = RequestMethod.POST)
    public Object updateCar(@RequestBody UpdateCarVo updateCarVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(updateCarVo.getCarNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"carNo"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCarVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"type"));
            return gson.toJson(fb);
        }
        int a=carService.updateCar(updateCarVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"updateCarSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateCarFailure"));
        }
        return gson.toJson(fb);

    }

    //删除车辆信息
    @ApiOperation(value = "删除车辆信息",notes = "删除车辆信息,carNo：需要删除的车牌号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/delCar",method = RequestMethod.POST)
    public Object delCar(@RequestBody String carNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String  port = request.getHeader("AccountLanguage");
        JSONObject jsonObject=JSON.parseObject(carNo);
        carNo=(String)jsonObject.get("carNo");
        if (StringUtils.isEmpty(carNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"车牌号不能为空"));
            return gson.toJson(fb);
        }
        int a=carService.delCar(carNo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deleteCarSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deleteCarFailure"));
        }
        return gson.toJson(fb);
    }


    //获取所有的车牌号
    @ApiOperation(value = "获取所有的车牌号",notes = "获取所有的车牌号,需要传递参数：" +
            "   carNo：车牌号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getTruckNo",method = RequestMethod.POST)
    public Object getTruckNo(@RequestBody String carNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(carNo)){
            fb.setMessage(i18n.getMessage(request,"carNo"));
            fb.setCode(0);
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(carNo);
        carNo=(String)jsonObject.get("carNo");
        return carService.getTruckNo(user.getCompanyNo(),carNo);
    }

}
