package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.common.utils.DateUtil;
import com.csvreader.CsvWriter;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.RealRedisService;

import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.example.tdl.util.CSVUtils;
import com.example.tdl.util.DataUtil;
import com.example.tdl.util.MapUtils;
import com.example.tdl.util.ZipCompressUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;

import org.apache.commons.lang3.StringUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import org.springframework.data.influxdb.InfluxDBTemplate;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.nio.charset.Charset;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.*;

@RestController
public class DataController {
    private Logger logger = LoggerFactory.getLogger(DataController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private RealRedisService realRedisService;

    @Autowired
    private ProbesService probesService;

    @Autowired
    private  AlarmLogService alarmLogService;

    @Autowired
    private I18nController i18n;

    @Value("${spring.influxdb.database}")
    private  String database ;

    private List<String> list = new ArrayList<String>() {{
        add("\"T\"");
        add("\"h\"");
        add("\"p\"");
        add("\"a0\"");
        add("\"ta\"");
    }};

    //获取坐标数据数据
    @ApiOperation(value = "获取坐标数据数据", notes = "获取坐标数据数据:" +
            "   time:时间," +
            "   lng：经度," +
            "   lat: 纬度.")
    @PostMapping("/getLocation")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType = "header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object getLocation(@RequestBody SearchLocationVo searchLocationVo, HttpServletRequest request) {
        UserRedisVo user = gson.fromJson(tokenRedisService.get("TOKEN_" + request.getHeader("Account_token")), UserRedisVo.class);
        if (StringUtils.isEmpty(searchLocationVo.getTransportationNo())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        //根据运输编号获取线路信息
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(searchLocationVo.getTransportationNo(), user.getCompanyNo());
        List<LocationDataVo> locationDataVoList = new ArrayList<>();
        if (resultCircuitVo == null) {
            return gson.toJson(locationDataVoList);
        }
        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
            return gson.toJson(locationDataVoList);
        }
        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
        if(resultCircuitVo.getEndTime() !=null  && infoRedisService.hasHashKey("TransportConfig",searchLocationVo.getTransportationNo())){
            return infoRedisService.getHash("TransportConfig",searchLocationVo.getTransportationNo());
        }
        //获取gateway的信息
        TDLLogVo tdlLogVo = tdlLogService.getByTransportationNo(searchLocationVo.getTransportationNo());
        if (tdlLogVo == null) {
            return gson.toJson(locationDataVoList);
        }
        String device = tdlLogVo.getgType() + "_" + tdlLogVo.getgSN();
        String sql = "";
        if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
            Long time = resultCircuitVo.getEndTime() * 1000000l;
            if ("baidu".equals(searchLocationVo.getType())) {
                sql = "SELECT \"bdlng\",\"bdlat\",\"motion\"  FROM \"tdl_policy\".\"" + device + "\"where  \"data_type\" = 'location' and time >= " + startTime + "  and time<= " + time + "  ORDER BY time ";
            } else {
                sql = "SELECT \"lng\",\"lat\",\"motion\"  FROM \"tdl_policy\".\"" + device + "\"where \"data_type\" = 'location' and  time >= " + startTime + "  and time<= " + time + "  ORDER BY time ";
            }
        } else {
            if ("baidu".equals(searchLocationVo.getType())) {
                                sql = "SELECT \"bdlng\",\"bdlat\",\"motion\"  FROM \"tdl_policy\".\"" + device + "\"where \"data_type\" = 'location' and  time >= " + startTime + "  ORDER BY time ";
//                sql = "SELECT \"bdlng\",\"bdlat\",\"motion\"  FROM \"tdl_policy\".\"" + device + "\"where \"data_type\" = 'location' and  time >= 1552579200000000000  ORDER BY time ";
            } else {
                sql = "SELECT \"lng\",\"lat\",\"motion\"  FROM \"tdl_policy\".\"" + device + "\"where  \"data_type\" = 'location' and time >= " + startTime + "  ORDER BY time ";
            }
        }
        //获取tdl信息
        QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
        //解析经纬度数据
        List<LocationDataVo> locationDataVos = new ArrayList<>();
        if (queryResult.getResults().get(0).getSeries() != null) {
            //博士测试线路：CHN20181016G3039
            locationDataVos =getLocationData(queryResult);
        }else{
            return gson.toJson(locationDataVoList);
        }
        //获取震动的数据
        String sql2 = "";
        if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
            Long time = resultCircuitVo.getEndTime() * 1000000l;
            sql2 = "SELECT \"a1\" FROM \"tdl_policy\".\"" + device + "\"  where \"event\" = 'Shock event'  and  \"a1\" >= 1.5 and  time >= " + startTime + " and time<=" + time + "  ORDER BY time";
        } else {
            sql2 = "SELECT \"a1\" FROM \"tdl_policy\".\"" + device + "\"  where \"event\" = 'Shock event'  and  \"a1\" >= 1.5 and  time >= " + startTime + " ORDER BY time";
        }
        QueryResult queryResult2 = influxDBTemplate.query(new Query(sql2, database));
        List<String> timeList = new ArrayList<>();
        if (queryResult2.getResults().get(0).getSeries() != null) {
            //获取震动的时间戳
            timeList = getTime(queryResult2);
        }
        if (timeList.size() > 0) {
            //获取震动数据的坐标
            String sql3 = "";
            for (String time : timeList) {
                Long t = Long.valueOf(time) * 1000000l;
                if ("baidu".equals(searchLocationVo.getType())) {
                    sql3 = "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\"" + device + "\" where \"data_type\" = 'location' and  time >= " + startTime + " and time<=" + t + " ORDER BY time desc limit 1;" + sql3;
                } else {
                    sql3 = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\" where  \"data_type\" = 'location' and time >= " + startTime + " and time<=" + t + " ORDER BY time desc limit 1;" + sql3;
                }
            }
            QueryResult queryResult3 = influxDBTemplate.query(new Query(sql3, database));
            List<String> time = new ArrayList<>();
            if (queryResult3.getResults() != null) {
                time = getShockLocation(queryResult3);
            }
            for (LocationDataVo locationDataVo : locationDataVos) {
                if (time.contains(locationDataVo.getTime())) {
                    locationDataVo.setError(1);
                    locationDataVoList.add(locationDataVo);
                } else {
                    locationDataVo.setError(0);
                    locationDataVoList.add(locationDataVo);
                }
            }
            return gson.toJson(locationDataVoList);
        } else {
            //如果是已经完结保存到redis中
            return gson.toJson(locationDataVos);
        }
    }


    //1-温度，2-湿度，3-震动，4-倾角
    @ApiOperation(value = "获取温度数据", notes = "获取温度数据:" +
            "   TDLSN:tdl编号," +
            "   temperatureDataVos:," +
            "   time:时间," +
            "   value:温度数据.")
    @PostMapping("/getTemperature")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getTemperature(@RequestBody String transportationNo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        return DataUtil.getData(transportationNo,request,i18n,user.getCompanyNo(),circuitService,tdlLogService,influxDBTemplate,database,"\"T\"","Regular measurement",alarmLogService,"1");
    }



    @ApiOperation(value = "获取温度数据", notes = "获取温度数据:" +
            "   TDLSN:tdl编号," +
            "   temperatureDataVos:," +
            "   time:时间," +
            "   value:温度数据.")
    @PostMapping("/getHumidity")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getHumidity(@RequestBody String transportationNo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        return DataUtil.getData(transportationNo,request,i18n,user.getCompanyNo(),circuitService,tdlLogService,influxDBTemplate,database,"\"h\"","Regular measurement",alarmLogService,"2");
    }



    //获取倾斜度数据
    @ApiOperation(value = "获取倾斜度数据", notes = "获取倾斜度数据")
    @PostMapping("/getTiltAngle")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getTiltAngle(@RequestBody String transportationNo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        return DataUtil.getData(transportationNo,request,i18n,user.getCompanyNo(),circuitService,tdlLogService,influxDBTemplate,database,"\"ta\"","TIlt event",alarmLogService,"4");
    }


    //获取震动能量
    @ApiOperation(value = "获取震动能量", notes = "获取震动能量")
    @PostMapping("/getShockEnergy")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getShockEnergy(@RequestBody String transportationNo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        return DataUtil.getData(transportationNo,request,i18n,user.getCompanyNo(),circuitService,tdlLogService,influxDBTemplate,database,"\"a0\"","Shock event",alarmLogService,"");
    }


    @ApiOperation(value = "获取震动", notes = "获取震动")
    @PostMapping("/getShock")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getShock(@RequestBody String transportationNo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        return DataUtil.getData(transportationNo,request,i18n,user.getCompanyNo(),circuitService,tdlLogService,influxDBTemplate,database,"\"a1\"","Shock event",alarmLogService,"3");
    }


    @ApiOperation(value = "获取压力数据", notes = "获取压力数据")
    @PostMapping("/getPressure")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getPressure(@RequestBody String transportationNo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        return DataUtil.getData(transportationNo,request,i18n,user.getCompanyNo(),circuitService,tdlLogService,influxDBTemplate,database,"\"p\"","Regular measurement",alarmLogService,"");
    }


    @ApiOperation(value = "获取压力和海拔数据", notes = "获取压力和海拔数据")
    @PostMapping("/getABBPressure")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getABBPressure(@RequestBody String transportationNo, HttpServletRequest request) {
        UserRedisVo user = gson.fromJson(tokenRedisService.get("TOKEN_" + request.getHeader("Account_token")), UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject = JSON.parseObject(transportationNo);
        transportationNo = jsonObject.getString("transportationNo");
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo, user.getCompanyNo());
        if (resultCircuitVo == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "notStarted"));
            return gson.toJson(fb);
        }
        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo);
        if (gatewayVo == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        String device = gatewayVo.getgType() + "_" + gatewayVo.getgSN();
        //获取tdl信息
        List<String> devList = tdlLogService.getTdlSN(transportationNo);
        if (devList.size() == 0) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        List<AltitudeResultDataVo> historyDataVoList = new ArrayList<>();
        for (int i = 0; i < devList.size(); i++) {
            AltitudeResultDataVo historyDataVo = new AltitudeResultDataVo();
            String TDLSN = devList.get(i).replace("TDL-", "").trim();
            String sqls = "";
            if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
                sqls = "SELECT count(\"p\") FROM \"tdl_policy\".\"" + device + "\" WHERE  \"event\" = 'Regular measurement' and   \"tdl\"='" + devList.get(i) + "'  and time>=" + startTime + " and time <=" + resultCircuitVo.getEndTime() * 1000000l;
            } else {
                sqls = "SELECT count(\"p\")  FROM \"tdl_policy\".\"" + device + "\" WHERE \"event\" = 'Regular measurement' and \"tdl\"='" + devList.get(i) + "'  and time>=" + startTime ;
            }
            QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
            Integer count;
            if (queryResults.getResults().get(0).getSeries() != null) {
                count = Double.valueOf(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString()).intValue();
            } else {
                historyDataVo.setTDLSN(TDLSN);
                historyDataVoList.add(historyDataVo);
                continue;
            }
            String sql = "";
            if (count > 500) {
                if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
                    sql = DataUtil.getSQL( gatewayVo.getgSN(),"p", device, devList.get(i),  "Regular measurement",startTime, resultCircuitVo.getEndTime() * 1000000l);
                } else {
                    sql = DataUtil.getSQL(gatewayVo.getgSN(), "p", device, devList.get(i),"Regular measurement", startTime, System.currentTimeMillis() * 1000000l);
                }
                QueryResult queryResult = influxDBTemplate.query(new Query(sql, database,false));
                if (queryResult.getResults() != null) {
                    historyDataVo = getHistoryDatas(queryResult);
                }
                historyDataVo.setTDLSN(TDLSN);
                historyDataVoList.add(historyDataVo);
            } else {
                if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
                    sql = "SELECT \"p\" FROM \"tdl_policy\".\"" + device + "\" where \"data_type\" = 'data' AND \"tdl\"='" + devList.get(i) + "'  and time >=" + startTime + "  and time<=" + resultCircuitVo.getEndTime() * 1000000l ;
                } else {
                    sql = "SELECT \"p\" FROM \"tdl_policy\".\"" + device + "\" where \"data_type\" = 'data' AND \"tdl\"='" + devList.get(i) + "'  and time >=" + startTime ;
                }
                QueryResult queryResult = influxDBTemplate.query(new Query(sql, database, true));
                if (queryResult.getResults().get(0).getSeries() != null) {
                    historyDataVo = getHistoryDatas(queryResult);
                }
                historyDataVo.setTDLSN(TDLSN);
                historyDataVoList.add(historyDataVo);
            }
        }
        return historyDataVoList;
    }


    //下载数据
    @ApiOperation(value = "下载线路数据", notes = "下载线路数据")
    @PostMapping("/downLoad")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType = "header", name = "AccountLanguage", value = "language", required = true, dataType = "String"),
    })
    public String downLoad(@RequestBody TransportationNoVo transportationNo, HttpServletRequest request){
        UserRedisVo user = gson.fromJson(tokenRedisService.get("TOKEN_" + request.getHeader("Account_token")), UserRedisVo.class);
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo.getTransportationNo(), user.getCompanyNo());
        if (resultCircuitVo == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(transportationNo.getOffset())){
            transportationNo.setOffset("8");
        }
        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "notStarted"));
            return gson.toJson(fb);
        }
        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo.getTransportationNo());
        if (gatewayVo == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        String device = gatewayVo.getgType() + "_" + gatewayVo.getgSN();
        //获取tdl信息
        List<String> devList = tdlLogService.getTdlSN(transportationNo.getTransportationNo());
        if (devList.size() == 0) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();

        List<Object> head ;
        if("cn".equals(request.getHeader("AccountLanguage"))){
            head  = new ArrayList<Object>(){{
                add("时间");
                add("压力(MPa)");
                add("温度(℃)");
                add("湿度(%RH)");
                add("震动能量(w)");
                add("震动强度(g)");
                add("倾斜角度(°)");
                add("经度");
                add("纬度");
            }};
        }else if("jp".equals(request.getHeader("AccountLanguage"))){
            head  = new ArrayList<Object>(){{
                add("时间");
                add("压力(MPa)");
                add("温度(℃)");
                add("湿度(%RH)");
                add("振動エネルギー(w)");
                add("振動(g)");
                add("回転(°)");
                add("経度");
                add("緯度");
            }};
        }else{
            head  = new ArrayList<Object>(){{
                add("Time");
                add("Pressure(MPa)");
                add("Temperature(℃)");
                add("Humidity(%RH)");
                add("Shock Intensity(w)");
                add("Shock(g)");
                add("Tilt(°)");
                add("Longitude");
                add("Latitude");
            }};
        }
        String sqls = "";
        if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
            sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a0\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +resultCircuitVo.getEndTime() * 1000000l+" ORDER BY time ";
        } else {
            sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a0\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +System.currentTimeMillis() * 1000000l+" ORDER BY time ";
        }
        QueryResult queryResult = influxDBTemplate.query(new Query(sqls, database));
        File myPath = new File( "./export/" );
        if ( !myPath.exists()){//若此目录不存在，则创建之
            myPath.mkdir();
        }
        File[] srcFiles = new File[devList.size()];
        File zipFile = new File("./export/" +transportationNo.getTransportationNo()+i18n.getMessage(request,"data")+".zip");
        if (queryResult.getResults().get(0).getSeries().get(0) != null) {
            for(int  i = 0;i<devList.size();i++){
                String address = "./export/";
                String fileName = devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data");
                List<List<Object>> values =  new ArrayList<>();
                for(List<Object> value : DataUtil.getValues(queryResult)){
                    if(value.get(1) == null || value.get(1).toString().equals(devList.get(i))){
                        List<Object> data = new ArrayList<>();
                        for (int j = 0; j < head.size(); j++) {
                            if(j==0){
                                data.add(timeStamp2Date(new Long(DataUtil.UTCToCST(String.valueOf(value.get(j)),transportationNo.getOffset()))));

                            }else {
                                data.add(value.get(j+1) == null ? "-" :value.get(j+1).toString());
                            }
                        }
                        values.add(data);
                    }
                }
                CSVUtils.createCSVFile(head,values,address,fileName);
                srcFiles[i]=new File("./export/"+devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data")+".csv");
            }
        }
        ZipCompressUtil.zipFiles(srcFiles,zipFile);
        try {
            String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
            if (md5Key != null) {
                for(int a =0;a<srcFiles.length;a++){
                    srcFiles[a].delete();
                }
                zipFile.delete();
                fb.setCode(1);
                fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/circuit/"+transportationNo.getTransportationNo()+i18n.getMessage(request,"data")+".zip");
                return gson.toJson(fb);
            } else {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"fileUpload"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
        }
        return  gson.toJson(fb);
    }




    //下载数据
    @ApiOperation(value = "abb下载线路数据", notes = "abb下载线路数据")
    @PostMapping("/downLoadfForABB")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType = "header", name = "AccountLanguage", value = "language", required = true, dataType = "String"),
    })
    public String downLoadfForABB(@RequestBody TransportationNoVo transportationNo, HttpServletRequest request){
        UserRedisVo user = gson.fromJson(tokenRedisService.get("TOKEN_" + request.getHeader("Account_token")), UserRedisVo.class);
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo.getTransportationNo(), user.getCompanyNo());
        if (resultCircuitVo == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(transportationNo.getOffset())){
            transportationNo.setOffset("8");
        }
        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "notStarted"));
            return gson.toJson(fb);
        }
        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo.getTransportationNo());
        if (gatewayVo == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        String device = gatewayVo.getgType() + "_" + gatewayVo.getgSN();
        //获取tdl信息
        List<String> devList = tdlLogService.getTdlSN(transportationNo.getTransportationNo());
        if (devList.size() == 0) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request, "noCircuit"));
            return gson.toJson(fb);
        }
        OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();

        List<Object> head ;
        if("cn".equals(request.getHeader("AccountLanguage"))){
            head  = new ArrayList<Object>(){{
                add("时间");
                add("压力(MPa)");
                add("海拔(m)");
                add("温度(℃)");
                add("湿度(%RH)");
                add("震动强度(g)");
                add("倾斜角度(°)");
                add("经度");
                add("纬度");
            }};
        }else if("jp".equals(request.getHeader("AccountLanguage"))){
            head  = new ArrayList<Object>(){{
                add("时间");
                add("压力(MPa)");
                add("海抜(m)");
                add("温度(℃)");
                add("湿度(%RH)");
                add("振動(g)");
                add("回転(°)");
                add("経度");
                add("緯度");
            }};
        }else{
            head  = new ArrayList<Object>(){{
                add("Time");
                add("Pressure(MPa)");
                add("Altitude(m)");
                add("Temperature(℃)");
                add("Humidity(%RH)");
                add("Shock(g)");
                add("Tilt(°)");
                add("Longitude");
                add("Latitude");
            }};
        }
        DecimalFormat df = new DecimalFormat("0.00");
        df.getRoundingMode();
        String sqls = "";
        if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
            sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +resultCircuitVo.getEndTime() * 1000000l+" ORDER BY time ";
        } else {
            sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +System.currentTimeMillis() * 1000000l+" ORDER BY time ";
        }
        QueryResult queryResult = influxDBTemplate.query(new Query(sqls, database));
        File[] srcFiles = new File[devList.size()];
        File myPath = new File( "./export/" );
        if ( !myPath.exists()){//若此目录不存在，则创建之
            myPath.mkdir();
        }
        File zipFile = new File("./export/" +transportationNo.getTransportationNo()+"_"+gatewayVo.getgSN()+i18n.getMessage(request,"data")+".zip");
        if (queryResult.getResults().get(0).getSeries().get(0) != null) {
            for(int  i = 0;i<devList.size();i++){
                String address = "./export/";
                String fileName = devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data");
                List<List<Object>> values =  new ArrayList<>();
                for(List<Object> value : DataUtil.getValues(queryResult)){
                    if(value.get(1) == null || value.get(1).toString().equals(devList.get(i))){
                        List<Object> data = new ArrayList<>();
                        for (int j = 0; j < head.size(); j++) {
                            if(j==0){
                                try {
                                    data.add(timeStamp2Date(new Long(DataUtil.UTCToCST(String.valueOf(value.get(j)),transportationNo.getOffset()))));
                                } catch (Exception e) {
                                    e.printStackTrace();
                                    logger.info(e.toString());
                                    continue;
                                }
                            }else if(j ==1) {
                                data.add(value.get(j+1) == null ? "-" :value.get(j+1).toString());
                            }else if(j ==2){
                                Double altitude = value.get(2) == null ? null  : 44300*(1-(Math.pow((Double.parseDouble(df.format(Float.valueOf(value.get(2).toString())))/1013.25), (float)1/5.256)));
                                data.add( altitude == null ? "-" : altitude.toString());
                            }else{
                                data.add(value.get(j) == null ? "-" :value.get(j).toString());
                            }
                        }
                        values.add(data);
                    }
                }
                CSVUtils.createCSVFile(head,values,address,fileName);
                srcFiles[i]=new File("./export/"+devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data")+".csv");
            }
        }
        ZipCompressUtil.zipFiles(srcFiles,zipFile);
        try {
            String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
            if (md5Key != null) {
                for(int a =0;a<srcFiles.length;a++){
                    srcFiles[a].delete();
                }
                zipFile.delete();
                fb.setCode(1);
                fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/circuit/"+transportationNo.getTransportationNo() + "_" +gatewayVo.getgSN() +i18n.getMessage(request,"data")+".zip");
                return gson.toJson(fb);
            } else {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"fileUpload"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
        }
        return  gson.toJson(fb);
    }



    //解析数据
    public List<LocationDataVo> getLocation(QueryResult queryResult) {
        List<List<Object>> values = DataUtil.getValues(queryResult);
        List<LocationDataVo> historyDataList = new ArrayList<>();
        for (List<Object> value : values) {
            LocationDataVo historyData = new LocationDataVo();
            if (value.get(1).toString().equals("0.0")) {
                continue;
            } else {
                historyData.setTime(DataUtil.parseTime(value.get(0).toString()));
                historyData.setLng(Double.parseDouble(value.get(1).toString()));
                historyData.setLat(Double.parseDouble(value.get(2).toString()));
                historyData.setError(0);
            }
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    public  List<LocationDataVo> getLocationData(QueryResult queryResult){
        List<List<Object>> values = DataUtil.getValues(queryResult);
        List<LocationDataVo> historyDataList = new ArrayList<>();
        for (int i =0,length = values.size();i<length;i++) {
            LocationDataVo historyData = new  LocationDataVo();
            Integer motion = values.get(i).get(3) == null ? 0 :Double.valueOf(values.get(i).get(3).toString()).intValue();
            if(i == 0){
                historyData.setTime(DataUtil.UTCToCST(values.get(i).get(0).toString(),"+8"));
                historyData.setLng(Double.parseDouble(values.get(i).get(1).toString()));
                historyData.setLat(Double.parseDouble(values.get(i).get(2).toString()));
                historyData.setError(0);
            }else{
                if(motion == 2){ //静止状态
                    String time = DataUtil.UTCToCST(values.get(i).get(0).toString(),"+8");
                    historyDataList.get(historyDataList.size()-1).setTime(time);
                }else{
                    String time1 = DataUtil.UTCToCST(values.get(i).get(0).toString(),"+8");
                    String time2 = DataUtil.UTCToCST(values.get(i-1).get(0).toString(),"+8");
                    Double lng1 = Double.parseDouble(values.get(i).get(1).toString());
                    Double lat1 = Double.parseDouble(values.get(i).get(2).toString());
                    Double lng2 = Double.parseDouble(values.get(i-1).get(1).toString());
                    Double lat2 = Double.parseDouble(values.get(i-1).get(2).toString());
                    Double speed=0d;
                    if(motion == 1){
                        speed   = MapUtils.getTransSpeed(lat1, lng1,lat2,lng2,(Long.valueOf(time1)-Long.valueOf(time2))/1000);
                    }else if(motion == 0){
                        speed   = MapUtils.getSpeed(lat1, lng1,lat2,lng2,(Long.valueOf(time1)-Long.valueOf(time2))/1000,3.3f);
                    }
                    if(speed >0){
                        historyData.setTime(time1);
                        historyData.setLng(lng1);
                        historyData.setLat(lat1);
                        historyData.setError(0);
                    }else{
                        LocationDataVo historyDatas =   historyDataList.get(historyDataList.size()-1);
                        historyDataList.remove(historyDataList.size()-1);
                        historyDatas.setTime(time1);
                        historyDataList.add(historyDatas);
                        continue;
                    }
                }
            }
            historyDataList.add(historyData);
        }
        return  historyDataList;
    }

    public  List<LocationDataVo> getData(QueryResult queryResult) {
        List<List<Object>> values = DataUtil.getValues(queryResult);
        Float sppeed = 3.3f;
        if(queryResult.getResults().get(0).getSeries().get(0).getName().contains("17110202")){
            sppeed =1.7f;
        }
        List<LocationDataVo> historyDataList = new ArrayList<>();
        for (int i =0,length =  values.size();i<length;i++) {
            LocationDataVo historyData = new LocationDataVo();
            if(values.size()<5){
                if (values.get(i).get(1).toString().equals("0.0")) {
                    continue;
                } else {
                    historyData.setTime(DataUtil.parseTime(values.get(i).get(0).toString()));
                    historyData.setLng(Double.parseDouble(values.get(i).get(1).toString()));
                    historyData.setLat(Double.parseDouble(values.get(i).get(2).toString()));
                    historyData.setError(0);
                }
            }else{
                if(i ==  0){
                    String time1 = DataUtil.parseTime(values.get(i).get(0).toString());
                    Double lng1 = Double.parseDouble(values.get(i).get(1).toString());
                    Double lat1 = Double.parseDouble(values.get(i).get(2).toString());
                    historyData.setTime(time1);
                    historyData.setLng(lng1);
                    historyData.setLat(lat1);
                }else{
                    String time1 = DataUtil.parseTime(values.get(i).get(0).toString());
                    String time2 = DataUtil.parseTime(values.get(i-1).get(0).toString());
                    Double lng1 = Double.parseDouble(values.get(i).get(1).toString());
                    Double lat1 = Double.parseDouble(values.get(i).get(2).toString());
                    Double lng2 = Double.parseDouble(values.get(i-1).get(1).toString());
                    Double lat2 = Double.parseDouble(values.get(i-1).get(2).toString());
                    Double speed2   = MapUtils.getSpeed(lat1, lng1,lat2,lng2,(Long.valueOf(time1)-Long.valueOf(time2))/1000,sppeed);
                    if(speed2 >0){
                        historyData.setTime(time1);
                        historyData.setLng(lng1);
                        historyData.setLat(lat1);
                    }else{
                        historyDataList.remove(historyDataList.size()-1);
                        historyData.setTime(time1);
                        historyData.setLng(lng1);
                        historyData.setLat(lat1);
                        historyDataList.add(historyData);
                        continue;
                    }
                }
            }

            historyDataList.add(historyData);
        }
        if(historyDataList.size() == 0){
            LocationDataVo historyData = new LocationDataVo();
            historyData.setTime(DataUtil.parseTime(values.get(0).get(0).toString()));
            historyData.setLng(Double.parseDouble(values.get(0).get(1).toString()));
            historyData.setLat(Double.parseDouble(values.get(0).get(2).toString()));
            historyDataList.add(historyData);
        }
        logger.info(gson.toJson(historyDataList));
        return  historyDataList;
    }

    public static List<List<Object>> getValues(QueryResult queryResult) {
        List<List<Object>> values = new ArrayList<>();
        for (QueryResult.Result result : queryResult.getResults()) {
            if (result.getSeries() != null) {
                for (QueryResult.Series series : result.getSeries()) {
                    if (series != null) {
                        values=series.getValues();
                    }
                }
            }
        }
        return values;
    }

    public static AltitudeResultDataVo getHistoryDatas(QueryResult queryResult) {
        String dateResult = null;
        List<List<Object>> value = DataUtil.getValues(queryResult);
        AltitudeResultDataVo altitudeResultDataVo = new AltitudeResultDataVo();
        List<HistoryData> pressureDataList = new ArrayList<>();
        List<HistoryData> altitudeDataList = new ArrayList<>();
        DecimalFormat df = new DecimalFormat("0.00");
        df.getRoundingMode();
        for (int i = 0; i < value.size(); i++) {
            HistoryData pressureData = new HistoryData();
            String time= DataUtil.parseTime(value.get(i).get(0).toString());
            pressureData.setTime(time);
            String press = value.get(i).get(1).toString();
            pressureData.setValue(press);
            HistoryData altitudeData = new HistoryData();
            altitudeData.setTime(time);
            //大气压力（kPa A) =101.325*(1-0.02257*海拔)^5.256
            //            Double height = 44300*(1-(Math.pow((Double.parseDouble(df.format(Float.valueOf(press)))/1013.25), (float)1/5.256)));
            Double height = 44300*(1-(Math.pow((Double.parseDouble(df.format(Float.valueOf(press)-10f))/1013.25), (float)1/5.256))); //气压临时减去10
            altitudeData.setValue(df.format(height).toString());
            pressureDataList.add(pressureData);
            altitudeDataList.add(altitudeData);
        }
        altitudeResultDataVo.setPressure(pressureDataList);
        altitudeResultDataVo.setAltitude(altitudeDataList);
        return altitudeResultDataVo;
    }

    //判断两个时间相差多少
    public static String getDistanceTime(Long time1, Long time2) {
        //获取当前时区，将时间戳转化成日期时间格式
        Date t1 = new Date(time1);
        Date t2 = new Date(time2);
        int days = differentDays(t1, t2);
        if (days > 0) {
            return days + "天";
        } else {
            return "a little minute";
        }
    }

    public static int differentDays(Date date1, Date date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);

        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        int day1 = cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        if (year1 != year2) { //不同年
            int timeDistance = 0;
            for (int i = year1; i < year2; i++) {
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0) {   //闰年
                    timeDistance += 366;
                } else {    //不是闰年
                    timeDistance += 365;
                }
            }
            return timeDistance + (day2 - day1);
        } else {
            return day2 - day1;
        }
    }

    //将时间戳转成时间
    public static String timeStamp2Date(Long seconds) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(new Date(seconds));
    }


    public static String timeStampDate(Long seconds) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH 时mm分ss秒");
        return sdf.format(new Date(seconds));
    }

    //解析震动时间
    public List<String> getTime(QueryResult queryResult) {
        List<List<Object>> values = DataUtil.getValues(queryResult);
        List<String> time = new ArrayList<>();
        for(int i = 0,length=values.size();i<length;i++){
            String ts = DataUtil.UTCToCST(values.get(i).get(0).toString(),"+8");
            if(i == 0){
                time.add(ts);
            }else{
                if(Long.valueOf(ts)-Long.valueOf(DataUtil.UTCToCST(values.get(i-1).get(0).toString(),"+8"))>60*1000){
                    time.add(ts);
                }else{
                    continue;
                }
            }
        }
        return time;
    }


    public List<String> getShockLocation(QueryResult queryResult) {
        List<List<Object>> values = new ArrayList<>();
        for (QueryResult.Result result : queryResult.getResults()) {
            if (result.getSeries() != null) {
                for (QueryResult.Series series : result.getSeries()) {
                    if (series != null) {
                        values.add(series.getValues().get(0));
                    }
                }
            }
        }
        List<String> timeList = new ArrayList<>();
        for (List<Object> value : values) {
            timeList.add(DataUtil.parseTime(value.get(0).toString()));
        }
        return timeList;
    }


    public List<DeviceDataVo> getDeviceData(QueryResult queryResult) {
        List<List<Object>> values = DataUtil.getValues(queryResult);
        List<DeviceDataVo> historyDataList = new ArrayList<>();
        for (List<Object> value : values) {
            DeviceDataVo historyData = new DeviceDataVo();
            historyData.setTime(DataUtil.parseTime(value.get(0).toString()));
            historyData.setTemperature(value.get(1) == null ? "" : value.get(1).toString());
            historyData.setHumidity(value.get(2) == null ? "" : value.get(2).toString());
            historyData.setPressure(value.get(3) == null ? "" : value.get(3).toString());
            historyData.setShockEnergy(value.get(4) == null ? "" : value.get(4).toString());
            historyData.setTiltAngle(value.get(5) == null ? "" : value.get(5).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }


}


//    //下载数据
//    @ApiOperation(value = "下载线路数据", notes = "下载线路数据")
//    @PostMapping("/downLoad")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
//            @ApiImplicitParam(paramType = "header", name = "AccountLanguage", value = "language", required = true, dataType = "String"),
//    })
//    public String downLoad(@RequestBody TransportationNoVo transportationNo, HttpServletRequest request){
//        UserRedisVo user = gson.fromJson(tokenRedisService.get("TOKEN_" + request.getHeader("Account_token")), UserRedisVo.class);
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo.getTransportationNo(), user.getCompanyNo());
//        if (resultCircuitVo == null) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        if(StringUtils.isEmpty(transportationNo.getOffset())){
//            transportationNo.setOffset("8");
//        }
//        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "notStarted"));
//            return gson.toJson(fb);
//        }
//        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
//        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo.getTransportationNo());
//        if (gatewayVo == null) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        String device = gatewayVo.getgType() + "_" + gatewayVo.getgSN();
//        //获取tdl信息
//        List<String> devList = tdlLogService.getTdlSN(transportationNo.getTransportationNo());
//        if (devList.size() == 0) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
//
//        List<Object> head ;
//        if("cn".equals(request.getHeader("AccountLanguage"))){
//            head  = new ArrayList<Object>(){{
//                add("时间");
//                add("压力(MPa)");
//                add("温度(℃)");
//                add("湿度(%RH)");
//                add("震动能量(w)");
//                add("震动强度(g)");
//                add("倾斜角度(°)");
//                add("经度");
//                add("纬度");
//            }};
//        }else if("jp".equals(request.getHeader("AccountLanguage"))){
//            head  = new ArrayList<Object>(){{
//                add("时间");
//                add("压力(MPa)");
//                add("温度(℃)");
//                add("湿度(%RH)");
//                add("振動エネルギー(w)");
//                add("振動(g)");
//                add("回転(°)");
//                add("経度");
//                add("緯度");
//            }};
//        }else{
//            head  = new ArrayList<Object>(){{
//                add("Time");
//                add("Pressure(MPa)");
//                add("Temperature(℃)");
//                add("Humidity(%RH)");
//                add("Shock Intensity(w)");
//                add("Shock(g)");
//                add("Tilt(°)");
//                add("Longitude");
//                add("Latitude");
//            }};
//        }
//
//        String[] rowsName;
//        if("cn".equals(request.getHeader("AccountLanguage"))){
//            rowsName = new String[]{"时间","压力(MPa)","温度(℃)","湿度(%RH)","震动能量(w)","震动强度(g)","倾斜角度(°)","经度","纬度"};
//        }else if("jp".equals(request.getHeader("AccountLanguage"))){
//            rowsName = new String[]{"时间","圧力(MPa)","温度(℃)","湿度(%RH)","振動エネルギー(w)","振動(g)","回転(°)","経度","緯度"};
//        }else{
//            rowsName = new String[]{"Time","Pressure(MPa)","Temperature(℃)","Humidity(%RH)","Shock Intensity(w)","Shock(g)","Tilt(°)","Longitude","Latitude"};
//
//        }
//        File file = new File("./export");
//        if(!file.exists()){
//            file.mkdir();
//        }
//        try {
//            String sqls = "";
//            if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
//                sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a0\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +resultCircuitVo.getEndTime() * 1000000l+" ORDER BY time ";
//            } else {
//                sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a0\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +System.currentTimeMillis() * 1000000l+" ORDER BY time ";
//            }
//            QueryResult queryResult = influxDBTemplate.query(new Query(sqls, database));
//            File[] srcFiles = new File[devList.size()];
//            File zipFile = new File("./export/" +transportationNo.getTransportationNo()+i18n.getMessage(request,"data")+".zip");
//            if (queryResult.getResults().get(0).getSeries().get(0) != null) {
//                for(int  i = 0;i<devList.size();i++){
//                    CsvWriter csvWriter = new CsvWriter("./export/"+devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data")+".csv" , ',', Charset.forName("GBK"));
//                    csvWriter.writeRecord(rowsName);
//                    List<List<Object>> values =  DataUtil.getValues(queryResult);
//                    for(List<Object> value : values){
//                        if(value.get(1) == null || value.get(1).toString().equals(devList.get(i))){
//                            String[] objs = new String[rowsName.length];
//                            for (int j = 0; j < objs.length; j++) {
//                                if(j==0){
//                                    try {
//                                        objs[j] =timeStamp2Date(new Long(DataUtil.UTCToCST(String.valueOf(value.get(j)),transportationNo.getOffset())));
//                                    } catch (Exception e) {
//                                        e.printStackTrace();
//                                        logger.info(e.toString());
//                                        fb.setMessage(i18n.getMessage(request,"timeparse"));
//                                    }
//                                }else {
//                                    objs[j] =value.get(j+1) == null ? "-" :value.get(j+1).toString();
//                                }
//                            }
//                            csvWriter.writeRecord(objs);
//                            csvWriter.flush();
//                        }
//                    }
//                    csvWriter.close();
//                    srcFiles[i]=new File("./export/"+devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data")+".csv");
//                }
//            }
//            ZipCompressUtil.zipFiles(srcFiles,zipFile);
//            try {
//                String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
//                if (md5Key != null) {
//                    for(int a =0;a<srcFiles.length;a++){
//                        srcFiles[a].delete();
//                    }
//                    zipFile.delete();
//                    fb.setCode(1);
//                    fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/circuit/"+transportationNo.getTransportationNo()+i18n.getMessage(request,"data")+".zip");
//                    return gson.toJson(fb);
//                } else {
//                    fb.setCode(0);
//                    fb.setMessage(i18n.getMessage(request,"fileUpload"));
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//                logger.info(e.toString());
//            }
//        }catch (Exception e){
//            e.printStackTrace();
//            logger.info(e.toString());
//        }
//
//        return  gson.toJson(fb);
//    }
//
//
//
//
//    //下载数据
//    @ApiOperation(value = "abb下载线路数据", notes = "abb下载线路数据")
//    @PostMapping("/downLoadfForABB")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
//            @ApiImplicitParam(paramType = "header", name = "AccountLanguage", value = "language", required = true, dataType = "String"),
//    })
//    public String downLoadfForABB(@RequestBody TransportationNoVo transportationNo, HttpServletRequest request){
//        UserRedisVo user = gson.fromJson(tokenRedisService.get("TOKEN_" + request.getHeader("Account_token")), UserRedisVo.class);
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo.getTransportationNo(), user.getCompanyNo());
//        if (resultCircuitVo == null) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        if(StringUtils.isEmpty(transportationNo.getOffset())){
//            transportationNo.setOffset("8");
//        }
//        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "notStarted"));
//            return gson.toJson(fb);
//        }
//        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
//        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo.getTransportationNo());
//        if (gatewayVo == null) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        String device = gatewayVo.getgType() + "_" + gatewayVo.getgSN();
//        //获取tdl信息
//        List<String> devList = tdlLogService.getTdlSN(transportationNo.getTransportationNo());
//        if (devList.size() == 0) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
//        String[] rowsName;
//        if("cn".equals(request.getHeader("AccountLanguage"))){
//            rowsName = new String[]{"时间","圧力(MPa)","海拔(m)","温度(℃)","湿度(%RH)","震动强度(g)","倾斜角度(°)","经度","纬度"};
//        }else if("jp".equals(request.getHeader("AccountLanguage"))){
//            rowsName = new String[]{"时间","圧力(MPa)","海抜(m)","温度(℃)","湿度(%RH)","振動(g)","回転(°)","経度","緯度"};
//        }else{
//            rowsName = new String[]{"Time","Pressure(MPa)","Altitude(m)","Temperature(℃)","Humidity(%RH)","Shock(g)","Tilt(°)","Longitude","Latitude"};
//
//        }
//        File file = new File("./export");
//        if(!file.exists()){
//            file.mkdir();
//        }
//        DecimalFormat df = new DecimalFormat("0.00");
//        df.getRoundingMode();
//        try {
//            String sqls = "";
//            if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
//                sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +resultCircuitVo.getEndTime() * 1000000l+" ORDER BY time ";
//            } else {
//                sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +System.currentTimeMillis() * 1000000l+" ORDER BY time ";
//            }
//            QueryResult queryResult = influxDBTemplate.query(new Query(sqls, database));
//            File[] srcFiles = new File[devList.size()];
//            File zipFile = new File("./export/" +transportationNo.getTransportationNo()+i18n.getMessage(request,"data")+".zip");
//            if (queryResult.getResults().get(0).getSeries().get(0) != null) {
//
//                for(int  i = 0;i<devList.size();i++){
//                    CsvWriter csvWriter = new CsvWriter("./export/"+devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data")+".csv" , ',', Charset.forName("GBK"));
//                    csvWriter.writeRecord(rowsName);
//                    List<List<Object>> values =  DataUtil.getValues(queryResult);
//                    for(List<Object> value : values){
//                        if(value.get(1) == null || value.get(1).toString().equals(devList.get(i))){
//                            String[] objs = new String[rowsName.length];
//                            for (int j = 0; j < objs.length; j++) {
//                                logger.info(j+"=-----"+ rowsName[j]);
//                                if(j==0){
//                                    try {
//                                        objs[j] =timeStamp2Date(new Long(DataUtil.UTCToCST(String.valueOf(value.get(j)),transportationNo.getOffset())));
//                                    } catch (Exception e) {
//                                        e.printStackTrace();
//                                        logger.info(e.toString());
//                                        continue;
//                                    }
//                                }else if(j ==1) {
//                                    objs[j] =value.get(j+1) == null ? "-" :value.get(j+1).toString();
//                                }else if(j ==2){
//                                    Double altitude = value.get(2) == null ? null  : 44300*(1-(Math.pow((Double.parseDouble(df.format(Float.valueOf(value.get(2).toString())))/1013.25), (float)1/5.256)));
//                                    objs[j] = altitude == null ? "-" : altitude.toString();
//                                }else{
//                                    objs[j] =value.get(j) == null ? "-" :value.get(j).toString();
//                                }
//                            }
//                            csvWriter.writeRecord(objs);
//                            csvWriter.flush();
//                        }
//                    }
//                    csvWriter.close();
//                    srcFiles[i]=new File("./export/"+devList.get(i).replace("TDL-", "").trim()+i18n.getMessage(request,"data")+".csv");
//                }
//            }
//            ZipCompressUtil.zipFiles(srcFiles,zipFile);
//            try {
//                String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
//                if (md5Key != null) {
//                    for(int a =0;a<srcFiles.length;a++){
//                        srcFiles[a].delete();
//                    }
//                    zipFile.delete();
//                    fb.setCode(1);
//                    fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/circuit/"+transportationNo.getTransportationNo()+i18n.getMessage(request,"data")+".zip");
//                    return gson.toJson(fb);
//                } else {
//                    fb.setCode(0);
//                    fb.setMessage(i18n.getMessage(request,"fileUpload"));
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//                logger.info(e.toString());
//            }
//        }catch (Exception e){
//            e.printStackTrace();
//            logger.info(e.toString());
//        }
//
//        return  gson.toJson(fb);
//    }


//    @ApiOperation(value = "获取坐标数据数据", notes = "获取坐标数据数据:" +
//            "   time:时间," +
//            "   lng：经度," +
//            "   lat: 纬度.")
//    @PostMapping("/getLocation")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
//            @ApiImplicitParam(paramType = "header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
//    })
//    public Object getLocation(@RequestBody SearchLocationVo searchLocationVo, HttpServletRequest request) {
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" + token);
//        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
//        if (StringUtils.isEmpty(searchLocationVo.getTransportationNo())) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "transportationNo"));
//            return gson.toJson(fb);
//        }
//        //根据运输编号获取线路信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(searchLocationVo.getTransportationNo(), user.getCompanyNo());
//        List<LocationDataVo> locationDataVoList = new ArrayList<>();
//        if (resultCircuitVo == null) {
//            return gson.toJson(locationDataVoList);
//        }
//        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
//            return gson.toJson(locationDataVoList);
//        }
//        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
//        if(resultCircuitVo.getEndTime() !=null  && infoRedisService.hasHashKey("TransportConfig",searchLocationVo.getTransportationNo())){
//            Object ret = infoRedisService.getHash("TransportConfig",searchLocationVo.getTransportationNo());
//            return ret;
//        }
//        //获取gateway的信息
//        TDLLogVo tdlLogVo = tdlLogService.getByTransportationNo(searchLocationVo.getTransportationNo());
//        if (tdlLogVo == null) {
//            return gson.toJson(locationDataVoList);
//        }
//        String device = tdlLogVo.getgType() + "_" + tdlLogVo.getgSN();
//        String sql = "";
//        if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
//            Long time = resultCircuitVo.getEndTime() * 1000000l;
//            if ("baidu".equals(searchLocationVo.getType())) {
//                sql = "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\"" + device + "\"where  \"data_type\" = 'location' and time >= " + startTime + "  and time<= " + time + "  ORDER BY time ";
//            } else {
//                sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"where \"data_type\" = 'location' and  time >= " + startTime + "  and time<= " + time + "  ORDER BY time ";
//            }
//        } else {
//            if ("baidu".equals(searchLocationVo.getType())) {
//                sql = "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\"" + device + "\"where \"data_type\" = 'location' and  time >= " + startTime + "  ORDER BY time ";
//            } else {
//                sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"where  \"data_type\" = 'location' and time >= " + startTime + "  ORDER BY time ";
//            }
//        }
//        //获取tdl信息
//        QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
//        //解析经纬度数据
//        List<LocationDataVo> locationDataVos = new ArrayList<>();
//        if (queryResult.getResults().get(0).getSeries() != null) {
//            //博士测试线路：CHN20181016G3039
//            if(searchLocationVo.getTransportationNo().equals("CHN20181016G3039")){
//                locationDataVos =getLocation(queryResult);
//            }else{
//                locationDataVos = getData(queryResult);
//            }
//        }else{
//            return gson.toJson(locationDataVoList);
//        }
//        //获取震动的数据
//        String sql2 = "";
//        if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
//            Long time = resultCircuitVo.getEndTime() * 1000000l;
//            sql2 = "SELECT \"a1\" FROM \"tdl_policy\".\"" + device + "\"  where \"event\" = 'Shock event'  and  \"a1\" >= 1.5 and  time >= " + startTime + " and time<=" + time + "  ORDER BY time";
//        } else {
//            sql2 = "SELECT \"a1\" FROM \"tdl_policy\".\"" + device + "\"  where \"event\" = 'Shock event'  and  \"a1\" >= 1.5 and  time >= " + startTime + " ORDER BY time";
//        }
//        QueryResult queryResult2 = influxDBTemplate.query(new Query(sql2, database));
//        List<String> timeList = new ArrayList<>();
//        if (queryResult2.getResults().get(0).getSeries() != null) {
//            //获取震动的时间戳
//            timeList = getTime(queryResult2);
//        }
//        if (timeList.size() > 0) {
//            //获取震动数据的坐标
//            String sql3 = "";
//            for (String time : timeList) {
//                Long t = Long.valueOf(time) * 1000000l;
//                if ("baidu".equals(searchLocationVo.getType())) {
//                    sql3 = "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\"" + device + "\" where \"data_type\" = 'location' and  time >= " + startTime + " and time<=" + t + " ORDER BY time desc limit 1;" + sql3;
//                } else {
//                    sql3 = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\" where  \"data_type\" = 'location' and time >= " + startTime + " and time<=" + t + " ORDER BY time desc limit 1;" + sql3;
//                }
//            }
//            QueryResult queryResult3 = influxDBTemplate.query(new Query(sql3, database));
//            List<String> time = new ArrayList<>();
//            if (queryResult3.getResults() != null) {
//                time = getShockLocation(queryResult3);
//            }
//            for (LocationDataVo locationDataVo : locationDataVos) {
//                if (time.contains(locationDataVo.getTime())) {
//                    locationDataVo.setError(1);
//                    locationDataVoList.add(locationDataVo);
//                } else {
//                    locationDataVo.setError(0);
//                    locationDataVoList.add(locationDataVo);
//                }
//            }
//            return gson.toJson(locationDataVoList);
//        } else {
//            //如果是已经完结保存到redis中
//            return gson.toJson(locationDataVos);
//        }
//    }

//    获取历史数据
//    @ApiOperation(value = "获取历史数据",notes = "获取历史数据:")
//    @PostMapping("/getHistoryData")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
//    })
//    public Object getHistoryData(@RequestBody String transportationNo,HttpServletRequest request){
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" +token);
//        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        String port = request.getHeader("AccountLanguage");
//        logger.info("" +System.currentTimeMillis());
//        logger.info("" + Calendar.getInstance().getTimeInMillis());
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        List<DataVo> dataVoList = new ArrayList<>();
//        if(StringUtils.isEmpty(transportationNo)){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"transportationNo"));
//            return gson.toJson(fb);
//        }
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo,user.getCompanyName());
//        if(resultCircuitVo ==null){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"noCircuit"));
//            return gson.toJson(fb);
//        }
//        if(resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"notStarted"));
//            return gson.toJson(fb);
//        }
//        Long startTime = resultCircuitVo.getStartTime() *1000000l;
//        if(resultCircuitVo.getEndTime() !=null && resultCircuitVo.getEndTime() !=0 && infoRedisService.hasHashKey("HistoryData",transportationNo)){
//            return infoRedisService.getHash("HistoryData",transportationNo);
//        }else{
//            TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo);
//            if(gatewayVo == null){
//                fb.setCode(0);
//                fb.setMessage(i18n.getMessage(request,"noCircuit"));
//                return gson.toJson(fb);
//            }
//            String device = gatewayVo.getgType() +"_" +gatewayVo.getgSN();
//            //获取tdl信息
//            List<String>  devList= tdlLogService.getTdlSN(transportationNo);
//            if(devList.size() == 0){
//                fb.setCode(0);
//                fb.setMessage(i18n.getMessage(request,"noCircuit"));
//                return gson.toJson(fb);
//            }
//            Map<String,String> map = new HashMap<>();
//            map.put("\"T\"","temp");
//            map.put("\"h\"","humidity");
//            map.put("\"p\"","pressure");
//            map.put("\"a0\"","shockEnergy");
//            map.put("\"ta\"","tilt");
//            for(int j = 0;j< list.size();j++){
//                DataVo dataVo = new DataVo();
//                dataVo.setDescription(map.get(list.get(j)));
//                List<HistoryDataVo> historyDataVoList = new ArrayList<>();
//                for( int i =0;i<devList.size();i++){
//                    HistoryDataVo historyDataVo = new HistoryDataVo();
//                    String TDLSN=devList.get(i).replace("TDL-","").trim();
//                    historyDataVo.setTDLSN(TDLSN);
//                   ResultAlarmVo resultAlarmVo =  tdlLogService.getByTdl(transportationNo,TDLSN);
//                    if(list.get(j).equals("\"T\"") ){
//                        //根据tdl货物报警信息
//                        dataVo.setMax(resultAlarmVo.getTemMax().toString());
//                        dataVo.setMin(resultAlarmVo.getTemMin().toString());
//                    }else if(list.get(j).equals("\"h\"")){
//                        dataVo.setMax(resultAlarmVo.getHumidityMax().toString());
//                        dataVo.setMin(resultAlarmVo.getHumidityMin().toString());
//                    }
//                    String sql="";
//                    if(resultCircuitVo.getEndTime() !=null && resultCircuitVo.getEndTime() !=0){
//                        sql = "SELECT "+list.get(j)+" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"='"+devList.get(i)+"'  and time >="+startTime + "  and time<="+resultCircuitVo.getEndTime() *1000000l+" ORDER BY time ";
//                    }else{
//                        sql = "SELECT "+list.get(j)+" FROM \"tdl_policy\".\""+device+"\" where \"tdl\"='"+devList.get(i)+"'  and time >="+startTime + " ORDER BY time ";
//                    }
//                    QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//                    List<HistoryData>  historyData = new ArrayList<>();
//                    if(queryResult.getResults().get(0).getSeries() != null){
//                        //解析数据
//                        historyData = getHistoryData(queryResult,list.get(j));
//                    }
//                    historyDataVo.setDataList(historyData);
//                    historyDataVoList.add(historyDataVo);
//                }
//                dataVo.setDataList(historyDataVoList);
//                dataVoList.add(dataVo);
//            }
//            if(resultCircuitVo.getEndTime()!=null && resultCircuitVo.getEndTime()!=0){
//                Map<Object,Object> m = new HashMap<>();
//                map.put(transportationNo,dataVoList.toString());
//                infoRedisService.setHash("HistoryData",m,System.currentTimeMillis());
//            }
//            return  gson.toJson(dataVoList);
//        }
//    }

//    根据运输编号获取路线以及设备信息
//    @ApiOperation(value = "获取路线以及设备信息",notes = "获取路线以及设备信息" +
//            "transportationNo:运输编号" +
//            "startCity:启运城市" +
//            "startTime:开始时间" +
//            "endCity:目的城市" +
//            "endTime:结束时间（未结束为\"\"）" +
//            "gatewaySN:网关编号" +
//            "battery:网关点量" +
//            "TDLSN:tdl编号" +
//            "batteryVoltage:tdl电压" +
//            "lastTime：运输时长（未结束为\"\"）" +
//            "circuitState:运输状态(0--未开始，1--运输中，2--完成)" +
//            "remark：备注信息")
//    @PostMapping("/getCircuit")
//    @ApiImplicitParams({
//        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public Object  getCircuit(@RequestBody String transportationNo,HttpServletRequest request){
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" +token);
//        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        ResultCircuit resultCircuit = new ResultCircuit();
//        resultCircuit.setTransportationNo(transportationNo);
//        //根据运输编号获取线路信息，设备信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo,user.getCompanyName());
//        resultCircuit.setStartCity(resultCircuitVo.getStartCity());
//        resultCircuit.setStartTime(resultCircuitVo.getStartTime());
//        resultCircuit.setEndCity(resultCircuitVo.getEndCity());
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(resultCircuitVo.getEndTime() == 0){
//            resultCircuit.setEndTime(null);
//            resultCircuit.setLastTime(null);
//            gatewayVo = circuitService.getByNo(transportationNo);
//        }else{
//            resultCircuit.setEndTime(resultCircuitVo.getEndTime());
//            resultCircuit.setLastTime(getDistanceTime(resultCircuit.getStartTime(),resultCircuit.getEndTime()));
//            gatewayVo = tdlLogService.getByNo(transportationNo);
//        }
//        //获取gateway的信息
//        if(gatewayVo==null){
//            fb.setCode(0);
//            fb.setMessage("该线路未绑定设备");
//            return gson.toJson(fb);
//        }
//        resultCircuit.setGatewaySN(gatewayVo.getSN());
//        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
//        Long startTime = resultCircuitVo.getStartTime() *1000000l;
//        //获取tdl信息
//        String sql = "SELECT \"battery\"  FROM \"tdl_policy\".\""+device+"\" where time >= "+startTime +"  ORDER BY time  desc limit 1";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        if(queryResult.getResults().get(0).getSeries() != null){
//            List<List<Object>> values = getValues(queryResult);
//            for(List<Object> value:values){
//                logger.info(value.get(1).toString());
//                resultCircuit.setBattery(value.get(1)==null ?  "": value.get(1).toString());
//            }
//        }
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        String key=gatewayVo.getSN()+"_"+gatewayVo.getType() +":"+gatewayVo.getSN()+"_"+gatewayVo.getType() ;
//        //获取网关以及TDL的电量
//        if(devList.size() == 0 ){
//            fb.setCode(0);
//            fb.setMessage("该线路未绑定设备");
//            return gson.toJson(fb);
//        }
//        for(int i = 0;i<devList.size();i++){
//            String sql2 = "SELECT \"b\"  FROM \"tdl_policy\".\""+device+"\" where \"tdl\"=\'tdl-"+devList.get(i)+"\'and  time >= "+startTime +"  ORDER BY time desc limit 1";
//            QueryResult  queryResult2  = influxDBTemplate.query(new Query(sql2,database));
//            if(queryResult2.getResults().get(0).getSeries() != null){
//                List<List<Object>> values = getValues(queryResult2);
//                for(List<Object> value:values){
//                    resultCircuit.setTDLSN(devList.get(i));
//                    resultCircuit.setBatteryVoltage(value.get(1)==null ? "" :value.get(1).toString());
//                }
//            }
//        }
//        resultCircuit.setCircuitState(resultCircuitVo.getCircuitState());
//        return  gson.toJson(resultCircuit);
//    }


//    根据时间获取经纬度信息
//    @ApiOperation(value = "根据时间获取经纬度信息",notes = "根据时间获取经纬度信息" +
//            "transportationNo:运输编号" +
//            "TDLSN:tdl编号" +
//            "gatewaySN:网关编号" +
//            "startCity:出发城市" +
//            "endCity:目的城市" +
//            "startTime:出发时间" +
//            "endTime:到达时间" +
//            "alarmTime:报警时间" +
//            "alarmType:报警类型" +
//            "alarmValue:报警值" +
//            "lng:经度" +
//            "lat:纬度")
//    @PostMapping("/getLocationByTime")
//    @ApiImplicitParams({
//        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public Object getLocationByTime(@RequestBody LocationVo locationVo,HttpServletRequest request){
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" +token);
//        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        //获取线路信息
//        //获取网关以及tdl信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(locationVo.getTransportationNo(),user.getCompanyName());
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(resultCircuitVo.getEndTime() == 0){
//            gatewayVo = circuitService.getByNo(locationVo.getTransportationNo());
//        }else{
//            gatewayVo = tdlLogService.getByNo(locationVo.getTransportationNo());
//        }
//        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
//        //根据时间获取坐标信息
//        String sql ="SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+devcie+"\" where time = "+locationVo.getTime() *1000000l;
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        List<LocationMessageVo>  locationDataVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            List<List<Object>> values = getValues(queryResult);
//            for(List<Object> value:values){
//                LocationMessageVo messageVo = new LocationMessageVo();
//                messageVo.setTransportationNo(locationVo.getTransportationNo());
//                messageVo.setGatewaySN(gatewayVo.getSN());
//                messageVo.setTDLSN(devList);
//                messageVo.setStartCity(resultCircuitVo.getStartCity());
//                messageVo.setEndCity(resultCircuitVo.getEndCity());
//                messageVo.setStartTime(resultCircuitVo.getStartTime());
//                messageVo.setEndTime(resultCircuitVo.getEndTime());
//                messageVo.setAlarmTime(Long.valueOf(parseTime(value.get(0).toString())));
//                messageVo.setAlarmType(locationVo.getAlarmType());
//                messageVo.setAlarmValue(locationVo.getVlaue());
//                messageVo.setLng(value.get(1)==null ?  "" : value.get(1).toString());
//                messageVo.setLat(value.get(2)==null ?  "" : value.get(2).toString());
//                locationDataVos.add(messageVo);
//            }
//        }
//        return  gson.toJson(locationDataVos);
//    }
//
////    获取温度、湿度、震动强度、震动能量、倾斜度
//    @PostMapping("/getDeviceData")
//    @ApiImplicitParams({
//        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public Object getDeviceData(@RequestBody String transportationNo,HttpServletRequest request){
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" +token);
//        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        //获取网关以及tdl信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo,user.getCompanyName());
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(resultCircuitVo.getEndTime() == 0){
//            gatewayVo = circuitService.getByNo(transportationNo);
//        }else{
//            gatewayVo = tdlLogService.getByNo(transportationNo);
//        }
//        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
//        Long startTime = resultCircuitVo.getStartTime() *1000000l;
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
//        //SELECT "h" FROM "tdl_policy"."WTD93LG_20180314" WHERE "tdl" = 'tdl-0A0B0C0D0F' ORDER BY time DESC
//        String sql = "SELECT \"t\",\"h\",\"p\",\"a0\",\"ta\" FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\" = \'"+devList.get(0)+"\' and  time>="+startTime+"  ORDER BY time ";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        //解析经纬度数据
//        List<DeviceDataVo>  deviceDataVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            deviceDataVos= getDeviceData(queryResult);
//        }
//        return gson.toJson(deviceDataVos);
//    }
//
//
//    //    获取实时数据
//    @ApiOperation(value = "获取实时数据",notes = "获取实时数据")
//    @ApiImplicitParams({
//        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    @PostMapping("/getRealTimeData")
//    public Object getRealTimeData(@RequestBody String transportationNo){
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        //获取gateway的信息
//       // ResultGatewayVo gatewayVo = circuitService.getByNo(transportationNo);
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(gatewayVo==null){
//            fb.setCode(0);
//            fb.setMessage("该线路未绑定设备");
//            return gson.toJson(fb);
//        }
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        //根据gateWay获取tdl信息
//        List<String> key = new ArrayList<>();
//        key.add(gatewayVo.getSN()+"_"+gatewayVo.getType() +":"+gatewayVo.getSN()+"_"+gatewayVo.getType());
//        for (int i = 0;i<devList.size();i++){
//            key.add(gatewayVo.getSN()+"_"+gatewayVo.getType()+":"+devList.get(i));
//        }
//        List<LiveDataVo> liveDataVos = new ArrayList<>();
//        for(int j=0;j<key.size();j++){
//            Map<Object,Object> ret = realRedisService.getHashAll(key.get(j));
//            if(ret == null){
//                continue;
//            }else{
//                Iterator entries = ret.entrySet().iterator();
//                List<String> aliasList = new ArrayList<>();
//                List<String> valueList = new ArrayList<>();
//                while(entries.hasNext()){
//                    Map.Entry entry = (Map.Entry) entries.next();
//                    aliasList.add((String) entry.getKey());
//                    valueList.add((String) entry.getValue());
//                }
//                //根据通道别名获取通道信息
//                List<ProbesVo> probesVoList =  probesService.getByAliasList(gatewayVo.getSN(),gatewayVo.getType(),aliasList);
//                for(int m = 0;m<probesVoList.size();m++){
//                    probesVoList.get(m).setValue(valueList.get(m));
//                }
//                LiveDataVo liveDataVo = new LiveDataVo();
//                liveDataVo.setKey(key.get(j).substring(key.get(j).indexOf(":"),key.get(j).length()));
//                liveDataVo.setProbesVoList(probesVoList);
//                liveDataVos.add(liveDataVo);
//            }
//        }
//        return  gson.toJson(liveDataVos);
//    }
//}


//    @ApiOperation(value = "下载线路数据", notes = "下载线路数据")
//    @PostMapping("/downLoad")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType = "header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public String downLoad(@RequestBody String transportationNo, HttpServletRequest request){
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" + token);
//        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
//        if (StringUtils.isEmpty(transportationNo)) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "transportationNo"));
//            return gson.toJson(fb);
//        }
//        JSONObject jsonObject = JSON.parseObject(transportationNo);
//        transportationNo = jsonObject.getString("transportationNo");
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo, user.getCompanyNo());
//        if (resultCircuitVo == null) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        if (resultCircuitVo.getStartTime() == null || resultCircuitVo.getStartTime() == 0) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "notStarted"));
//            return gson.toJson(fb);
//        }
//        Long startTime = resultCircuitVo.getStartTime() * 1000000l;
//        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo);
//        if (gatewayVo == null) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        String device = gatewayVo.getgType() + "_" + gatewayVo.getgSN();
//        //获取tdl信息
//        List<String> devList = tdlLogService.getTdlSN(transportationNo);
//        if (devList.size() == 0) {
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request, "noCircuit"));
//            return gson.toJson(fb);
//        }
//        OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
//
//        String[] rowsName = new String[]{"时间","压力(MPa)","温度(℃)","湿度(%RH)","震动能量(w)","震动强度(g)","倾斜角度(°)","经度","纬度"};
//        File file = new File("./export");
//        if(!file.exists()){
//            file.mkdir();
//        }
//        try {
//                String sqls = "";
//                if (resultCircuitVo.getEndTime() != null && resultCircuitVo.getEndTime() != 0) {
//                    sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a0\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +resultCircuitVo.getEndTime() * 1000000l+" ORDER BY time ";
//                } else {
//                    sqls = "SELECT  \"tdl\",\"p\",\"T\",\"h\",\"a0\",\"a1\",\"ta\",\"bdlng\",\"bdlat\"  FROM \"tdl_policy\".\""+device+"\" WHERE   time >=" +startTime +"  and  time <=" +System.currentTimeMillis() * 1000000l+" ORDER BY time ";
//                }
//                QueryResult queryResult = influxDBTemplate.query(new Query(sqls, database));
//                 File[] srcFiles = new File[devList.size()];
//                 File zipFile = new File("./export/" +transportationNo+"运输数据.zip");
//                if (queryResult.getResults().get(0).getSeries().get(0) != null) {
//
//                    for(int  i = 0;i<devList.size();i++){
//
//                        CsvWriter csvWriter = new CsvWriter("./export/"+devList.get(i).replace("TDL-", "").trim()+"运输数据.csv" , ',', Charset.forName("GB2312"));
//                        csvWriter.writeRecord(rowsName);
//                        List<List<Object>> values =  getValues(queryResult);
//                        for(List<Object> value : values){
////                            logger.info(value.get(1)==null ? "  " :value.get(1).toString() +"---------"+devList.get(i));
//                            if(value.get(1) == null || value.get(1).toString().equals(devList.get(i))){
//                                String[] objs = new String[rowsName.length];
//                                for (int j = 0; j < objs.length; j++) {
//                                    if(j==0){
//                                        try {
//                                            objs[j] =timeStampDate(new Long(UTCToCST(String.valueOf(value.get(j)))));
////                                            objs[j] =timeStampDate(new Long(dateToStamp(String.valueOf(value.get(j)))));
//                                        } catch (Exception e) {
//                                            e.printStackTrace();
//                                            logger.info(e.toString());
//                                            fb.setMessage("时间解析异常");
//                                        }
//                                    }else {
//                                        objs[j] =value.get(j+1) == null ? "-" :value.get(j+1).toString();
//                                    }
//                                }
//                                csvWriter.writeRecord(objs);
//                                csvWriter.flush();
//                            }
//                        }
//                        csvWriter.close();
//                        srcFiles[i]=new File("./export/"+devList.get(i).replace("TDL-", "").trim()+"运输数据.csv");
//                    }
//                }
//
//            ZipCompressUtil.zipFiles(srcFiles,zipFile);
//            try {
//                String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
//                if (md5Key != null) {
//                    for(int a =0;a<srcFiles.length;a++){
//                        srcFiles[a].delete();
//                    }
//                    zipFile.delete();
//                    fb.setCode(1);
//                    fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/circuit/"+transportationNo+"运输数据.zip");
//                    return gson.toJson(fb);
//                } else {
//                    fb.setCode(0);
//                    fb.setMessage("上传失败");
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//                logger.info(e.toString());
//            }
//        }catch (Exception e){
//            e.printStackTrace();
//            logger.info(e.toString());
//        }
//
//        return  gson.toJson(fb);
//    }
