package com.example.tdl.web;

import com.aliyun.oss.OSSClient;
import com.example.tdl.common.WarehouseExcel;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.Config;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.example.tdl.util.DataUtil;
import com.example.tdl.util.ZipCompressUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/download")
public class DownloadController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    Gson gson=new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private TDLLogService tdlLogService;

    @Value("${spring.influxdb.database}")
    private  String database ;

    @ApiOperation(value = "仓库数据下载",notes = "仓库数据下载")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/download",method = RequestMethod.POST)
    public String downLoad(HttpServletResponse response,HttpServletRequest request,@RequestBody DownloadForWarehouseVo downloadForWarehouseVo) throws ParseException {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (downloadForWarehouseVo.getStartTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"downloadStartTime"));
            return gson.toJson(fb);
        }
        if (downloadForWarehouseVo.getStopTime() ==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"downloadEndTime"));
            return gson.toJson(fb);
        }
        if(downloadForWarehouseVo.getStopTime()<downloadForWarehouseVo.getStartTime()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"downLoad"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(downloadForWarehouseVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(downloadForWarehouseVo.getWarehouseNo(),user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        List<ResultWarehouseNoAndTDLSNAndTimeVo> resultWarehouseNoAndTDLSNAndTimeVos=tdlLogService.getByWarehouseNoAndTime(downloadForWarehouseVo);
        if (resultWarehouseNoAndTDLSNAndTimeVos.size()==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noData"));
            return gson.toJson(fb);
        }
        File zipFile = new File("./export/" +downloadForWarehouseVo.getWarehouseNo()+i18n.getMessage(request, "warehousedata")+".zip");
        File[] srcFiles = new File[resultWarehouseNoAndTDLSNAndTimeVos.size()];
        for (int a=0;a<resultWarehouseNoAndTDLSNAndTimeVos.size();a++) {
            ResultWarehouseNoAndTDLSNAndTimeVo resultWarehouseNoAndTDLSNAndTimeVo=resultWarehouseNoAndTDLSNAndTimeVos.get(a);
            Long startTime = 0l;
            Long stopTime = 0l;
            if(downloadForWarehouseVo.getStartTime() >=resultWarehouseNoAndTDLSNAndTimeVo.getCreateTime()){
                startTime = downloadForWarehouseVo.getStartTime()*1000000l;
            }else{
                startTime = resultWarehouseNoAndTDLSNAndTimeVo.getCreateTime()*1000000l;
            }
            stopTime = downloadForWarehouseVo.getStopTime()*1000000l;
            String sqls = "SELECT \"b\", \"T\", \"h\" FROM \"tdl_policy\".\"" + resultWarehouseNoAndTDLSNAndTimeVo.getGatewayType() + "_" + resultWarehouseNoAndTDLSNAndTimeVo.getGatewaySN() + "\" WHERE \"tdl\" = 'TDL-" + resultWarehouseNoAndTDLSNAndTimeVo.getTDLSN() + "' and  time >= " + startTime + "  and time<= " + stopTime + " ORDER BY time ";
            QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
            if(queryResults.getResults().get(0).getSeries()==null){
                continue;
            }
            String  fileName =downloadForWarehouseVo.getWarehouseNo() + i18n.getMessage(request, "warehousedata")+"-"+resultWarehouseNoAndTDLSNAndTimeVo.getTDLSN();
            File myPath = new File( "./export/" );
            if ( !myPath.exists()){//若此目录不存在，则创建之
                myPath.mkdir();
            }
            srcFiles[a] = new File("./export/"+fileName+".xls") ;
            List<DownloadWarehouseVo> downloadWarehouseVos = getWarehouseData(queryResults,downloadForWarehouseVo.getOffset());
            for (int i = 0; i < downloadWarehouseVos.size(); i++) {
                if (downloadWarehouseVos.get(i).getTemperature() > resultWarehouseNoAndTDLSNAndTimeVos.get(a).getTemMax() ||
                        downloadWarehouseVos.get(i).getTemperature() < resultWarehouseNoAndTDLSNAndTimeVos.get(a).getTemMin() ||
                        downloadWarehouseVos.get(i).getHumidity() > resultWarehouseNoAndTDLSNAndTimeVos.get(a).getHumidityMax() ||
                        downloadWarehouseVos.get(i).getHumidity() < resultWarehouseNoAndTDLSNAndTimeVos.get(a).getHumidityMin()) {
                    downloadWarehouseVos.get(i).setAlarm(i18n.getMessage(request,"Alarm"));
                } else {
                    downloadWarehouseVos.get(i).setAlarm(i18n.getMessage(request,"Nomal"));
                }
            }
            resultWarehouseNoAndTDLSNAndTimeVo.setDownloadWarehouseVos(downloadWarehouseVos);
            try {
                WarehouseExcel warehouseExcel = new WarehouseExcel(fileName,user.getUserName(),resultWarehouseNoAndTDLSNAndTimeVo,request,i18n);
                HSSFWorkbook wb = warehouseExcel.downloadExcel();
                request.setCharacterEncoding("utf-8");
                response.setCharacterEncoding("utf-8");
                response.setContentType("application/vnd.ms-excel");
                response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
                OutputStream os = new FileOutputStream(new File("./export/"+fileName+".xls"));
                wb.write(os);
                os.flush();
                os.close();
            } catch (Exception e) {
                e.printStackTrace();
                logger.info(e.toString());
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"downloadfail"));
                return  gson.toJson(fb);
            }
        }
        if(srcFiles.length== 0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noData"));
            return gson.toJson(fb);
        }
        try {
            ZipCompressUtil.zipFiles(srcFiles,zipFile);
            OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
            String md5Key = AliyunOSSClientUtil.uploadObject2OSS(ossClient, zipFile);
            if (md5Key != null) {
                for(int a =0;a<srcFiles.length;a++){
                    if(srcFiles[a] == null){
                        continue;
                    } else{
                        srcFiles[a].delete();
                    }
                }
                zipFile.delete();
                fb.setCode(1);
                fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/warehouse/"+zipFile.getName());
                return gson.toJson(fb);
            } else {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"fileUpload"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"fileUpload"));
            return  gson.toJson(fb);
        }
        return  gson.toJson(fb);
    }

    /**
     * 先根遍历序递归删除文件夹
     *
     * @param dirFile 要被删除的文件或者目录
     * @return 删除成功返回true, 否则返回false
     */
    public static boolean deleteFile(File dirFile) {
        // 如果dir对应的文件不存在，则退出
        if (!dirFile.exists()) {
            return false;
        }

        if (dirFile.isFile()) {
            return dirFile.delete();
        } else {
            for (File file : dirFile.listFiles()) {
                deleteFile(file);
            }
        }

        return dirFile.delete();
    }

    public List<DownloadWarehouseVo> getWarehouseData(QueryResult queryResult,String offset){
        List<List<Object>> values = DataUtil.getValues(queryResult);
        List<DownloadWarehouseVo> downloadWarehouseVos = new ArrayList<>();
        for(List<Object> value:values){
            DownloadWarehouseVo downloadWarehouseVo = new DownloadWarehouseVo();
            downloadWarehouseVo.setTime(DataUtil.UTCToCST(String.valueOf(value.get(0)),offset));
            downloadWarehouseVo.setPressure(Float.parseFloat(value.get(1).toString()));
            downloadWarehouseVo.setTemperature(Float.parseFloat(value.get(2).toString()));
            downloadWarehouseVo.setHumidity(Float.parseFloat(value.get(3).toString()));
            downloadWarehouseVos.add(downloadWarehouseVo);
        }
        return downloadWarehouseVos;
    }

}
