package com.example.tdl.web;


import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;

import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.stereotype.Controller;


@Controller
public class I18nController {

    private static final Logger LOGGER = LoggerFactory.getLogger(I18nController.class);

    private MessageSource messageSource;

    @Value("${spring.messages.basename}")
    private String basename;

    @Value("${spring.messages.cache-seconds}")
    private long cacheMillis;

    @Value("${spring.messages.encoding}")
    private String encoding;


    /**
     * 初始化
     * @return
     */
    private MessageSource initMessageSource() {
        ReloadableResourceBundleMessageSource messageSource=new ReloadableResourceBundleMessageSource();
        LOGGER.info("baseName====>:" + this.basename);
        messageSource.setBasename(basename);
        messageSource.setDefaultEncoding(encoding);
        messageSource.setCacheMillis(cacheMillis);
        return messageSource;
    }

    public  Locale getLocale(HttpServletRequest request){
        String lauage=request.getHeader("AccountLanguage");
        //默认没有就是请求地区的语言
        Locale locale=null;
        if(lauage == null){
            locale=Locale.CHINA;
        } else if("jp".equals(lauage)){
            locale=Locale.JAPANESE;
        } else if("en".equals(lauage)){
            locale=Locale.ENGLISH;
        } else if("cn".equals(lauage)){
            locale=Locale.CHINA;
        } else{
            locale=Locale.CHINA;
        }
        return  locale;
    }


    /**
     * 设置当前的返回信息
     * @param request
     * @param code
     * @return
     */
    public String getMessage(HttpServletRequest request, String code){
        if(messageSource==null){
            messageSource=initMessageSource();
        }
        String result=null;
        try {
            result = messageSource.getMessage(code, null,  getLocale(request));
        } catch (NoSuchMessageException e) {
            e.printStackTrace();
            LOGGER.error("Cannot find the error message of internationalization, return the original error message.");
        }
        if(result==null){
            return code;
        }
        return result;
    }


    public String getMessage(HttpServletRequest request,Object[] args, String code){
        if(messageSource==null){
            messageSource=initMessageSource();
        }
        try {
            return  messageSource.getMessage(code, args,getLocale(request));
        } catch (NoSuchMessageException e) {
            e.printStackTrace();
            LOGGER.error("Cannot find the error message of internationalization, return the original error message.");
            return code;
        }
    }


}
