package com.example.tdl.web;


import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.LoginLogTermVo;
import com.example.tdl.domain.vo.UserRedisVo;
import com.example.tdl.service.LoginLogService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

@RestController
@RequestMapping("/loginLog")
public class LoginLogController {

    private Logger logger = LoggerFactory.getLogger(LoginLogController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private LoginLogService loginLogService=new LoginLogService();

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;


    //查询登录日志
    @ApiOperation(value = "查询登录日志",notes = "查询登录日志，返回值说明：" +
            "   userName:用户名," +
            "   createTime:登录时间," +
            "   ipAddress:登录地址。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return loginLogService.getAll(user.getCompanyNo());
    }

    //根据时间查询登录日志
    @ApiOperation(value = "根据时间查询登录日志",notes = "根据时间查询登录日志，传值说明：" +
            "   startTime:查询开始时间," +
            "   stopTime:查询结束时间。" +
            "返回值说明：" +
            "   userName:用户名," +
            "   createTime:登录时间," +
            "   ipAddress:登录地址。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByTime",method = RequestMethod.POST)
    public Object getByTime(@RequestBody LoginLogTermVo loginLogTermVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (loginLogTermVo.getStartTime() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"logStartTime"));
            return gson.toJson(fb);
        }
        if (loginLogTermVo.getStopTime() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"logEndTime"));
            return gson.toJson(fb);
        }
        if(loginLogTermVo.getStopTime() < loginLogTermVo.getStartTime()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongTime"));
            return gson.toJson(fb);
        }
        return loginLogService.getByTime(loginLogTermVo.getStartTime(),loginLogTermVo.getStopTime(),user.getCompanyNo());
    }
}
