package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;

import com.example.tdl.domain.vo.*;
import com.example.tdl.service.PermissionService;
import com.example.tdl.service.RoleHasPermissionService;
import com.example.tdl.service.SideService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/permission")
public class PermissionController {

    private Logger logger = LoggerFactory.getLogger(PermissionController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private PermissionService permissionService=new PermissionService();

    @Autowired
    private RoleHasPermissionService roleHasPermissionService;

    @Autowired
    private SideService sideService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;


    //查询本公司的用户
    @ApiOperation(value = "查询所有权限信息",notes = "查询所有权限信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String name = "Administrator";
        List<String> permissionLists = roleHasPermissionService.getByRoleName(name,user.getCompanyNo());
        //获取所有父侧边栏的名字
        List<String> parentSideLists=sideService.getSideTitleParentName();
        List<SideGroupVo> sideGroupVos=new ArrayList<>();
        for (String parentSideName: parentSideLists){
            SideGroupVo sideGroupVo = new SideGroupVo();
            //获取子侧边栏的名字str.replace(" ", "")
            List<String> sideTitles = sideService.getSideTitleByParentName(parentSideName);
            List<SideAndPermissionVo> sideAndPermissionVos = new ArrayList<>();
            for(String sideName:sideTitles){
                SideAndPermissionVo sideAndPermissionVo =new SideAndPermissionVo();
                //获取侧边栏对应的父权限名
                String permissionName=sideService.getParentPermissionBySideTitle(sideName);
                PermissionGroupVo permissionGroupVo=new PermissionGroupVo();
                permissionGroupVo.setParentName(i18n.getMessage(request,permissionName.replace(" ","")));
                //获取父权限对应的子权限
                List<String> permissionNames=permissionService.getPermissionByPermissionName(permissionName);
                List<PermissionShowVo> permissionShowVoList = new ArrayList<>();
                for (String permissionName1:permissionNames){
                    if(permissionLists.contains(permissionName1)) {
                        PermissionShowVo permissionShowVo = new PermissionShowVo();
                        permissionShowVo.setPermissionName(i18n.getMessage(request,permissionName1.replace(" ","")));
                        permissionShowVo.setChecked(false);
                        permissionShowVoList.add(permissionShowVo);
                    }
                }
                if(permissionShowVoList.size() >0){
                    sideAndPermissionVo.setSideTitle(i18n.getMessage(request,sideName.replace(" ","")));
                    sideAndPermissionVo.setPermissionShowVos(permissionShowVoList);
                    sideAndPermissionVos.add(sideAndPermissionVo);
                }
            }
            if(sideAndPermissionVos.size() >0){
                sideGroupVo.setParentSide(i18n.getMessage(request,parentSideName.replace(" ","")));
                sideGroupVo.setSideAndPermissionVos(sideAndPermissionVos);
                sideGroupVos.add(sideGroupVo);
            }
        }
        return sideGroupVos;

    }

    //根据角色获取所有的权限
    @ApiOperation(value = "根据角色获取所有的权限",notes = "获取该公司最高权限")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value = "/getPermissionByRoleName",method = RequestMethod.POST)
    public Object getPermissionByRoleName(@RequestBody String roleName,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(roleName)){
            fb.setCode(0);
            fb.setMessage("查询的角色名不能为空");
            return gson.toJson(fb);
        }
        JSONObject json = JSON.parseObject(roleName);
        roleName= (String) json.get("roleName");
        //获取该公司的所有权限
        String name = "Administrator";
        List<String> permissionLists = roleHasPermissionService.getByRoleName(name,user.getCompanyNo());
        List<String> rolePermission = roleHasPermissionService.getByRoleName(roleName,user.getCompanyNo());

        //获取父侧边栏的名字
        List<String> parentSideLists=sideService.getSideTitleParentName();
        List<SideGroupVo> sideGroupVos=new ArrayList<>();
        for (String parentSideName: parentSideLists){
            SideGroupVo sideGroupVo = new SideGroupVo();
            //获取子侧边栏的名字
            List<String> sideTitles = sideService.getSideTitleByParentName(parentSideName);
            List<SideAndPermissionVo> sideAndPermissionVos = new ArrayList<>();
            for(String sideName:sideTitles){
                SideAndPermissionVo sideAndPermissionVo =new SideAndPermissionVo();
                //获取侧边栏对应的父权限名
                String permissionName=sideService.getParentPermissionBySideTitle(sideName);
                PermissionGroupVo permissionGroupVo=new PermissionGroupVo();
                permissionGroupVo.setParentName(i18n.getMessage(request,permissionName.replace(" ","")));
                //获取父权限对应的子权限
                List<String> permissionNames=permissionService.getPermissionByPermissionName(permissionName);
                List<PermissionShowVo> permissionShowVoList = new ArrayList<>();
                for (String permissionName1:permissionNames){
                    if(permissionLists.contains(permissionName1)) {
                        PermissionShowVo permissionShowVo = new PermissionShowVo();
                        permissionShowVo.setPermissionName(i18n.getMessage(request,permissionName1.replace(" ","")));
                        if (rolePermission.contains(permissionName1)) {
                            permissionShowVo.setChecked(true);
                        } else {
                            permissionShowVo.setChecked(false);
                        }
                        permissionShowVoList.add(permissionShowVo);
                    }
                }
                if (permissionShowVoList.size() > 0) {
                    sideAndPermissionVo.setSideTitle(i18n.getMessage(request,sideName.replace(" ","")));
                    sideAndPermissionVo.setPermissionShowVos(permissionShowVoList);
                    sideAndPermissionVos.add(sideAndPermissionVo);
                }
            }
            if (sideAndPermissionVos.size() > 0) {
                sideGroupVo.setParentSide(i18n.getMessage(request,parentSideName.replace(" ","")));
                sideGroupVo.setSideAndPermissionVos(sideAndPermissionVos);
                sideGroupVos.add(sideGroupVo);
            }
        }
        return sideGroupVos;
    }


    //新增或者修改权限
    @ApiOperation(value = "添加或修改权限", notes = "根据角色添加或修改权限")
    @RequestMapping(value = "/addOrSavePermission",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public String addOrSavePermission(@RequestBody AddOrUpdatePermissionVo addOrUpdatePermissionVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addOrUpdatePermissionVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        if("Administrator".equals(addOrUpdatePermissionVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"adminInoperable"));
            return gson.toJson(fb);
        }
        String permissionVar="";
        for(int i=0;i<addOrUpdatePermissionVo.getPermissionGroupVos().size();i++){
            permissionVar+=addOrUpdatePermissionVo.getPermissionGroupVos().get(i)+"&";
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",addOrUpdatePermissionVo.getRoleName());
            map.put("companyNo",user.getCompanyNo());
            map.put("permissionName",permissionVar);
            map.put("num",addOrUpdatePermissionVo.getPermissionGroupVos().size());
            roleHasPermissionService.addRoleHasPermission(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("权限操作成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        } catch (DataAccessException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("操作失败");
        }
        return gson.toJson(fb);
    }

}
