package com.example.tdl.web;

import com.aliyun.oss.OSSClient;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.service.*;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import java.awt.image.BufferedImage;
import java.io.IOException;

@RestController("/picUpload")
public class PicUploadController {

    private final Logger LOGGER = LoggerFactory.getLogger(PicUploadController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private  UserService userService;

    @Autowired
    private CarService carService;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private CompanyService companyService;


    @Autowired
    private I18nController i18n;

    // 允许上传的格式
    private static final String[] IMAGE_TYPE = new String[]{".jpg",".png" };


    //上传图片
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/upload",method = RequestMethod.POST)
    public String upload(MultipartFile multipartFile, @RequestParam(value = "imageType",required = true) String imageType, HttpServletRequest request) {
        if (multipartFile == null ||multipartFile.isEmpty() || StringUtils.isBlank(multipartFile.getOriginalFilename())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"nonePicture"));
            return gson.toJson(fb);
        }
        if (multipartFile.getSize() > 512000) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"pictureSize"));
            return gson.toJson(fb);
        }
        try {
            BufferedImage sourceImg = ImageIO.read(multipartFile.getInputStream());
            // 仓库 150*120  //货车 90*60
            if(sourceImg == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"pictureFormat"));
                return  gson.toJson(fb);
            }else{
                if("truck".equals(imageType)){
                    if(sourceImg.getWidth()>90){
                        fb.setCode(0);
                        fb.setMessage(i18n.getMessage(request,"truckPictureDimension"));
                        return  gson.toJson(fb);
                    }
                    if(sourceImg.getHeight()>60){
                        fb.setCode(0);
                        fb.setMessage(i18n.getMessage(request,"truckPictureDimension"));
                        return  gson.toJson(fb);
                    }
                }else if("warehouse".equals(imageType)) {
                    if (sourceImg.getWidth() > 150) {
                        fb.setCode(0);
                        fb.setMessage(i18n.getMessage(request, "warePictureDimension"));
                        return gson.toJson(fb);
                    }
                    if (sourceImg.getHeight() > 120) {
                        fb.setCode(0);
                        fb.setMessage(i18n.getMessage(request, "warePictureDimension"));
                        return gson.toJson(fb);
                    }
                }
//                }else if("company".equals(imageType)){
//                    if(sourceImg.getWidth()>150){
//                        fb.setCode(0);
//                        fb.setMessage(i18n.getMessage(request,"comPictureDimension"));
//                        return  gson.toJson(fb);
//                    }
//                    if(sourceImg.getHeight()>200){
//                        fb.setCode(0);
//                        fb.setMessage(i18n.getMessage(request,"comPictureDimension"));
//                        return  gson.toJson(fb);
//                    }
//                }
            }
        } catch (IOException e) {
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"pictureFormat"));
            return  gson.toJson(fb);
        }
        // 校验图片格式
        boolean isLegal = false;
        // 判断文件后缀名是否符合
        for (String type : IMAGE_TYPE) {
            if (StringUtils.endsWithIgnoreCase(multipartFile.getOriginalFilename(), type)) {
                isLegal = true;
                break;
            }
        }
        if (isLegal) {
            try {
                //初始化OSSClient
                OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
                LOGGER.info(multipartFile.getOriginalFilename());
                String url = AliyunOSSClientUtil.uploadImg2Oss(ossClient, multipartFile,System.currentTimeMillis()+"TM",imageType);
                if (url == null) {
                    fb.setMessage(i18n.getMessage(request,"uploadFailure"));
                    fb.setCode(0);
                } else {
                    if("user".equals(imageType)){
                        //https://tdlcloud.oss-cn-shanghai.aliyuncs.com
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/userImage/" + url);
                    }else if("company".equals(imageType)){
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/companyLog/" + url);
                    }else if("truck".equals(imageType)){
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/palteImage/" + url);
                    }else if ("warehouse".equals(imageType)){
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/wareHouseImage/" + url);
                    }else if("circuit".equals(imageType)){
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/userImage/" + url);
                    }
                    fb.setCode(1);
                }
            } catch (Exception e) {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"uploadFailure"));
                e.printStackTrace();
                LOGGER.info(e.toString());
            }
        } else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"pictureFormat"));
        }
        return  gson.toJson(fb);
    }


    //上传图片
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/updateImg",method = RequestMethod.POST)
    public String updateImg(MultipartFile multipartFile, @RequestParam(value = "imageType",required = true) String imageType,@RequestParam(value = "number",required = true) String number,HttpServletRequest request){
        if (multipartFile.isEmpty() || StringUtils.isBlank(multipartFile.getOriginalFilename())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"nonePicture"));
            return gson.toJson(fb);
        }
        if (multipartFile.getSize() > 512000) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"pictureSize"));
            return gson.toJson(fb);
        }
        try {
            BufferedImage sourceImg = ImageIO.read(multipartFile.getInputStream());
            // 仓库 150*120  //货车 90*60
            if(sourceImg == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"pictureFormat"));
                return  gson.toJson(fb);
            }
            if("truck".equals(imageType)){
                if(sourceImg.getWidth()>90){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"truckPictureDimension"));
                    return  gson.toJson(fb);
                }
                if(sourceImg.getHeight()>60){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"truckPictureDimension"));
                    return  gson.toJson(fb);
                }
            }else if("warehouse".equals(imageType)){
                if(sourceImg.getWidth()>150){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"warePictureDimension"));
                    return  gson.toJson(fb);
                }
                if(sourceImg.getHeight()>120){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"warePictureDimension"));
                    return  gson.toJson(fb);
                }
            }
//            else if("company".equals(imageType)){
//                if(sourceImg.getWidth()>150){
//                    fb.setCode(0);
//                    fb.setMessage(i18n.getMessage(request,"comPictureDimension"));
//                    return  gson.toJson(fb);
//                }
//                if(sourceImg.getHeight()>200){
//                    fb.setCode(0);
//                    fb.setMessage(i18n.getMessage(request,"comPictureDimension"));
//                    return  gson.toJson(fb);
//                }
//            }
        } catch (IOException e) {
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"pictureDimension"));
        }
        boolean isLegal = false;
        // 判断文件后缀名是否符合
        for (String type : IMAGE_TYPE) {
            if (StringUtils.endsWithIgnoreCase(multipartFile.getOriginalFilename(), type)) {
                isLegal = true;
                break;
            }
        }
        if (isLegal) {
            try {
                OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
                LOGGER.info(multipartFile.getOriginalFilename());
                String url = AliyunOSSClientUtil.uploadImg2Oss(ossClient, multipartFile,number,imageType);
                if (url == null) {
                    fb.setMessage(i18n.getMessage(request,"uploadFailure"));
                    fb.setCode(0);
                } else {
                    fb.setCode(1);
                    String img = "https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/";
                    int i  = 0;
                    if("user".equals(imageType)){
                        img = img +"userImage/" +url;
                        //修改用户图片
//                      i =    userService.updateImg(number,img);
                    }else if("company".equals(imageType)){
                        img = img +"companyLog/" +url;
                        i =  companyService.updateImg(img,number);
                    }else if("truck".equals(imageType)){
                        img = img +"palteImage/" +url;
                        i =  carService.updateImg(img,number);
                    }else if ("warehouse".equals(imageType)) {
                        img = img + "wareHouseImage/" + url;
                        i = warehouseService.updateImg(number, img,System.currentTimeMillis());
                    }
                    if(i>0){
                        fb.setCode(1);
                        fb.setMessage(img);
                    }else{
                        fb.setCode(0);
                        fb.setMessage(i18n.getMessage(request,"uploadFailure"));
                    }
                }
            } catch (Exception e) {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"pictureFormat"));
                e.printStackTrace();
                LOGGER.info(e.toString());
            }
        } else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"pictureFormat"));
        }
        return  gson.toJson(fb);
    }

}
