package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.RegionService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;


@RestController
@RequestMapping("/region")
public class RegionController {

    private Logger logger = LoggerFactory.getLogger(RegionController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private RegionService regionService=new RegionService();

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController i18n;

    //查询所有预警信息
    @ApiOperation(value = "查询所有区域信息",notes = "查询所有区域信息,返回值说明：" +
            "   regionNo：区域编号，" +
            "   regionName：区域名，" +
            "   companyNo：所属公司，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return regionService.getAll(user.getCompanyNo());
    }

    //添加区域信息
    @ApiOperation(value = "添加区域信息",notes = "添加预警信息,regionNo,companyNo不用传，需要传递参数：" +
            "   regionName：区域名，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addRegion",method = RequestMethod.POST)
    public Object addRegion(@RequestBody ResultRegionVo regionVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);

        if (StringUtils.isEmpty(regionVo.getRegionName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"regionName"));
            return gson.toJson(fb);
        }
        if(regionService.getByRegionName(user.getCompanyNo(),regionVo.getRegionName())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repetitionRegionName"));
            return gson.toJson(fb);
        }
        regionVo.setRegionNo("RG"+System.currentTimeMillis());
        regionVo.setCompanyNo(user.getCompanyNo());
        int a=regionService.addRegion(regionVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addRegionSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addRegionFailure"));
        }
        return gson.toJson(fb);
    }

    //修改区域信息
    @ApiOperation(value = "修改区域信息",notes = "修改区域信息,companyNo不用传，需要传递参数：" +
            "   regionNo：需要修改的区域编号，" +
            "   regionName：修改后的区域名，" +
            "   remark：修改后的备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateRegion",method = RequestMethod.POST)
    public Object updateRegion(@RequestBody UpdateRegionVo updateRegionVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(updateRegionVo.getRegionNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"regionNo"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateRegionVo.getRegionName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"regionName"));
            return gson.toJson(fb);
        }
        if(regionService.getByRegionName(user.getCompanyNo(),updateRegionVo.getRegionName())!=null&&!regionService.getByRegionNo(user.getCompanyNo(),updateRegionVo.getRegionNo()).getRegionName().equals(updateRegionVo.getRegionName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repetitionRegionName"));
            return gson.toJson(fb);
        }
        updateRegionVo.setCompanyNo(user.getCompanyNo());
        int a=regionService.updateRegion(updateRegionVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"updateRegionSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateRegionFailure"));
        }
        return gson.toJson(fb);

    }

    //删除区域信息
    @ApiOperation(value = "删除区域信息",notes = "删除区域信息,companyNo不用传，传值说明：" +
            "   regionNo：需要删除的区域编号" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/deleteRegion",method = RequestMethod.POST)
    public Object deleteRegion(@RequestBody String regionNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        JSONObject jsonObject= JSON.parseObject(regionNo);
        regionNo=jsonObject.getString("regionNo");

        if (StringUtils.isEmpty(regionNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"regionNo"));
            return gson.toJson(fb);
        }
        if (regionService.getByRegion(regionNo)>0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"useingRegion"));
            return gson.toJson(fb);
        }
        int a=regionService.deleteRegion(regionNo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deleteRegionSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deleteRegionFailure"));
        }
        return gson.toJson(fb);
    }

}
