package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;

import com.example.tdl.domain.vo.*;
import com.example.tdl.service.RoleHasPermissionService;
import com.example.tdl.service.RoleService;
import com.example.tdl.service.UserService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.dao.DataAccessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/role")
public class RoleController {

    private Logger logger = LoggerFactory.getLogger(RoleController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private RoleService roleService=new RoleService();

    @Autowired
    private UserService userService=new UserService();

    @Autowired
    private RoleHasPermissionService roleHasPermissionService=new RoleHasPermissionService();

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;

    //获取所有的角色
    @ApiOperation(value = "获取所有的角色",notes = "获取所有角色，返回值说明" +
            "   roleName:角色名," +
            "   createTime:创建时间," +
            "   updateTime:修改时间," +
            "   remark:备注," +
            "   companyName:公司名.")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @GetMapping(path="/getAll")
    public @ResponseBody Object getAll(HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<ResultRoleVo> list=roleService.getAll(user.getCompanyNo());
        return list;
    }

    //查询所有角色名
    @ApiOperation(value = "根据公司名查询所有角色名",notes = "查询所有角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getRoleNameByCompanyName",method = RequestMethod.POST)
    public Object getRoleNameByCompanyName(@RequestBody String companyName, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        JSONObject json = JSON.parseObject(companyName);
        companyName= (String) json.get("companyName");
        List<RoleNameVo> roleNameVoList =  roleService.getRoleNameByCompanyName(companyName);
        return  roleNameVoList;
    }

    //添加角色
    @ApiOperation(value = "添加角色",notes = "添加角色，公司名和时间不用传，传递参数：" +
            "   roleName:角色名," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/addRole", method = RequestMethod.POST)
    public Object addRole(@RequestBody ResultRoleVo resultRoleVo,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(resultRoleVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        Integer a=roleService.getByRoleName(resultRoleVo.getRoleName(),user.getCompanyNo());
        if (a>0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedRole"));
            return gson.toJson(fb);
        }
        try {
            resultRoleVo.setCreateTime(String.valueOf(System.currentTimeMillis()));
            resultRoleVo.setUpdateTime(String.valueOf(System.currentTimeMillis()));
            resultRoleVo.setCompanyNo(user.getCompanyNo());
            int s=roleService.addRole(resultRoleVo);
            if (s>0){
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"addRoleSuccess"));
            }else{
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"addRoleFailure"));
            }
        }catch (Exception e){
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage("添加失败");
        }
        return gson.toJson(fb);
    }

    //添加角色的同时添加权限
    @ApiOperation(value = "添加角色的同时添加权限",notes = "添加角色的同时添加权限，传递参数：" +
            "   permissions: [" +
            "       权限名" +
            "   ]," +
            "   roleName:角色名," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/addRoleAndPermission", method = RequestMethod.POST)
    public Object addRoleAndPermission(@RequestBody AddRoleForPermissionVo addRoleForPermissionVo,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addRoleForPermissionVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        if (addRoleForPermissionVo.getRoleName().equals("Administrator")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"adminInoperable"));
            return gson.toJson(fb);
        }
        Integer a=roleService.getByRoleName(addRoleForPermissionVo.getRoleName(),user.getCompanyNo());
        if (a>0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedRole"));
            return gson.toJson(fb);
        }
        String permissionVar="";
        for(int i=0;i<addRoleForPermissionVo.getPermissions().size();i++){
            permissionVar+=addRoleForPermissionVo.getPermissions().get(i)+" ";
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",addRoleForPermissionVo.getRoleName());
            map.put("createTime",System.currentTimeMillis());
            map.put("updateTime",System.currentTimeMillis());
            map.put("remark",addRoleForPermissionVo.getRemark());
            map.put("companyNo",user.getCompanyNo());
            map.put("permissionName",permissionVar);
            map.put("num",addRoleForPermissionVo.getPermissions().size());
            roleService.addRoleAndPermission(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("角色添加成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        } catch (DataAccessException e) {
            fb.setCode(0);
            fb.setMessage("操作失败");
        }
        return gson.toJson(fb);
    }

    //修改角色
    @ApiOperation(value = "修改角色",notes = "根据角色名修改角色，公司不用传" +
            "   oldRoleName：修改之前的名字，" +
            "   roleName：修改之后的名字，" +
            "   updateTime:修改时间，" +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/updateRole", method = RequestMethod.POST)
    public Object updateRole(@RequestBody UpdateRoleVo updateRoleVo,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateRoleVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        logger.info(roleService.getByRoleName(updateRoleVo.getRoleName(),user.getCompanyNo()) +"-----");
        if (roleService.getByRoleName(updateRoleVo.getRoleName(),user.getCompanyNo())!=0&&!updateRoleVo.getOldRoleName().equals(updateRoleVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedRole"));
            return gson.toJson(fb);
        }
        if (updateRoleVo.getRoleName().equals("Administrator")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"adminInoperable"));
            return gson.toJson(fb);
        }
        try {
            updateRoleVo.setCompanyNo(user.getCompanyNo());
            int c=roleService.updateRole(updateRoleVo);
            if (c>0){
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"updateSuccess"));
            }else{
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"updateFailure"));
            }
        }catch (Exception e){
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage("修改失败");
        }
        return gson.toJson(fb);
    }

    //删除角色
    @ApiOperation(value = "删除角色",notes = "根据角色名删除角色，只用传角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/deleteRole", method = RequestMethod.POST)
    public Object deleteRole(@RequestBody String roleName,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        JSONObject jsonObject= JSON.parseObject(roleName);
        roleName=(String)jsonObject.get("roleName");
        if (StringUtils.isEmpty(roleName)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        if (roleName.equals("Administrator")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"adminInoperable"));
            return gson.toJson(fb);
        }
        int f=userService.getByRoleName(roleName,user.getCompanyNo());
        if (f>0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"usedRole"));
            return gson.toJson(fb);
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",roleName);
            map.put("companyNo",user.getCompanyNo());
            roleService.deleteRole(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("删除角色成功")){
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"deleteSuccess"));
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        }catch (Exception e){
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deleteFailure"));
        }
        return gson.toJson(fb);
    }

    //辉度修改其他公司的角色权限
    @ApiOperation(value = "辉度修改其他公司的角色权限",notes = "辉度修改其他公司的角色权限，传递参数:" +
            "   roleName:角色名，" +
            "   companyNameVar:需要修改的公司")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/updateRoleForWitium", method = RequestMethod.POST)
    public Object updateRoleForWitium(@RequestBody UpdateRoleForWitiumVo updateRoleForWitiumVo,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (updateRoleForWitiumVo.getCompanyNameVar().size()==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"companyName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateRoleForWitiumVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        if (!user.getCompanyNo().equals("TM201803298320")){
            fb.setCode(0);
            fb.setMessage("只有辉度能修改其他公司的权限");
            return gson.toJson(fb);
        }
        String companyNameVar="";
        for(int i=0;i<updateRoleForWitiumVo.getCompanyNameVar().size();i++){
            companyNameVar+=updateRoleForWitiumVo.getCompanyNameVar().get(i)+"&";
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",updateRoleForWitiumVo.getRoleName());
            map.put("companyNameVar",companyNameVar);
            map.put("num",updateRoleForWitiumVo.getCompanyNameVar().size());
            roleService.updatePermissionForWitium(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("修改权限成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        } catch (org.springframework.dao.DataAccessException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("操作失败");
        }
        return gson.toJson(fb);
    }

    //辉度添加其他公司角色
    @ApiOperation(value = "辉度添加其他公司角色",notes = "辉度添加其他公司角色，传递参数：" +
            "   roleName:角色名," +
            "   companyName:需要添加的公司," +
            "   permissions: [" +
            "       权限名" +
            "   ]," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/addRoleForWitium", method = RequestMethod.POST)
    public Object addRoleForWitium(@RequestBody AddRoleForWitiumVo addRoleForWitiumVo,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (!user.getCompanyNo().equals("TM201803298320")){
            fb.setCode(0);
            fb.setMessage("只有辉度能添加其他公司的角色");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addRoleForWitiumVo.getCompanyName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"companyName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addRoleForWitiumVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        Integer a=roleService.getByRoleName(addRoleForWitiumVo.getRoleName(),addRoleForWitiumVo.getCompanyName());
        if (a>0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedRole"));
            return gson.toJson(fb);
        }
        String permissionVar="";
        for(int i=0;i<addRoleForWitiumVo.getPermissionGroupVos().size();i++){
            permissionVar+=addRoleForWitiumVo.getPermissionGroupVos().get(i)+" ";
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",addRoleForWitiumVo.getRoleName());
            map.put("createTime",System.currentTimeMillis());
            map.put("updateTime",System.currentTimeMillis());
            map.put("remark",addRoleForWitiumVo.getRemark());
            map.put("companyName",addRoleForWitiumVo.getCompanyName());
            map.put("permissionName",permissionVar);
            map.put("num",addRoleForWitiumVo.getPermissionGroupVos().size());
            roleService.addRoleAndPermission(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("角色添加成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        } catch (DataAccessException e) {
            fb.setCode(0);
            fb.setMessage("操作失败");
        }
        return gson.toJson(fb);
    }



}


/*@ApiOperation(value = "辉度修改其他公司的角色权限",notes = "辉度修改其他公司的角色权限，传递参数:" +
            "   roleName:角色名，" +
            "   permissions: [" +
            "       权限名" +
            "   ]," +
            "   companyName:角色所属公司")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/updateRoleForWitium", method = RequestMethod.POST)
    public Object updateRoleForWitium(@RequestBody UpdateRoleForWitiumVo updateRoleForWitiumVo,HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateRoleForWitiumVo.getCompanyName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"companyName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateRoleForWitiumVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        String permissionVar="";
        for(int i=0;i<updateRoleForWitiumVo.getPermissionGroupVos().size();i++){
            permissionVar+=updateRoleForWitiumVo.getPermissionGroupVos().get(i)+" ";
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",updateRoleForWitiumVo.getRoleName());
            map.put("companyName",updateRoleForWitiumVo.getCompanyName());
            map.put("permissionName",permissionVar);
            map.put("num",updateRoleForWitiumVo.getPermissionGroupVos().size());
            roleHasPermissionService.addRoleHasPermission(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("权限操作成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        } catch (org.springframework.dao.DataAccessException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("操作失败");
        }
        return gson.toJson(fb);
    }*/
