package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.TDLDataOffset;
import com.example.tdl.service.TDLDeviceService;
import com.example.tdl.service.redis.AlarmRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AlarmRule;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;


@RestController
@RequestMapping("/tdlDevice")
public class TDLDeviceController {

    private Logger logger = LoggerFactory.getLogger(TDLDeviceController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private TDLDeviceService tdlDeviceService=new TDLDeviceService();

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private AlarmRedisService alarmRedisService;

    @Autowired
    private I18nController  i18n;

    //查询TDL信息
    @ApiOperation(value = "查询TDL信息",notes = "查询TDL信息，返回值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号," +
            "   counts:使用次数," +
            "   lastTime:最后使用时间," +
            "   useScene:使用场景（0为空闲，1为运输，2为仓库）," +
            "   warehouseName:仓库名," +
            "   transportationNo:运输编号," +
            "   gatewaySN:网关序列号," +
            "   gatewayType:网关类型," +
            "   companyName:公司名。" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return tdlDeviceService.getAll(user.getCompanyNo());
    }

    //根据条件查询TDL信息
    @ApiOperation(value = "根据条件查询TDL信息",notes = "根据条件查询TDL信息，companyNo不用传，传值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号," +
            "   useScene:使用场景（0为空闲，1为仓库，2为运输）。" +
            "返回值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号," +
            "   counts:使用次数," +
            "   lastTime:最后使用时间," +
            "   useScene:使用场景（0为空闲，1为仓库，2为运输）," +
            "   warehouseName:仓库名," +
            "   transportationNo:运输编号," +
            "   gatewaySN:网关序列号," +
            "   gatewayType:网关类型。" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody TDLDeviceTermVo tdlDeviceTermVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        tdlDeviceTermVo.setCompanyNo(user.getCompanyNo());
        return tdlDeviceService.getByTerm(tdlDeviceTermVo);
    }

    //添加TDL信息
    @ApiOperation(value = "添加TDL信息",notes = "添加TDL信息，companyNo不用传，传值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addTDLDevice",method = RequestMethod.POST)
    public Object addTDLDevice(@RequestBody AddTDLDeviceVo addTDLDeviceVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addTDLDeviceVo.getTDLName())){
            fb.setCode(0);
            fb.setMessage("TDL名称不能为空");
            return  gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addTDLDeviceVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage("TDL序列号不能为空");
            return  gson.toJson(fb);
        }
        if (tdlDeviceService.getByTDLSN(addTDLDeviceVo.getTDLSN())!=null){
            fb.setCode(0);
            fb.setMessage("添加的TDL序列号已存在");
            return gson.toJson(fb);
        }
        addTDLDeviceVo.setCompanyNo(user.getCompanyNo());
        int a = tdlDeviceService.addTDLDevice(addTDLDeviceVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加TDL信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加TDL信息失败");
        }
        return gson.toJson(fb);
    }

    //修改TDL信息
    @ApiOperation(value = "修改TDL信息",notes = "修改TDL信息，TDLSN不能改，传值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:需要修改的TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateTDLDevice",method = RequestMethod.POST)
    public Object updateTDLDevice(@RequestBody UpdateTDLDeviceVo updateTDLDeviceVo,HttpServletRequest request){

        if (StringUtils.isEmpty(updateTDLDeviceVo.getTDLName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLName"));
            return  gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTDLDeviceVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLSN"));
            return  gson.toJson(fb);
        }
        int a = tdlDeviceService.updateTDLDevice(updateTDLDeviceVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"updateTDLSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateTDLFailure"));
        }
        return gson.toJson(fb);
    }

    //删除TDL信息
    @ApiOperation(value = "删除TDL信息",notes = "删除TDL信息，传值说明：" +
            "   TDLSN:需要删除的TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delTDLDevice",method = RequestMethod.POST)
    public Object delTDLDevice(@RequestBody TdlSNVo tdlSNVo,HttpServletRequest request){
        if (StringUtils.isEmpty(tdlSNVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLSN"));
            return  gson.toJson(fb);
        }
        Object[] o={tdlSNVo.getTDLSN()};
        if (tdlDeviceService.getByTDLSN(tdlSNVo.getTDLSN()).getUseScene()!=0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,o,"tdlUsed"));
            return gson.toJson(fb);
        }
        int a = tdlDeviceService.delTDLDevice(tdlSNVo.getTDLSN());
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deleteTDLSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deleteTDLFailure"));
        }
        return gson.toJson(fb);

    }



    @ApiOperation(value = "获取未被使用的TDL序列号",notes = "获取未被使用的TDL序列号，传值说明：" +
            "   TDLSN:需要查询的TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/getUnusedTDL",method = RequestMethod.POST)
    public  Object getUnusedTDL(@RequestBody String TDLSN, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(TDLSN)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tdl"));
            return  gson.toJson(fb);
        }
        JSONObject jsonObject=JSON.parseObject(TDLSN);
        TDLSN=(String)jsonObject.get("TDLSN");
        return  tdlDeviceService.getUnusedTDL(TDLSN,user.getCompanyNo());
    }



    //分配TDL
    @ApiOperation(value = "分配TDL",notes = "分配TDL，传值说明：" +
            "   TDLSN:需要分配的网关序列号，" +
            "   companyNo:分配的公司编号。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/updateCompany",method = RequestMethod.POST)
    public Object updateCompany(@RequestBody UpdateTDLCompanyVo updateTDLCompanyVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(updateTDLCompanyVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLSN"));
            return  gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTDLCompanyVo.getCompanyNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"companyNo"));
            return gson.toJson(fb);
        }
        int a=tdlDeviceService.updateCompany(updateTDLCompanyVo.getTDLSN(),updateTDLCompanyVo.getCompanyNo());
        if (a>=0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"updateTDLCompanySuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateTDLCompanyFailure"));
        }
        return gson.toJson(fb);
    }

    //根据TDL序列号修改TDL误差校准值
    @ApiOperation(value = "根据TDL序列号修改TDL误差校准值",notes = "根据TDL序列号修改TDL误差校准值,传值说明：" +
            "    {" +
            "        tdlSN: TDL序列号," +
            "        offset: {" +
            "            tempOffset: 温度校准值," +
            "            humidityOffset: humidityOffset," +
            "        }" +
            "    }"
    )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateTDLOffset",method = RequestMethod.POST)
    public Object updateTDLOffset(@RequestBody UpdateTDLOffsetVo updateTDLOffsetVo, HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        /* 参数合法性判断 */
        if(StringUtils.isBlank(updateTDLOffsetVo.getTdlSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLSN"));
            return gson.toJson(fb);
        }
        if (updateTDLOffsetVo.getOffset() == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongTDLOffset"));
            return gson.toJson(fb);
        }
        /* 查找该TDL是否存在 */
        if (tdlDeviceService.getByTDLSN(updateTDLOffsetVo.getTdlSN()) == null) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tdlExist"));
            return gson.toJson(fb);
        }

        String offset = gson.toJson(updateTDLOffsetVo.getOffset(), TDLDataOffset.class);
        int a = tdlDeviceService.updateTDLOffset(updateTDLOffsetVo.getTdlSN(), offset);
        if (a>=0){
            /* 更新Redis里面的offset信息 */
            ResultTDLDeviceVo resultTDLDeviceVo = tdlDeviceService.getByTDLSN(updateTDLOffsetVo.getTdlSN());
            if (resultTDLDeviceVo.getGatewaySN() != null && resultTDLDeviceVo.getGatewayType() != null) {
                String topic = "TDL/"+resultTDLDeviceVo.getGatewayType()+"/" +resultTDLDeviceVo.getGatewaySN()+"/Data";
                AlarmRule.updateOffset(topic, updateTDLOffsetVo.getTdlSN(),
                        updateTDLOffsetVo.getOffset(), alarmRedisService);
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"updateTDLOffsetSuccess"));
            } else {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"TDLNotBoundGateway"));
            }
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateTDLOffsetFailure"));
        }
        return gson.toJson(fb);
    }
}
