package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddTopicModelVo;
import com.example.tdl.domain.vo.UpdateTopicModelVo;

import com.example.tdl.service.TopicModelService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path = "/topicModel")
public class TopicModelController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private TopicModelService topicModelService;

    @Autowired
    private TokenRedisService tokenRedisService;

    //获取设备的所有主题
    @ApiOperation(value = "查询所有主题模板",notes = "查询所有主题模板的返回值对应的意义:" +
        "      topicModelName: 主题模板名," +
        "      prefixName:添加主题的前缀名，" +
        "      createTime: 模板创建时间," +
        "      updateTime: 模板最新修改时间")
    @RequestMapping(value = "/getAll",method = RequestMethod.GET)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(){
        return topicModelService.getAll();
    }

    @ApiOperation(value = "添加主题模板",notes = "添加主题模板的同时添加主题信息，传递的参数:" +
        "      topicModelName: 新增的主题模板名," +
        "      prefixName:新增的主题名前缀," +
        "      topicName: 主题名（[\"主题1\",\"主题2\"]）")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addTopicModel",method = RequestMethod.POST)
    public String addTopicModel(@RequestBody AddTopicModelVo addTopicModelVo){
        if (StringUtils.isEmpty(addTopicModelVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("添加的主题模板不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addTopicModelVo.getPrefixName())){
            fb.setCode(0);
            fb.setMessage("添加的主题名前缀不能为空");
            return gson.toJson(fb);
        }
        int s=topicModelService.getTopicModelCount();
        if (s>=20){
            fb.setCode(0);
            fb.setMessage("添加的主题数量不能超过20个");
            return gson.toJson(fb);
        }
        if (topicModelService.getByTopicModelName(addTopicModelVo.getTopicModelName())!=null){
            fb.setCode(0);
            fb.setMessage("添加的模板名已存在");
            return gson.toJson(fb);
        }
        String topicNameVar="";
        for (int a=0;a<addTopicModelVo.getTopicName().size();a++){
            topicNameVar=topicNameVar+addTopicModelVo.getTopicName().get(a)+" ";
        }
        Map<String,Object> map=new HashMap<>();
        map.put("topicModelName",addTopicModelVo.getTopicModelName());
        map.put("prefixName",addTopicModelVo.getPrefixName());
        map.put("createTime",System.currentTimeMillis());
        map.put("updateTime",System.currentTimeMillis());
        map.put("topicNameVar",topicNameVar);
        map.put("num",addTopicModelVo.getTopicName().size());
        topicModelService.addTopicModel(map);
        Map<String,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加主题模板成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "修改主题模板",notes = "修改主题模板只能修改主题模板名，updateTime不用传，传递的参数:" +
            "   topicModelName: 修改后的主题模板名," +
            "   prefixName:修改后的前缀名," +
            "   oldTopicModelName: 修改以前的主题模板名")
    @RequestMapping(value = "/updateTopic",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public  String updateTopic(@RequestBody UpdateTopicModelVo updateTopicModelVo){
        if (StringUtils.isEmpty(updateTopicModelVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("修改的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicModelVo.getPrefixName())){
            fb.setCode(0);
            fb.setMessage("修改的主题名前缀不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicModelVo.getOldTopicModelName())){
            fb.setCode(0);
            fb.setMessage("需要修改的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (topicModelService.getByTopicModelName(updateTopicModelVo.getTopicModelName())!=null&&!updateTopicModelVo.getTopicModelName().equals(updateTopicModelVo.getOldTopicModelName())){
            fb.setCode(0);
            fb.setMessage("修改的主题名不能相同");
            return gson.toJson(fb);
        }
        updateTopicModelVo.setUpdateTime(System.currentTimeMillis());
        int a=topicModelService.updateTopicModel(updateTopicModelVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改主题模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改主题模板失败");
        }
        return gson.toJson(fb);
    }

    //删除
    @ApiOperation(value = "删除主题模板",notes = "删除主题模板需要传的参数:" +
        "      topicModelName: 需要删除的主题模板名")
    @RequestMapping(value = "/delTopicModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delTopicModel(@RequestBody String topicModelName){
        JSONObject jsonObject= JSON.parseObject(topicModelName);
        topicModelName=(String)jsonObject.get("topicModelName");
        if (StringUtils.isEmpty(topicModelName)){
            fb.setCode(0);
            fb.setMessage("删除的主题模板名不能为空");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("topicModelName",topicModelName);
        topicModelService.delTopicModel(map);
        Map<String,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除主题模板成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }
}
