package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddUserAddressVo;
import com.example.tdl.domain.vo.UpdateUserAddressVo;
import com.example.tdl.domain.vo.UserRedisVo;
import com.example.tdl.service.UserAddressService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping("/userAddress")
public class UserAddressController {

    private Logger logger = LoggerFactory.getLogger(UserAddressController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private UserAddressService userAddressService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;


    //根据用户编号查询用户常用地址
    @ApiOperation(value = "根据用户编号查询用户常用地址",notes = "根据用户编号查询用户常用地址，返回值说明：" +
            "   userNumber：用户编号," +
            "   name：常用地址名," +
            "   country：国家," +
            "   city：城市," +
            "   addressDetail：具体地址," +
            "   lng：经度," +
            "   lat：纬度," +
            "   postCode：邮编," +
            "   remark：备注。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByUserNumber",method = RequestMethod.GET)
    public Object getByUserNumber(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return userAddressService.getByUserNumber(user.getUserNumber());
    }

    //添加用户常用地址
    @ApiOperation(value = "添加用户常用地址",notes = "添加用户常用地址，需要传的参数：" +
            "   name:地址名," +
            "   country:国家," +
            "   city:城市," +
            "   addressDetail:具体地址," +
            "   lng:经度," +
            "   lat:纬度," +
            "   postCode:邮编," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addUserAddress",method = RequestMethod.POST)
    public Object addUserAddress(@RequestBody AddUserAddressVo addUserAddressVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(addUserAddressVo.getName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"name"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addUserAddressVo.getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"country"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addUserAddressVo.getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"city"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addUserAddressVo.getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addUserAddressVo.getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lat"));
            return gson.toJson(fb);
        }
        if(!addUserAddressVo.getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addUserAddressVo.getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lng"));
            return gson.toJson(fb);
        }
        if(!addUserAddressVo.getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLng"));
            return gson.toJson(fb);
        }
        if (userAddressService.getByUserNumberAndName(user.getUserNumber(),addUserAddressVo.getName())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addUserAddressVo.getPostCode())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"postCode"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("userNumber",user.getUserNumber());
        map.put("name",addUserAddressVo.getName());
        map.put("country",addUserAddressVo.getCountry());
        map.put("city",addUserAddressVo.getCity());
        map.put("addressDetail",addUserAddressVo.getAddressDetail());
        map.put("lng",addUserAddressVo.getLng());
        map.put("lat",addUserAddressVo.getLat());
        map.put("postCode",addUserAddressVo.getPostCode());
        map.put("remark",addUserAddressVo.getRemark());
        userAddressService.addUserAddress(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加用户常用地址成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addAddressSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addAddressFailure"));
        }
        return gson.toJson(fb);
    }

    //修改用户常用地址
    @ApiOperation(value = "修改用户常用地址",notes = "修改用户常用地址，需要传的参数：" +
            "   oldName:修改之前的地址名," +
            "   name:地址名," +
            "   country:国家," +
            "   city:城市," +
            "   addressDetail:具体地址," +
            "   lng:经度," +
            "   lat:纬度," +
            "   postCode:邮编," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateUserAddress",method = RequestMethod.POST)
    public Object updateUserAddress(@RequestBody UpdateUserAddressVo updateUserAddressVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(updateUserAddressVo.getOldName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateUserAddressVo.getName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedNewName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateUserAddressVo.getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"country"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateUserAddressVo.getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"city"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateUserAddressVo.getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateUserAddressVo.getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lat"));
            return gson.toJson(fb);
        }
        if(!updateUserAddressVo.getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateUserAddressVo.getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lng"));
            return gson.toJson(fb);
        }
        if(!updateUserAddressVo.getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLng"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateUserAddressVo.getPostCode())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"postCode"));
            return gson.toJson(fb);
        }
        if (!updateUserAddressVo.getOldName().equals(updateUserAddressVo.getName()) && userAddressService.getByUserNumberAndName(user.getUserNumber(),updateUserAddressVo.getName())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedRepeatedName"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("userNumber",user.getUserNumber());
        map.put("oldName", updateUserAddressVo.getOldName());
        map.put("name", updateUserAddressVo.getName());
        map.put("country", updateUserAddressVo.getCountry());
        map.put("city", updateUserAddressVo.getCity());
        map.put("addressDetail", updateUserAddressVo.getAddressDetail());
        map.put("lng", updateUserAddressVo.getLng());
        map.put("lat", updateUserAddressVo.getLat());
        map.put("postCode", updateUserAddressVo.getPostCode());
        map.put("remark", updateUserAddressVo.getRemark());
        userAddressService.updateUserAddress(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("修改用户常用地址成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"modifiedAddressSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedAddressFailure"));
        }
        return gson.toJson(fb);
    }

    //删除用户常用地址
    @ApiOperation(value = "删除用户常用地址",notes = "删除用户常用地址,只用传需要删除的地址名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/delUserAddress",method = RequestMethod.POST)
    public Object delUserAddress(@RequestBody String name, HttpServletRequest request){
        JSONObject jsonObject= JSON.parseObject(name);
        name=(String)jsonObject.get("name");
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(name)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedName"));
            return gson.toJson(fb);
        }
        int a = userAddressService.delUserAddress(user.getUserNumber(),name);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deletedAddressSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedAddressFailure"));
        }
        return gson.toJson(fb);
    }

}
