package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.CompanyService;
import com.example.tdl.service.RoleHasPermissionService;
import com.example.tdl.service.RoleService;

import com.example.tdl.service.UserService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.service.redis.UserTokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.w3c.dom.NodeList;

import javax.servlet.http.HttpServletRequest;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.Node;
import java.net.URL;
import java.util.List;


@RestController
@RequestMapping("/user")
public class UserController {

    private Logger logger = LoggerFactory.getLogger(UserController.class);

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private UserService userService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private UserTokenRedisService userTokenService;


    //获取所有用户信息
    @ApiOperation(value = "获取所有用户信息",notes = "获取所有用户信息：" +
            "   userNumber:用户编号," +
            "   userName:用户名," +
            "   nickName:昵称," +
            "   phone:电话," +
            "   email:邮箱," +
            "   roleName:角色名," +
            "   companyName:公司名.")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return userService.getAll(user.getCompanyNo());
    }

    //根据条件查询
    @ApiOperation(value = "根据条件查询",notes = "根据条件查询：需要传的值：" +
            "   email:邮件," +
            "   phone:电话," +
            "   roleName:角色名," +
            "   userName:用户名." +
            "返回值说明：" +
            "   userNumber:用户编号," +
            "   userName:用户名," +
            "   nickName:昵称," +
            "   phone:电话," +
            "   email:邮箱," +
            "   roleName:角色名。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody UserTermVo userTermVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(userTermVo.getUserName())){
            userTermVo.setUserName(null);
        }
        if(StringUtils.isEmpty(userTermVo.getPhone())){
            userTermVo.setPhone(null);
        }
        if(StringUtils.isEmpty(userTermVo.getEmail())){
            userTermVo.setEmail(null);
        }
        if(StringUtils.isEmpty(userTermVo.getRoleName())){
            userTermVo.setRoleName(null);
        }
        userTermVo.setCompanyNo(user.getCompanyNo());
        return userService.getByTerm(userTermVo);
    }

    //新增用户
    @ApiOperation(value = "新增用户",notes = "新增用户时，userNumber,createTime,updateTime不用传。传递参数:" +
            "   userName：用户名，" +
            "   password：密码，" +
            "   nickName：昵称，" +
            "   phone：电话，" +
            "   email：邮件，" +
            "   roleName：角色名，" +
            "   companyName：用户所属公司")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addUser",method = RequestMethod.POST)
    public Object addUser(@RequestBody AddUserVo userVo,HttpServletRequest request){
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(userVo.getUserName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userName"));
            return gson.toJson(fb);
        }
        if (!userVo.getUserName().matches("^[a-zA-Z][a-zA-Z0-9_]{5,19}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"usernameRule"));
            return gson.toJson(fb);
        }
        if(userService.getByUserName(userVo.getUserName()) !=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedUserName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPassword())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"password"));
            return gson.toJson(fb);
        }
        if(!userVo.getPassword().matches("^[a-zA-Z\\d_]{6,20}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"passwordRule"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getNickName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"nickName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"phone"));
            return gson.toJson(fb);
        }
        if( !userVo.getPhone().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"phoneRule"));
            return gson.toJson(fb);
        }
        if ((userService.getByPhone(userVo.getPhone(),userVo.getCompanyName()))!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedRule"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"email"));
            return gson.toJson(fb);
        }
        if( !userVo.getEmail().matches("^\\w[-\\w.+]*@([A-Za-z0-9][-A-Za-z0-9]+\\.)+[A-Za-z]{2,14}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"emailRule"));
            return gson.toJson(fb);
        }
        if ((userService.getByEmail(userVo.getEmail(),userVo.getCompanyName()))!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedEmail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        if (roleService.getByRoleName(userVo.getRoleName(), userVo.getCompanyName())==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existRoleName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getCompanyName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"companyName"));
            return gson.toJson(fb);
        }
        if (companyService.getByCompanyName(userVo.getCompanyName())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existCompany"));
            return gson.toJson(fb);
        }
        String userNumber="U"+System.currentTimeMillis();
        userVo.setUserNumber(userNumber);
        userVo.setCreateTime(System.currentTimeMillis());
        userVo.setUpdateTime(System.currentTimeMillis());
        userVo.setPassword(DigestUtils.sha256Hex(userVo.getPassword()));
        int a=userService.addUser(userVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addUserSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addUserFailure"));
        }
        return gson.toJson(fb);
    }


    //修改用户
    @ApiOperation(value = "修改用户",notes = "修改用户,companyName,updateTime不用传，需要传递的参数：" +
            "   userNumber：用户编号，" +
            "   userName:用户名," +
            "   nickName：昵称，" +
            "   phone：电话，" +
            "   email：邮件，" +
            "   roleName：角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateUser",method = RequestMethod.POST)
    public Object updateUser(@RequestBody UpdateUserVo userVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(userVo.getUserNumber())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userNumber"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getNickName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"nickName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"phone"));
            return gson.toJson(fb);
        }
        if(!userVo.getPhone().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"phoneRule"));
            return gson.toJson(fb);
        }
        ResultUserVo userInfo=userService.getByUserNumber(userVo.getUserNumber());
        if (userInfo!=null&& !userInfo.getPhone().equals(userVo.getPhone()) && userService.getByPhone(userVo.getPhone(),userInfo.getCompanyNo()) !=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedRule"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"email"));
            return gson.toJson(fb);
        }
        if( !userVo.getEmail().matches("^\\w[-\\w.+]*@([A-Za-z0-9][-A-Za-z0-9]+\\.)+[A-Za-z]{2,14}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"emailRule"));
            return gson.toJson(fb);
        }
        if (userInfo!=null && !userInfo.getEmail().equals(userVo.getEmail()) && userService.getByEmail(userVo.getEmail(),userInfo.getCompanyNo()) !=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedEmail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        if (roleService.getByRoleName(userVo.getRoleName(),userInfo.getCompanyName())==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existRoleName"));
            return gson.toJson(fb);
        }
        userVo.setCompanyName(userInfo.getCompanyName());
        userVo.setUpdateTime(String.valueOf(System.currentTimeMillis()));
        int a=userService.updateUser(userVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"modifiedUserSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedUserFailure"));
        }
        return gson.toJson(fb);
    }

    //删除用户
    @ApiOperation(value = "删除用户",notes = "删除用户")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),

    })
    @RequestMapping(value="/delUser",method = RequestMethod.POST)
    public Object delUser(@RequestBody String userNumber,HttpServletRequest request){
        String port = request.getHeader("AccountLanguage");
        JSONObject jsonObject= JSON.parseObject(userNumber);
        userNumber=jsonObject.getString("userNumber");
        if (StringUtils.isEmpty(userNumber)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userNumber"));
            return gson.toJson(fb);
        }
        int a = userService.delUser(userNumber);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deleteUserSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deleteUserFailure"));
        }
        return gson.toJson(fb);
    }

    //重置密码
    @ApiOperation(value = "重置密码",notes = "重置密码")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/resetPassword",method = RequestMethod.POST)
    public Object resetPassword(@RequestBody String userNumber,HttpServletRequest request){
        String port = request.getHeader("AccountLanguage");
        JSONObject jsonObject= JSON.parseObject(userNumber);
        userNumber=jsonObject.getString("userNumber");
        if (StringUtils.isEmpty(userNumber)) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userNumber"));
            return gson.toJson(fb);
        }
        String pwd=DigestUtils.sha256Hex("Witium123");
        int a = userService.updatePassword(new UpdatePasswordVo(userNumber,pwd));
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"resetPasswordSuccess"));
            String key =  userTokenService.getHashKeys(userNumber).toString();
            List<String> keys =  gson.fromJson(key,List.class);
            for( int  i = 0;i<keys.size();i++){
                tokenRedisService.delKey(keys.get(i));
            }
            userTokenService.delKey(userNumber);
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"resetPasswordFailure"));
        }
        return gson.toJson(fb);
    }


    //修改密码
    @ApiOperation(value = "修改密码",notes = "修改密码")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updatePassword",method = RequestMethod.POST)
    public Object updatePassword(@RequestBody UpdatePasswordVo updatePasswordVo,HttpServletRequest request){
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(updatePasswordVo.getUserNumber())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userNumber"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updatePasswordVo.getPassword())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"password"));
            return gson.toJson(fb);
        }
        String pwd=DigestUtils.sha256Hex(updatePasswordVo.getPassword());
        updatePasswordVo.setPassword(pwd);
        int a = userService.updatePassword(updatePasswordVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"modifiedPasswordSuccess"));
            String key =  userTokenService.getHashKeys(updatePasswordVo.getUserNumber()).toString();
            List<String> keys =  gson.fromJson(key,List.class);
            for( int  i = 0;i<keys.size();i++){
                tokenRedisService.delKey(keys.get(i));
            }
            userTokenService.delKey(updatePasswordVo.getUserNumber());
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedPasswordFailure"));
        }
        return gson.toJson(fb);
    }


    //获取当前的用户名
    @ApiOperation(value = "获取当前用户的信息",notes = "获取用户的信息" +
            "userName:用户名" +
            "time：时间" +
            "ipAddres:ip" +
            "companyAlias:公司简称" +
            "themeColors：主题颜色")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @GetMapping(value="/getUserName")
    public Object getUserName(HttpServletRequest request) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        String ip = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest().getRemoteAddr();
        ResultCompanyVo resultCompanyVo=companyService.getByCompanyNo(userRedisVo.getCompanyNo());
        String img=resultCompanyVo.getImg();
        String companyAlias=resultCompanyVo.getAlias();
//        if(resultCompanyVo.getCompanyNo().equals("TM201808097604")){
//            return new UserNameVo(userRedisVo.getUserName(),System.currentTimeMillis() +"",ip,companyAlias,img,resultCompanyVo.getThemeColors(),resultCompanyVo.getCompanyNo(),resultCompanyVo.getFontColors());
//        }else{
            return new UserNameVo(userRedisVo.getUserName(),System.currentTimeMillis() +"",ip,img,companyAlias,resultCompanyVo.getThemeColors(),resultCompanyVo.getCompanyNo(),resultCompanyVo.getFontColors());
//        }
    }



    //获取本公司的所有用户名
    @ApiOperation(value = "获取本公司的所有用户名",notes = "获取用户的信息" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @GetMapping(value="/getAllUserName")
    public Object getAllUserName(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        return userService.getAllByCompanyNo(user.getCompanyNo());
    }

}
